@@ -18,10 +18,11 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/calma/CalmaRdpt.c,v 1.5 2001/07/23 17:49:49 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/calma/CalmaRdpt.c,v 1.2 2006/07/25 19:10:29 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <stdlib.h>                 /* for abs() */
 #include <sys/types.h>
 
 #include <netinet/in.h>
@@ -228,6 +229,8 @@ calmaElementBoundary()
 	plane = NULL;
 	calmaLayerError("Unknown layer/datatype in boundary", layer, dt);
     }
+    else
+	plane = cifCurReadPlanes[ciftype];
 
     /* Read the path itself, building up a path structure */
     if (!calmaReadPath(&pathheadp, (plane == NULL) ? 0 : 1)) 
@@ -238,8 +241,8 @@ calmaElementBoundary()
     }
 
     /* Note that calmaReadPath() may reallocate planes of cifCurReadPlanes */
-
-    plane = cifCurReadPlanes[ciftype];
+    /* so we need to set it again.					   */
+    if (ciftype >= 0) plane = cifCurReadPlanes[ciftype];
 
     /* Convert the polygon to rectangles. */
 
@@ -442,6 +445,10 @@ calmaElementPath()
     Rect segment;
     Plane *plane;
     int first,last;
+#ifdef NONMANHATTAN
+    CellUse *use;
+    CellDef *savedef, *newdef = NULL;
+#endif
 
     /* Skip CALMA_ELFLAGS, CALMA_PLEX */
     calmaSkipSet(calmaElementIgnore);
@@ -509,9 +516,60 @@ calmaElementPath()
     else
     {
 	plane = cifCurReadPlanes[ciftype];
+
+#ifdef NONMANHATTAN
+	if (CalmaSubcellPolygons && (calmaNonManhattan > 0))
+	{
+	    /* Place the polygon in its own subcell */
+	    char newname[] = "polygonXXXXX";
+	    HashEntry *he;
+
+	    savedef = cifReadCellDef;
+
+	    /* Make up name for cell */
+	    sprintf(newname + 7, "%05d", ++CalmaPolygonCount);
+
+	    he = HashFind(&calmaDefInitHash, newname);
+	    if (!HashGetValue(he))
+	    {
+		newdef = calmaFindCell(newname, NULL);
+		cifReadCellDef = newdef;
+		DBCellClearDef(cifReadCellDef);
+		DBCellSetAvail(cifReadCellDef);
+
+		/* cifEditCellPlanes is not used by the gds reader, so it's */
+		/* available to be used to store the polygon.		*/
+
+		cifCurReadPlanes = cifEditCellPlanes;
+		if (plane != NULL)
+		    plane = cifCurReadPlanes[ciftype];
+	    }
+	}
+#endif /* NONMANHATTAN */
+
 	CIFPaintWirePath(pathheadp, width,
 		(pathtype == CALMAPATH_SQUAREFLUSH) ? FALSE : TRUE,
 		plane, CIFPaintTable, (PaintUndoInfo *)NULL);
+
+#ifdef NONMANHATTAN
+	if (cifCurReadPlanes == cifEditCellPlanes)
+	{
+	    CIFPaintCurrent();
+	    DBReComputeBbox(cifReadCellDef);
+	    DRCCheckThis(cifReadCellDef, TT_CHECKPAINT, &cifReadCellDef->cd_bbox);
+	    DBWAreaChanged(cifReadCellDef, &cifReadCellDef->cd_bbox,
+			DBW_ALLWINDOWS, &DBAllButSpaceBits);
+	    DBCellSetModified(cifReadCellDef, TRUE);
+	    DBGenerateUniqueIds(cifReadCellDef, FALSE);  /* Is this necessary? */
+
+	    cifCurReadPlanes = cifSubcellPlanes;
+	    cifReadCellDef = savedef;
+
+	    use = DBCellNewUse(newdef, (char *)NULL);
+	    DBSetTrans(use, &GeoIdentityTransform);
+	    DBPlaceCell(use, cifReadCellDef);
+	}
+#endif /* NONMANHATTAN */
     }
 }
 
@@ -697,7 +755,7 @@ calmaReadPath(pathheadpp, iscale)
 		phead = phead->cifp_next;
 	    }
 	}
-	if (ABS(path.cifp_x) > 50000000 || ABS(path.cifp_y) > 50000000) {
+	if (ABS(path.cifp_x) > 0x0fffffff || ABS(path.cifp_y) > 0x0fffffff) {
 	    calmaReadError("Warning:  Very large point in path:  (%d, %d)\n",
 		path.cifp_x, path.cifp_y);
 	}
