@@ -17,10 +17,11 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/calma/CalmaRead.c,v 1.5 2001/07/23 17:49:49 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/calma/CalmaRead.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
 
@@ -133,7 +134,7 @@ CalmaReadFile(file, filename)
     FILE *file;			/* File from which to read Calma */
     char *filename;		/* The real name of the file read */
 {
-    int version;
+    int k, version;
     char *libname = NULL;
     MagWindow *mw;
     static int hdrSkip[] = { CALMA_FORMAT, CALMA_MASK, CALMA_ENDMASKS,
@@ -183,7 +184,15 @@ CalmaReadFile(file, filename)
     calmaSkipSet(skipBeforeLib);
     if (!calmaReadStringRecord(CALMA_LIBNAME, &libname)) goto done;
     if ((libname != NULL) && (libname[0] != '\0'))
+    {
+	/* Avoid generating a magic name with spaces in it. . .	*/
+	/* (added by Mike Godfrey, 7/17/05)			*/
+
+	for (k = 0; k < strlen(libname); k++)
+	    if (libname[k] == ' ')
+		libname[k] = '_';
 	TxPrintf("Library name: %s\n", libname);
+    }
 
     /* Skip the reflibs, fonts, etc. cruft */
     calmaSkipSet(hdrSkip);
@@ -476,6 +485,7 @@ calmaReadError(format, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10)
  * ----------------------------------------------------------------------------
  */
 
+void
 CalmaTechInit()
 {
     ASSERT(sizeof(FourByteInt)==4, "definition in calmaInt.h");
