@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/calma/CalmaWrite.c,v 1.4 2001/07/23 17:49:49 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-7.5/calma/CalmaWrite.c,v 1.3 2006/09/13 21:02:54 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -47,6 +47,7 @@ static char rcsid[]="$Header: /ufs/repository/magic/calma/CalmaWrite.c,v 1.4 200
 #include "utils/styles.h"
 #include "textio/textio.h"
 #include "calma/calmaInt.h"
+#include "utils/main.h"		/* for Path and CellLibPath */
 
     /* Exports */
 bool CalmaDoLabels = TRUE;	 /* If FALSE, don't output labels with GDS-II */
@@ -154,18 +155,18 @@ static char calmaMapTableStrict[] =
       0,    0,    0,    0,    0,    0,    0,    0,	/* BS  - SI  */
       0,    0,    0,    0,    0,    0,    0,    0,	/* DLE - ETB */
       0,    0,    0,    0,    0,    0,    0,    0,	/* CAN - US  */
-      0,    0,    0,    0,  '$',    0,    0,    0,	/* SP  - '   */
-      0,    0,    0,    0,    0,    0,    0,    0,	/* (   - /   */
+    '_',  '_',  '_',  '_',  '$',  '_',  '_',  '_',	/* SP  - '   */
+    '_',  '_',  '_',  '_',  '_',  '_',  '_',  '_',	/* (   - /   */
     '0',  '1',  '2',  '3',  '4',  '5',  '6',  '7',	/* 0   - 7   */
-    '8',  '9',    0,    0,    0,    0,    0,    0,	/* 8   - ?   */
-      0,  'A',  'B',  'C',  'D',  'E',  'F',  'G',	/* @   - G   */
+    '8',  '9',  '_',  '_',  '_',  '_',  '_',  '_',	/* 8   - ?   */
+    '_',  'A',  'B',  'C',  'D',  'E',  'F',  'G',	/* @   - G   */
     'H',  'I',  'J',  'K',  'L',  'M',  'N',  'O',	/* H   - O   */
     'P',  'Q',  'R',  'S',  'T',  'U',  'V',  'W',	/* P   - W   */
-    'X',  'Y',  'Z',    0,    0,    0,    0,  '_',	/* X   - _   */
-      0,  'a',  'b',  'c',  'd',  'e',  'f',  'g',	/* `   - g   */
+    'X',  'Y',  'Z',  '_',  '_',  '_',  '_',  '_',	/* X   - _   */
+    '_',  'a',  'b',  'c',  'd',  'e',  'f',  'g',	/* `   - g   */
     'h',  'i',  'j',  'k',  'l',  'm',  'n',  'o',	/* h   - o   */
     'p',  'q',  'r',  's',  't',  'u',  'v',  'w',	/* p   - w   */
-    'x',  'y',  'z',    0,    0,    0,    0,    0,	/* x   - DEL */
+    'x',  'y',  'z',  '_',  '_',  '_',  '_',  0,	/* x   - DEL */
 };
 
 static char calmaMapTablePermissive[] =
@@ -174,7 +175,7 @@ static char calmaMapTablePermissive[] =
       0,    0,    0,    0,    0,    0,    0,    0,	/* BS  - SI  */
       0,    0,    0,    0,    0,    0,    0,    0,	/* DLE - ETB */
       0,    0,    0,    0,    0,    0,    0,    0,	/* CAN - US  */
-      0,  '!',  '"',  '#',  '$',  '&',  '%', '\'',	/* SP  - '   */
+    '_',  '!',  '"',  '#',  '$',  '&',  '%', '\'',	/* SP  - '   */
     '(',  ')',  '*',  '+',  ',',  '-',  '.',  '/',	/* (   - /   */
     '0',  '1',  '2',  '3',  '4',  '5',  '6',  '7',	/* 0   - 7   */
     '8',  '9',  ':',  ';',  '<',  '=',  '>',  '?',	/* 8   - ?   */
@@ -356,7 +357,7 @@ calmaProcessDef(def, outf)
     FILE *outf;		/* Stream file */
 {
     char *filename;
-    bool isReadOnly;
+    bool isReadOnly, oldStyle, hasContent;
 
     /* Skip if already output */
     if ((int) def->cd_client > 0)
@@ -387,17 +388,28 @@ calmaProcessDef(def, outf)
      * not, or if there is any problem obtaining the original cell
      * definition, resort to writing out magic's version of the def,
      * and print a warning message.
+     * 
+     * Treat the lack of a GDS_START property as an indication
+     * that we should treat this cell like a reference-only
+     * cell.  That is, the instance will be called but no
+     * definition will appear in the output.
      */
 
+    DBPropGet(def, "GDS_START", &hasContent);
     filename = (char *)DBPropGet(def, "GDS_FILE", &isReadOnly);
-    if (isReadOnly)
+
+    if (isReadOnly && hasContent)
     {
 	char *buffer, *offptr;
 	size_t defsize, numbytes;
 	off_t cellstart, cellend;
 	FILE *fi;
 
-	fi = fopen(filename, "r");
+
+	/* Use PaOpen() so the paths searched are the same as were	*/
+	/* searched to find the .mag file that indicated this GDS file.	*/
+
+	fi = PaOpen(filename, "r", "", Path, CellLibPath, (char **)NULL);
 	if (fi == NULL)
 	{
 	    TxError("Calma output error:  Can't find GDS file for vendor cell."
@@ -406,37 +418,67 @@ calmaProcessDef(def, outf)
 	}
 	else
 	{
-	    offptr = (char *)DBPropGet(def, "GDS_BEGIN", NULL);
+	    sscanf(offptr, "%d", &cellend);
+	    offptr = (char *)DBPropGet(def, "GDS_BEGIN", &oldStyle);
+	    if (!oldStyle)
+	    {
+		offptr = (char *)DBPropGet(def, "GDS_START", NULL);
+
+		/* Write our own header and string name, to ensure	*/
+		/* that the magic cell name and GDS name match.		*/
+
+		/* Output structure header */
+		calmaOutRH(28, CALMA_BGNSTR, CALMA_I2, outf);
+		calmaOutDate(def->cd_timestamp, outf);
+		calmaOutDate(time((time_t *) 0), outf);
+
+		/* Name structure the same as the magic cellname */
+		calmaOutStructName(CALMA_STRNAME, def, outf);
+	    }
+
 	    sscanf(offptr, "%d", &cellstart);
 	    fseek(fi, cellstart, SEEK_SET);
-	    offptr = (char *)DBPropGet(def, "GDS_END", NULL);
-	    sscanf(offptr, "%d", &cellend);
-	    defsize = (size_t)(cellend - cellstart);
 
-	    buffer = (char *)mallocMagic(defsize);
-	    numbytes = fread(buffer, sizeof(char), (size_t)defsize, fi);
-	    if (numbytes == defsize)
+	    if (cellend < cellstart)	/* Sanity check */
 	    {
-		numbytes = fwrite(buffer, sizeof(char), (size_t)defsize, outf);
-		if (numbytes <= 0)
-		{
-		    TxError("Calma output error:  Can't write cell from vendor GDS."
-			"  Using magic's internal definition\n");
-		    isReadOnly = FALSE;
-		}
+		TxError("Calma output error:  Bad vendor GDS file reference!");
+		isReadOnly = FALSE;
 	    }
 	    else
 	    {
-		TxError("Calma output error:  Can't read cell from vendor GDS."
-			"  Using magic's internal definition\n");
-		isReadOnly = FALSE;
+		defsize = (size_t)(cellend - cellstart);
+
+		buffer = (char *)mallocMagic(defsize);
+		numbytes = fread(buffer, sizeof(char), (size_t)defsize, fi);
+		if (numbytes == defsize)
+		{
+		    numbytes = fwrite(buffer, sizeof(char), (size_t)defsize, outf);
+		    if (numbytes <= 0)
+		    {
+			TxError("Calma output error:  Can't write cell from vendor GDS."
+				"  Using magic's internal definition\n");
+			isReadOnly = FALSE;
+		    }
+		}
+		else
+		{
+		    TxError("Calma output error:  Can't read cell from vendor GDS."
+				"  Using magic's internal definition\n");
+		    isReadOnly = FALSE;
+		}
+		freeMagic(buffer);
 	    }
-	    freeMagic(buffer);
 	    fclose(fi);
+
+	    /* Mark the definition as vendor GDS so that magic doesn't	*/
+	    /* try to generate subcell interaction or array interaction	*/
+	    /* paint for it.						*/
+
+	    def->cd_flags |= CDVENDORGDS;
 	}
     }
 
-    /* Output this cell */
+    /* Output this cell definition from the Magic database */
     if (!isReadOnly)
 	calmaOutFunc(def, outf, &TiPlaneRect);
 
@@ -474,6 +516,7 @@ calmaOutFunc(def, f, cliprect)
     CIFLayer *layer;
     Rect bigArea;
     int type;
+    int dbunits;
     calmaOutputStruct cos;
 
     cos.f = f;
@@ -487,21 +530,32 @@ calmaOutFunc(def, f, cliprect)
     /* Output structure name */
     calmaOutStructName(CALMA_STRNAME, def, f);
 
-    /* Since Calma database units are millimicrons, multiply all units by 10,
+    /* Since Calma database units are nanometers, multiply all units by 10,
      * modified by the scale multiplier.
      */
 
-    if ((10 % CIFCurStyle->cs_expander) == 0)
+    dbunits = (CIFCurStyle->cs_flags & CWF_ANGSTROMS) ? 100 : 10;
+    if ((dbunits % CIFCurStyle->cs_expander) == 0)
     {
-	calmaWriteScale = CIFCurStyle->cs_scaleFactor * 10 / CIFCurStyle->cs_expander;
-	calmaPaintScale = 10 / CIFCurStyle->cs_expander;
+	calmaWriteScale = CIFCurStyle->cs_scaleFactor * dbunits
+		/ CIFCurStyle->cs_expander;
+	calmaPaintScale = dbunits / CIFCurStyle->cs_expander;
     }
     else
     {
 	TxError("Calma output error:  Output scale units are %2.1f nanometers.\n",
-		(float)10 / (float)CIFCurStyle->cs_expander);
-	TxError("Magic Calma output supports values in integer nanometers or larger.\n");
+		(float)dbunits / (float)CIFCurStyle->cs_expander);
 	TxError("Magic Calma output will be scaled incorrectly!\n");
+	if ((dbunits == 10) && ((100 % CIFCurStyle->cs_expander) == 0))
+	{
+	    TxError("Please add \"units angstroms\" to the cifoutput section"
+			" of the techfile.\n");
+	}
+	else
+	{
+	    TxError("Magic GDS output is limited to a minimum dimension of"
+			" 1 angstrom.\n");
+	}
 	/* Set expander to 10 so output scales are not zero. */
 	calmaWriteScale = CIFCurStyle->cs_scaleFactor;
 	calmaPaintScale = 1;
@@ -518,8 +572,10 @@ calmaOutFunc(def, f, cliprect)
     GEO_EXPAND(&def->cd_bbox, CIFCurStyle->cs_radius, &bigArea);
     CIFErrorDef = def;
     CIFGen(def, &bigArea, CIFPlanes, &DBAllTypeBits, TRUE, TRUE, (ClientData) f);
-    CIFGenSubcells(def, &bigArea, CIFPlanes);
-    CIFGenArrays(def, &bigArea, CIFPlanes);
+    if (!CIFHierWriteDisable)
+	CIFGenSubcells(def, &bigArea, CIFPlanes);
+    if (!CIFArrayWriteDisable)
+	CIFGenArrays(def, &bigArea, CIFPlanes);
 
     for (type = 0; type < CIFCurStyle->cs_nLayers; type++)
     {
@@ -834,7 +890,7 @@ calmaOutStructName(type, def, f)
     int calmanum;
     char *table;
 
-    if(CIFCurStyle->cs_flags & CWF_PERMISSIVE_LABELS) 
+    if (CIFCurStyle->cs_flags & CWF_PERMISSIVE_LABELS) 
     {
 	table = calmaMapTablePermissive;
     } else {
@@ -843,8 +899,16 @@ calmaOutStructName(type, def, f)
 
     /* Is the def name a legal Calma name? */
     for (cp = def->cd_name; c = (unsigned char) *cp; cp++)
-	if (c > 127 || table[c] == 0)
+    {
+	if ((c > 127) || (table[c] == 0))
 	    goto bad;
+	else if ((unsigned char)table[c] != c)
+	{
+	    TxError("Warning: character \'%c\' changed to \'%c\' in"
+		" name %s\n", (char)c, table[c], def->cd_name);
+	}
+	/* We really should ensure that the new name is unique. . . */
+    }
     if (cp <= def->cd_name + CALMANAMELENGTH)
     {
 	/* Yes, it's legal: use it */
@@ -857,6 +921,8 @@ bad:
 	calmanum = (int) def->cd_client;
 	if (calmanum < 0) calmanum = -calmanum;
 	(void) sprintf(defname, "XXXXX%d", calmanum);
+	TxError("Warning: string in output unprintable; changed to \'%s\'\n",
+		 defname);
     }
 
     calmaOutStringRecord(type, defname, f);
@@ -1046,7 +1112,7 @@ calmaWriteContacts(f)
 	    /* units, and the clipping rectangle passed to	*/
 	    /* calmaOutFunc is also in CIF/GDS units.		*/
 
-	    halfsize = CIFGetContactSize(type) >> 1;
+	    halfsize = CIFGetContactSize(type, NULL, NULL, NULL) >> 1;
 
             /* Get minimum width for layer by rounding halfsize	*/
 	    /* to the nearest lambda value.			*/
@@ -1323,7 +1389,10 @@ calmaOutHeader(rootDef, f)
      */
 
     calmaOutRH(20, CALMA_UNITS, CALMA_R8, f);
+    if (CIFCurStyle->cs_flags & CWF_ANGSTROMS) useru = 0.0001;
     calmaOutR8(useru, f);	/* User units per database unit */
+
+    if (CIFCurStyle->cs_flags & CWF_ANGSTROMS) mum = 1e-10;
     calmaOutR8(mum, f);		/* Meters per database unit */
 }
 
@@ -1388,7 +1457,7 @@ calmaOutStringRecord(type, str, f)
 {
     int len;
     unsigned char c;
-    char *table;
+    char *table, *locstr, *origstr = NULL;
 
     if(CIFCurStyle->cs_flags & CWF_PERMISSIVE_LABELS) 
     {
@@ -1398,6 +1467,7 @@ calmaOutStringRecord(type, str, f)
     }
 
     len = strlen(str);
+    locstr = str;
 
     /*
      * Make sure length is even.
@@ -1412,18 +1482,35 @@ calmaOutStringRecord(type, str, f)
     /* Output the string itself */
     while (len--)
     {
-	c = (unsigned char) *str++;
+	c = (unsigned char) *locstr++;
 	if (c == 0) putc('\0', f);
 	else
 	{
-	    if (c > 127 || table[c] == 0) c = 'X';
-	    else c = table[c];
+	    if ((c > 127) || (c == 0))
+	    {
+		TxError("Warning: Unprintable character changed "
+			"to \'X\' in label.\n");
+		c = 'X';
+	    }
+	    else
+	    {
+		if (((unsigned char)table[c] != c) && (origstr == NULL))
+		    origstr = StrDup(NULL, locstr);
+
+		c = table[c];
+	    }
 	    if (!CalmaDoLower && islower(c))
 		(void) putc(toupper(c), f);
 	    else
 		(void) putc(c, f);
 	}
     }
+    if (origstr != NULL)
+    {
+	TxError("Warning: characters changed in string \'%s\'; "
+		"modified string is \'%s\'\n", origstr, locstr);
+	freeMagic(origstr);
+    }
 }
 
 /*
