@@ -18,15 +18,12 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/cif/CIFrdpt.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFrdpt.c,v 1.4 2006/10/04 17:30:17 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
-
-#ifdef NONMANHATTAN
 #include <math.h>		/* for wire path-to-poly path conversion */
-#endif
 
 #include "utils/magic.h"
 #include "utils/geometry.h"
@@ -260,7 +257,7 @@ CIFPaintWirePath(pathheadp, width, endcap, plane, ptable, ui)
 {
     CIFPath *pathp, *previousp, *nextp, *polypath, *newpath;
     LinkedRect	*rectp;
-    double theta, phi, alpha, delta, cwidth, dtmp;
+    double theta, phi, alpha, delta, cwidth, testmitre;
     double xmaxoff, ymaxoff, xminoff, yminoff;
     double xmin, ymin, xmax, ymax, xnext, ynext;
     bool firstpoint;
@@ -326,8 +323,7 @@ CIFPaintWirePath(pathheadp, width, endcap, plane, ptable, ui)
 	}
 
 	alpha = 0.5 * (phi - theta);
-	delta = (0.5 * (phi + theta)) - 1.5708;
-	cwidth = (double)width / (cos(alpha));
+	testmitre = fabs(cos(alpha));
 
 	/* This routine does not (yet) do mitre limits, so for	*/
 	/* now, we do a sanity check.  In the case of an	*/
@@ -338,13 +334,28 @@ CIFPaintWirePath(pathheadp, width, endcap, plane, ptable, ui)
 	/* than this tend to cause difficulties for the		*/
 	/* CIFMakeManhattanPath() routine.			*/
 
-	if (fabs(cos(alpha)) < 0.25) 
-	{
-	    TxError("Error: mitre limit exceeded at wire junction.\n");
-	    TxError("Route has been truncated.\n");
-	    break;
+	if (testmitre < 0.25) {
+	    if (testmitre < 1.0e-10) {
+		/* Wire reverses direction.  Treat midpoint as an endpoint */
+		phi = theta;
+		if (endcap)
+		{
+		    xmax += (double)width * cos(theta);
+		    ymax += (double)width * sin(theta);
+		}
+		alpha = 0.5 * (phi - theta);
+		firstpoint = TRUE;
+	    }
+	    else {
+		TxError("Error: mitre limit exceeded at wire junction.\n");
+		TxError("Route has been truncated.\n");
+		break;
+	   }
 	}
 
+	delta = (0.5 * (phi + theta)) - 1.5708;
+	cwidth = (double)width / (cos(alpha));
+
 	xmaxoff = cwidth * cos(delta);
 	ymaxoff = cwidth * sin(delta);
 
@@ -355,42 +366,26 @@ CIFPaintWirePath(pathheadp, width, endcap, plane, ptable, ui)
 	newpath = (CIFPath *)mallocMagic(sizeof(CIFPath));
 	newpath->cifp_next = polypath;
 	polypath = newpath;
-	dtmp = (xmax + xmaxoff) / 2;
-	dtmp += (dtmp < 0) ? -0.5 : 0.5;
-	newpath->cifp_x = (int)dtmp;
-	dtmp = (ymax + ymaxoff) / 2;
-	dtmp += (dtmp < 0) ? -0.5 : 0.5;
-	newpath->cifp_y = (int)dtmp;
+	newpath->cifp_x = round((xmax + xmaxoff) / 2);
+	newpath->cifp_y = round((ymax + ymaxoff) / 2);
 
 	newpath = (CIFPath *)mallocMagic(sizeof(CIFPath));
 	newpath->cifp_next = polypath;
 	polypath = newpath;
-	dtmp = (xmax - xmaxoff) / 2;
-	dtmp += (dtmp < 0) ? -0.5 : 0.5;
-	newpath->cifp_x = (int)dtmp;
-	dtmp = (ymax - ymaxoff) / 2;
-	dtmp += (dtmp < 0) ? -0.5 : 0.5;
-	newpath->cifp_y = (int)dtmp;
+	newpath->cifp_x = round((xmax - xmaxoff) / 2);
+	newpath->cifp_y = round((ymax - ymaxoff) / 2);
 
 	newpath = (CIFPath *)mallocMagic(sizeof(CIFPath));
 	newpath->cifp_next = polypath;
 	polypath = newpath;
-	dtmp = (xmin - xminoff) / 2;
-	dtmp += (dtmp < 0) ? -0.5 : 0.5;
-	newpath->cifp_x = (int)dtmp;
-	dtmp = (ymin - yminoff) / 2;
-	dtmp += (dtmp < 0) ? -0.5 : 0.5;
-	newpath->cifp_y = (int)dtmp;
+	newpath->cifp_x = round((xmin - xminoff) / 2);
+	newpath->cifp_y = round((ymin - yminoff) / 2);
 
 	newpath = (CIFPath *)mallocMagic(sizeof(CIFPath));
 	newpath->cifp_next = polypath;
 	polypath = newpath;
-	dtmp = (xmin + xminoff) / 2;
-	dtmp += (dtmp < 0) ? -0.5 : 0.5;
-	newpath->cifp_x = (int)dtmp;
-	dtmp = (ymin + yminoff) / 2;
-	dtmp += (dtmp < 0) ? -0.5 : 0.5;
-	newpath->cifp_y = (int)dtmp;
+	newpath->cifp_x = round((xmin + xminoff) / 2);
+	newpath->cifp_y = round((ymin + yminoff) / 2);
 
 	/* Slow draw for non-Manhattan paths:			*/
 	/* Break the area up into triangles and rectangles	*/
