@@ -18,15 +18,13 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/cif/CIFtech.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFtech.c,v 1.3 2006/11/02 23:44:07 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include <ctype.h>
-#ifdef SYSV
-#include <string.h>
-#endif
 #include <stdlib.h>	/* for atof()	*/
+#include <string.h>
+#include <ctype.h>
 #include <math.h>	/* for pow()	*/
 
 #include "utils/magic.h"
@@ -43,12 +41,6 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/cif/
 #include "utils/malloc.h"
 #include "cif/cif.h"
 
-/* The following points to a list of all CIF styles currently
- * understood:
- */
-
-CIFKeep *CIFStyleList = NULL;
-
 /* The following statics are used to keep track of things between
  * calls to CIFTechLine.
  */
@@ -69,28 +61,25 @@ TileTypeBitMask CIFSolidBits;
 void cifTechStyleInit();
 bool cifCheckCalmaNum();
 
-
 /*
  * ----------------------------------------------------------------------------
  *
- * cifTechNewStyle --
+ * cifTechFreeStyle --
  *
- * 	This procedure creates a new CIF style at the end of
- *	the list of style and initializes it to completely
- *	null.
+ * 	This procedure frees memory for the current CIF style, and
+ *	sets the current style to NULL.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	A new element is added to the end of CIFStyleList, and CIFCurStyle
- *	is set to point to it.
+ *	Memory is free'd.
  *
  * ----------------------------------------------------------------------------
  */
 
 void
-cifTechNewStyle()
+cifTechFreeStyle()
 {
     int i;
     CIFOp	*op;
@@ -117,8 +106,34 @@ cifTechNewStyle()
 		freeMagic((char *)layer);
 	    }
 	}
+	freeMagic(CIFCurStyle);
+	CIFCurStyle = NULL;
     }
-    CIFCurStyle = (CIFStyle *) mallocMagic(sizeof(CIFStyle));
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * cifTechNewStyle --
+ *
+ * 	This procedure creates a new CIF style at the end of
+ *	the list of style and initializes it to completely
+ *	null.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	A new element is added to the end of CIFStyleList, and CIFCurStyle
+ *	is set to point to it.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+cifTechNewStyle()
+{
+    cifTechFreeStyle();
     cifTechStyleInit();
 }
 
@@ -137,12 +152,16 @@ cifTechStyleInit()
 {
     int i;
 
+    if (CIFCurStyle == NULL)
+	CIFCurStyle = (CIFStyle *) mallocMagic(sizeof(CIFStyle));
+
     CIFCurStyle->cs_name = NULL;
     CIFCurStyle->cs_status = TECH_NOT_LOADED;
 
     CIFCurStyle->cs_nLayers = 0;
     CIFCurStyle->cs_scaleFactor = 0;
     CIFCurStyle->cs_stepSize = 0;
+    CIFCurStyle->cs_gridLimit = 0;
     CIFCurStyle->cs_reducer = 0;
     CIFCurStyle->cs_expander = 1;
     CIFCurStyle->cs_yankLayers = DBZeroTypeBits;
@@ -319,9 +338,7 @@ CIFTechInit()
 
     /* Cleanup any old info. */
 
-    cifTechNewStyle();
-    freeMagic(CIFCurStyle);
-    CIFCurStyle = NULL;
+    cifTechFreeStyle();
 
     /* forget the list of styles */
 
@@ -364,6 +381,52 @@ CIFTechStyleInit()
     cifGotLabels = FALSE;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CIFTechLimitScale --
+ *
+ *	Determine if the scalefactor (ns / ds), applied to the current
+ *	grid scaling, would result in a grid finer than the minimum
+ *	resolution allowed by the process, as set by the "gridlimit"
+ *	statement in the "cifoutput" section (note that the scaling
+ *	depends on the output style chosen, and can be subverted by
+ *	scaling while a fine-grid output style is active, then switching
+ *	to a coarse-grid output style).
+ *
+ *	Note that even if the scalefactor is larger than the minimum
+ *	grid, it must be a MULTIPLE of the minimum grid, or else geometry
+ *	can be generated off-grid.
+ *
+ * Results:
+ *	TRUE if scaling by (ns / ds) would violate minimum grid resolution,
+ *	FALSE if not.
+ *
+ * Side effects:
+ *	None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+CIFTechLimitScale(ns, ds)
+    int ns, ds;
+{
+    int gridup, scaledown;
+
+    /* If no output style is active, it's okay to refine the	*/
+    /* grid arbitrarily.					*/
+
+    if (CIFCurStyle == NULL) return FALSE;
+
+    gridup = CIFCurStyle->cs_gridLimit * ds;
+    if (gridup == 0) return FALSE;
+
+    scaledown = CIFCurStyle->cs_scaleFactor * ns;
+    if ((scaledown % gridup) != 0) return TRUE;
+    return FALSE;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -647,6 +710,24 @@ CIFTechLine(sectionName, argc, argv)
 	return TRUE;
     }
 
+    /* Process "gridlimit" line next. */
+    if (strncmp(argv[0], "grid", 4) == 0)
+    {
+	if (StrIsInt(argv[1]))
+	{
+	    CIFCurStyle->cs_gridLimit = atoi(argv[1]);
+	    if (CIFCurStyle->cs_gridLimit < 0)
+	    {
+		TechError("Grid limit must be a positive integer.\n");
+		CIFCurStyle->cs_gridLimit = 0;
+	    }
+	}
+	else
+	    TechError("Unable to parse grid limit value.\n");
+
+	return TRUE;
+    }
+
     /* Process "variant" lines next. */
 
     if (strncmp(argv[0], "variant", 7) == 0)
@@ -1724,7 +1805,7 @@ CIFLoadStyle(stylename)
     invcif = TechSectionGetMask("cifoutput", NULL);
     TechLoad(NULL, invcif);
 
-    CIFTechFinal();
+    /* CIFTechFinal(); */  /* handled by TechLoad() */
     CIFTechOutputScale(DBLambda[0], DBLambda[1]);
 }
 
@@ -1836,6 +1917,8 @@ CIFTechOutputScale(n, d)
     BloatData *bloats;
     bool has_odd_space;
 
+    if (ostyle == NULL) return;
+
     /* For contact half-grid centering, check for odd numbers. . . */
 
     if (ostyle->cs_scaleFactor & 0x1)
