@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/commands/CmdCD.c,v 1.2 2005/05/13 19:31:58 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdCD.c,v 1.2 2006/04/14 06:51:24 anonymous Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -744,7 +744,7 @@ CmdCellname(w, cmd)
 		Tcl_AppendElement(magicinterp, "available");
 	    if (cellDef->cd_flags & CDMODIFIED)
 		Tcl_AppendElement(magicinterp, "modified");
-	    if (cellDef->cd_flags & CDMODIFIED)
+	    if (cellDef->cd_flags & CDNOEDIT)
 		Tcl_AppendElement(magicinterp, "readonly");
 #else
 	    TxPrintf("Flag settings for cell %s:\n", cellname);
@@ -1325,6 +1325,7 @@ CmdClockwise(w, cmd)
     }
     else goto badusage;
 
+    if (!ToolGetEditBox((Rect *)NULL)) return;
     if (degrees < 0) degrees += 360;
 
     switch (degrees)
@@ -1563,6 +1564,8 @@ CmdCopy(w, cmd)
 	int indx, amountx, amounty;
 	int xdelta, ydelta;
 
+	if (!ToolGetEditBox((Rect *)NULL)) return;
+
 	if (strcmp(cmd->tx_argv[1], "to") == 0)
 	{
 	    if (cmd->tx_argc != 4)
@@ -2735,6 +2738,8 @@ CmdDelete(w, cmd)
     TxCommand *cmd;
 {
     if (cmd->tx_argc != 1) goto badusage;
+    if (!ToolGetEditBox((Rect *)NULL)) return;
+
     SelectDelete("deleted", TRUE);
     return;
 
@@ -2890,10 +2895,11 @@ cmdDownEnumFunc(selUse, use, transform, area)
 #define DRC_OFF		10
 #define DRC_ON		11
 #define DRC_STATUS	12
-#define PRINTRULES	13
-#define RULESTATS	14
-#define STATISTICS	15
-#define WHY		16
+#define DRC_STYLE	13
+#define PRINTRULES	14
+#define RULESTATS	15
+#define STATISTICS	16
+#define WHY		17
 
 void
 CmdDrc(w, cmd)
@@ -2911,8 +2917,12 @@ CmdDrc(w, cmd)
     char 	**msg;
     bool	wizardHelp;
     bool	incremental;
+    bool	doforall = FALSE;
+    bool	dolist = FALSE;
     int		count_total;
     DRCCountList *dcl, *dclsrch;
+    int argc = cmd->tx_argc;
+    char **argv = cmd->tx_argv;
 
     static char *cmdDrcOption[] =
     {	
@@ -2929,6 +2939,7 @@ CmdDrc(w, cmd)
 	"off                    turn off background checker",
 	"on                     reenable background checker",
 	"status			report if the drc checker is on or off",
+	"style			set the DRC style",
 	"printrules [file]      print out design rules in file or on tty",
 	"rulestats              print out stats about design rule database",
 	"statistics             print out statistics gathered by checker",
@@ -2936,27 +2947,37 @@ CmdDrc(w, cmd)
 	NULL
     };
 
-    if (cmd->tx_argc < 2)
+    if (argc < 2)
     {
 	TxError("No option given in \":drc\" command.\n");
 	option = DRC_HELP;
     }
     else
     {
-	option = Lookup(cmd->tx_argv[1], cmdDrcOption);
+	if (!strncmp(argv[1], "list", 4))
+	{
+	    dolist = TRUE;
+	    if (!strncmp(argv[1], "listall", 7))
+	    {
+		doforall = TRUE;
+	    }
+	    argv++;
+	    argc--;
+	}
+	option = Lookup(argv[1], cmdDrcOption);
 	if (option < 0)
 	{
-	    TxError("%s isn't a valid drc option.\n", cmd->tx_argv[1]);
+	    TxError("%s isn't a valid drc option.\n", argv[1]);
 	    option = DRC_HELP;
-	    cmd->tx_argc = 2;
+	    argc = 2;
 	}
-	if ((cmd->tx_argc > 2) && (option != PRINTRULES) && (option != FIND)
+	if ((argc > 2) && (option != PRINTRULES) && (option != FIND)
 	    && (option != SHOWINT) && (option != DRC_HELP) && (option != EUCLIDEAN)
-	    && (option != DRC_STEPSIZE) && (option != DRC_HALO) && (option != COUNT))
+	    && (option != DRC_STEPSIZE) && (option != DRC_HALO) && (option != COUNT)
+	    && (option != DRC_STYLE))
 	{
 	    badusage:
-	    TxError("Wrong arguments in \"drc %s\" command:\n",
-		cmd->tx_argv[1]);
+	    TxError("Wrong arguments in \"drc %s\" command:\n", argv[1]);
 	    TxError("    :drc %s\n", cmdDrcOption[option]);
 	    TxError("Try \":drc help\" for more help.\n");
 	    return;
@@ -2972,8 +2993,8 @@ CmdDrc(w, cmd)
 	    break;
 	
 	case SHOWINT:
-	    if (cmd->tx_argc != 3) goto badusage;
-	    radius = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
+	    if (argc != 3) goto badusage;
+	    radius = cmdParseCoord(w, argv[2], TRUE, FALSE);
 	    if (radius < 0)
 	    {
 		TxPrintf("Radius must not be negative\n");
@@ -2991,6 +3012,13 @@ CmdDrc(w, cmd)
 	    ToolMoveBox(TOOL_BL, &area.r_ll, FALSE, rootUse->cu_def);
 	    ToolMoveCorner(TOOL_TR, &area.r_ur, FALSE, rootUse->cu_def);
 	    break;
+
+	case DRC_STYLE:
+	    if (argc == 2)
+		DRCPrintStyle(dolist, doforall, !doforall);
+	    else
+		DRCSetStyle(argv[2]);
+	    break;
 	
 	case CATCHUP:
 	    DRCCatchUp();
@@ -3005,8 +3033,8 @@ CmdDrc(w, cmd)
 	
 	case COUNT:
 	    count_total = -1;
-	    if (cmd->tx_argc == 3)
-		if (!strncmp(cmd->tx_argv[2], "total", 5))
+	    if (argc == 3)
+		if (!strncmp(argv[2], "total", 5))
 		    count_total = 0;
 
 	    if ((window = w) == NULL)
@@ -3045,7 +3073,7 @@ CmdDrc(w, cmd)
 	    break;
 
 	case EUCLIDEAN:
-	    if (!strcmp(cmd->tx_argv[2], "on"))
+	    if (!strcmp(argv[2], "on"))
 		DRCEuclidean = TRUE;
 	    else
 		DRCEuclidean = FALSE;
@@ -3059,15 +3087,15 @@ CmdDrc(w, cmd)
 	    }
 	    else
 		rootArea = w->w_surfaceArea;
-	    if (cmd->tx_argc > 3) goto badusage;
+	    if (argc > 3) goto badusage;
 
 	    rootUse = (CellUse *) window->w_surfaceID;
 	    rootDef = rootUse->cu_def;
 
 	    incremental = FALSE;
-	    if (cmd->tx_argc == 3 && StrIsInt(cmd->tx_argv[2]))
+	    if (argc == 3 && StrIsInt(argv[2]))
 	    {
-		drc_nth = atoi(cmd->tx_argv[2]);
+		drc_nth = atoi(argv[2]);
 		if (drc_nth <= 0) drc_nth = 1;
 	    }
 	    else
@@ -3100,29 +3128,33 @@ CmdDrc(w, cmd)
 	    break;
 	
 	case DRC_HALO:
-	    if (cmd->tx_argc == 3)
+	    if (argc == 3)
 	    {
-		TechHalo = atoi(cmd->tx_argv[2]);
-		if (TechHalo > MaxHalo)
-		    TechHalo = MaxHalo;
+		DRCTechHalo = cmdScaleCoord(w, argv[2], FALSE, TRUE);
+		DRCTechHalo *= DRCCurStyle->DRCScaleFactor;
+		if (DRCTechHalo > DRCCurStyle->DRCTechHalo)
+		    DRCTechHalo = DRCCurStyle->DRCTechHalo;
 
-		if (TechHalo < MaxHalo)
+		if (DRCTechHalo < DRCCurStyle->DRCTechHalo)
 		    TxPrintf("Warning: rulechecking limited to halo of %d lambda.\n",
-				TechHalo);
+				DRCTechHalo / DRCCurStyle->DRCScaleFactor);
 		else
-		    TxPrintf("DRC checks all rules (halo of %d lambda)\n", TechHalo);
+		    TxPrintf("DRC checks all rules (halo of %d lambda)\n",
+				DRCTechHalo / DRCCurStyle->DRCScaleFactor);
 	    }
 	    else
 	    {
-		TxPrintf("DRC halo is %d lambda\n", TechHalo);
-		if (TechHalo != MaxHalo)
-		    TxPrintf("Maximum rule distance is %d lambda\n", MaxHalo);
+		TxPrintf("DRC halo is %d lambda\n",
+				DRCTechHalo / DRCCurStyle->DRCScaleFactor);
+		if (DRCTechHalo != DRCCurStyle->DRCTechHalo)
+		    TxPrintf("Maximum rule distance is %d lambda\n",
+			DRCCurStyle->DRCTechHalo / DRCCurStyle->DRCScaleFactor);
 	    }
 	    break;
 
 	case DRC_HELP:
-	    if ((cmd->tx_argc == 3)
-		    && (strcmp(cmd->tx_argv[2], "wizard") == 0))
+	    if ((argc == 3)
+		    && (strcmp(argv[2], "wizard") == 0))
 		wizardHelp = TRUE;
 	    else wizardHelp = FALSE;
 	    TxPrintf("DRC commands have the form \":drc option\",");
@@ -3155,12 +3187,12 @@ CmdDrc(w, cmd)
 	    break;
 	
 	case PRINTRULES:
-	    if (cmd->tx_argc > 3) goto badusage;
-	    if (cmd->tx_argc < 3)
+	    if (argc > 3) goto badusage;
+	    if (argc < 3)
 		fp = stdout;
-	    else if ((fp = fopen (cmd->tx_argv[2],"w")) == (FILE *) NULL)
+	    else if ((fp = fopen (argv[2],"w")) == (FILE *) NULL)
 	    {
-		TxError("Cannot write file %s\n", cmd->tx_argv[2]);
+		TxError("Cannot write file %s\n", argv[2]);
 		return;
 	    }
 	    DRCPrintRulesTable (fp);
@@ -3177,16 +3209,21 @@ CmdDrc(w, cmd)
 	    break;
 
 	case DRC_STEPSIZE:
-	    if (cmd->tx_argc == 3)
+	    if (argc == 3)
 	    {
-		DRCStepSize = atoi(cmd->tx_argv[2]);
-		TxPrintf("DRC step size is now %d lambda)\n", DRCStepSize);
+		DRCStepSize = cmdScaleCoord(w, argv[2], FALSE, TRUE);
+		DRCStepSize *= DRCCurStyle->DRCScaleFactor;
+		TxPrintf("DRC step size is now %d units)\n",
+				DRCStepSize / DRCCurStyle->DRCScaleFactor);
 	    }
 	    else
 	    {
-		TxPrintf("DRC step size is %d lambda\n", DRCStepSize);
-		if (DRCStepSize != (16 * MaxHalo))
-		    TxPrintf("Recommended step size is %d lambda\n", 16 * MaxHalo);
+		TxPrintf("DRC step size is %d units\n",
+			DRCStepSize / DRCCurStyle->DRCScaleFactor);
+		if (DRCStepSize != (16 * DRCCurStyle->DRCTechHalo))
+		    TxPrintf("Recommended step size is %d units\n",
+				(16 * DRCCurStyle->DRCTechHalo) /
+				DRCCurStyle->DRCScaleFactor);
 	    }
 	    break;
 
