@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdFI.c,v 1.4 2001/12/10 16:34:16 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdFI.c,v 1.2 2006/06/16 16:16:11 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -51,6 +51,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdFI.c,v 1.4 200
 #include "drc/drc.h"
 #include "textio/txcommands.h"
 #include "utils/styles.h"
+#include "graphics/graphics.h"
 #include "extract/extract.h"
 #include "utils/malloc.h"
 #include "select/select.h"
@@ -67,7 +68,53 @@ struct cmdFillArea
     TileType cfa_type;			/* Type of material. */
     struct cmdFillArea *cfa_next;	/* Next in list of areas to fill. */
 };
-
+
+/* The following structure passes arguments needed by the	  */
+/* DBWFeedbackAdd command to the feedPolyFunc() callback routine. */
+
+struct cmdFPArg
+{
+   CellDef *def;
+   int style;
+   char *text;
+};
+
+#define FEEDMAGNIFY 20	/* Allow feedback to be drawn to 1/20 of an 	*/
+			/* internal unit.				*/
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * feedPolyFunc --
+ *
+ *	This procedure generates feedback entries in a polygonal area
+ *	by calling DBWFeedbackArea per tile on simple 1-bit types in
+ *	a plane.  The use of the temporary plane allows us to use the
+ *	same path decomposition routine used for CIF input, polygon
+ *	drawing, and wire segments.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+feedPolyFunc(tile, arg)
+    Tile *tile;
+    struct cmdFPArg *arg;
+{
+    Rect area;
+    TiToRect(tile, &area);
+
+    DBWFeedbackAdd(&area, arg->text, arg->def, FEEDMAGNIFY,
+#ifdef NONMANHATTAN
+        arg->style |
+                (TiGetTypeExact(tile) & (TT_DIAGONAL | TT_DIRECTION | TT_SIDE)));
+        /* (preserve information about the geometry of a diagonal tile) */
+#else
+        arg->style);
+#endif
+    return 0;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -106,8 +153,8 @@ CmdFeedback(w, cmd)
 {
     static char *cmdFeedbackOptions[] =
     {
-	"add text [style]		create new feedback area over box",
-	"clear				clear all feedback info",
+	"add text [style] [points...]	create new feedback area over box",
+	"clear [substring]		clear all or selected feedback info",
 	"count				count # feedback entries",
 	"find [nth]			put box over next [or nth] entry",
 	"help				print this message",
@@ -128,8 +175,8 @@ CmdFeedback(w, cmd)
     static int nth = 0;			/* Last entry displayed in
 					 * "feedback find".
 					 */
-    int option, i, style;
-    Rect box;
+    int option, i, style, pstart;
+    Rect box, r;
     char *text, **msg;
     CellDef *rootDef;
     HashTable table;
@@ -154,34 +201,173 @@ CmdFeedback(w, cmd)
 	TxError("    save\n");
 	return;
     }
+    pstart = 4;		/* argument # where point list starts, if any */
     switch (option)
     {
 	case ADD:
-	    if (cmd->tx_argc == 3) style = STYLE_PALEHIGHLIGHTS;
-	    else
+	    style = STYLE_PALEHIGHLIGHTS;
+	    if (cmd->tx_argc > 3)
 	    {
-		if (cmd->tx_argc != 4) goto badusage;
 		i = Lookup(cmd->tx_argv[3], cmdFeedbackStyleNames);
 		if (i < 0)
 		{
-		    TxError("%s isn't a valid display style.  Try one of:\n",
-			cmd->tx_argv[3]);
-		    TxError("    dotted        pale\n");
-		    TxError("    medium        solid\n");
-		    TxError("    outline\n");
-		    break;
+		    if (StrIsNumeric(cmd->tx_argv[3]))
+			pstart = 3;
+		    else
+		    {
+		        style = GrGetStyleFromName(cmd->tx_argv[3]);
+			if (style < 0)
+			{
+			    TxError("%s isn't a valid display style.  Try one of:\n",
+					cmd->tx_argv[3]);
+			    TxError("    dotted, pale, mediaum, solid, outline,\n");
+			    TxError("    or a long name from the .dstyle file\n");
+			    break;
+			}
+			if (style < 1 || style >= TECHBEGINSTYLES)
+	 		{
+			    TxError("Numbered styles must be between 1 and %d\n",
+					TECHBEGINSTYLES - 1);
+			    break;
+			}
+		    }
 		}
-		style = cmdFeedbackStyles[i];
+		else style = cmdFeedbackStyles[i];
+	    }
+	    if (cmd->tx_argc - pstart > 0)
+	    {
+		/* points must be in X Y pairs */
+		if ((cmd->tx_argc - pstart) & 1) goto badusage;
+
+	        if (w == NULL) return;
+		rootDef = ((CellUse *) w->w_surfaceID)->cu_def;
+
+		/* Read coordinates in FEEDMAGNIFY x internal units,	*/
+		/* then scale the highlight accordingly.		*/
+
+		if ((cmd->tx_argc - pstart) == 2)
+		{
+		    /* Single point highlight.  Style MUST be outlined	*/
+		    /* or else the point won't be visible.		*/
+
+		    if (GrStyleTable[style].outline == 0)
+			style = STYLE_OUTLINEHIGHLIGHTS;
+
+		    box.r_xbot = box.r_xtop = cmdScaleCoord(w, cmd->tx_argv[pstart++],
+				FALSE, TRUE, FEEDMAGNIFY);
+		    box.r_ybot = box.r_ytop = cmdScaleCoord(w, cmd->tx_argv[pstart++],
+				FALSE, FALSE, FEEDMAGNIFY);
+
+		    DBWFeedbackAdd(&box, cmd->tx_argv[2], rootDef, FEEDMAGNIFY, style);
+		}
+		else if ((cmd->tx_argc - pstart) == 4)
+		{
+		    /* Single line highlight.  Style MUST be outlined,	*/
+		    /* or else the line won't be visible.		*/
+
+		    if (GrStyleTable[style].outline == 0)
+			style = STYLE_OUTLINEHIGHLIGHTS;
+		    r.r_xbot = cmdScaleCoord(w, cmd->tx_argv[pstart++],
+				FALSE, TRUE, FEEDMAGNIFY);
+		    r.r_ybot = cmdScaleCoord(w, cmd->tx_argv[pstart++],
+				FALSE, FALSE, FEEDMAGNIFY);
+		    r.r_xtop = cmdScaleCoord(w, cmd->tx_argv[pstart++],
+				FALSE, TRUE, FEEDMAGNIFY);
+		    r.r_ytop = cmdScaleCoord(w, cmd->tx_argv[pstart++],
+				FALSE, FALSE, FEEDMAGNIFY);
+		    if (r.r_xbot != r.r_xtop && r.r_ybot != r.r_ytop)
+		    {
+			/* Line at an angle.  The hack of setting TT_SIDE */
+			/* without setting TT_DIAGONAL allows the drawing */
+			/* routine to treat this case as a line, not a	  */
+			/* triangle.					  */
+
+			style |= TT_SIDE;
+			if ((r.r_xbot > r.r_xtop && r.r_ybot < r.r_ytop) ||
+				(r.r_xbot < r.r_xtop && r.r_ybot > r.r_ytop))
+			    style |= TT_DIRECTION;
+		    }
+		    GeoCanonicalRect(&r, &box);
+		    DBWFeedbackAdd(&box, cmd->tx_argv[2], rootDef, FEEDMAGNIFY, style);
+		}
+		else
+		{
+		    Plane *plane;
+		    PaintResultType ptable[2] = {1, 1}; /* simple 1 bit table */
+		    TileTypeBitMask feedSimpleMask;
+		    Point *plist;
+		    int points, j;
+		    struct cmdFPArg fpargs;
+
+		    points = (cmd->tx_argc - pstart) >> 1;
+		    plist = (Point *)mallocMagic(points * sizeof(Point));
+		    fpargs.def = rootDef;
+		    fpargs.style = style;
+		    fpargs.text = cmd->tx_argv[2];
+
+		    for (i = 0, j = pstart; i < points; i++) 
+		    {
+			plist[i].p_x = cmdScaleCoord(w, cmd->tx_argv[j++],
+				FALSE, TRUE, FEEDMAGNIFY);
+			plist[i].p_y = cmdScaleCoord(w, cmd->tx_argv[j++],
+				FALSE, FALSE, FEEDMAGNIFY);
+		    }
+		    plane = DBNewPlane((ClientData)0);
+		    TTMaskZero(&feedSimpleMask);
+		    TTMaskSetType(&feedSimpleMask, 1);
+		    PaintPolygon(plist, points, plane, ptable, (PaintUndoInfo *)NULL);
+		    i = DBWFeedbackCount;
+		    DBSrPaintArea((Tile *)NULL, plane, &TiPlaneRect, &feedSimpleMask,
+			feedPolyFunc, (ClientData)&fpargs);
+		    TiFreePlane(plane);
+
+		    if (i == DBWFeedbackCount)
+		    {
+			/* Pathological condition---feedback area is so thin	*/
+			/* that the decomposition to rectangles and triangles	*/
+			/* was degenerate.  If so, then we can just treat the	*/
+			/* plist as several lines and/or points.		*/
+
+			if (GrStyleTable[style].outline == 0)
+			    style = STYLE_OUTLINEHIGHLIGHTS;
+
+			for (i = 0; i < points - 1; i++)
+			{
+			    r.r_xbot = plist[i].p_x;
+			    r.r_ybot = plist[i].p_y;
+			    r.r_xtop = plist[i + 1].p_x;
+			    r.r_ytop = plist[i + 1].p_y;
+			    if (r.r_xbot != r.r_xtop && r.r_ybot != r.r_ytop)
+		    	    {
+				style |= TT_SIDE;
+				if ((r.r_xbot > r.r_xtop && r.r_ybot < r.r_ytop) ||
+					(r.r_xbot < r.r_xtop && r.r_ybot > r.r_ytop))
+			    	style |= TT_DIRECTION;
+			    }
+			    GeoCanonicalRect(&r, &box);
+			    DBWFeedbackAdd(&box, cmd->tx_argv[2], rootDef,
+						FEEDMAGNIFY, style);
+			}
+		    }
+		    freeMagic(plist);
+		}	
+	    }
+	    else
+	    {
+		w = ToolGetBoxWindow(&box, (int *) NULL);
+	        if (w == NULL) return;
+		rootDef = ((CellUse *) w->w_surfaceID)->cu_def;
+		DBWFeedbackAdd(&box, cmd->tx_argv[2], rootDef, 1, style);
 	    }
-	    w = ToolGetBoxWindow(&box, (int *) NULL);
-	    if (w == NULL) return;
-	    rootDef = ((CellUse *) w->w_surfaceID)->cu_def;
-	    DBWFeedbackAdd(&box, cmd->tx_argv[2], rootDef, 1, style);
 	    break;
 	
 	case CLEAR:
-	    if (cmd->tx_argc != 2) goto badusage;
-	    DBWFeedbackClear();
+	    if (cmd->tx_argc == 3)
+	        DBWFeedbackClear(cmd->tx_argv[2]);
+	    else if (cmd->tx_argc == 2)
+	        DBWFeedbackClear(NULL);
+	    else
+		goto badusage;
 	    nth = 0;
 	    break;
 	
@@ -270,7 +456,7 @@ CmdFeedback(w, cmd)
 	    if (w == NULL) return;
 	    rootDef = ((CellUse *) w->w_surfaceID)->cu_def;
 	    HashInit(&table, 16, 0);
-	    for (i=0; i<DBWFeedbackCount; i++)
+	    for (i = 0; i < DBWFeedbackCount; i++)
 	    {
 		Rect area;
 		CellDef *fbRootDef;
@@ -979,7 +1165,7 @@ badusage:
  *
  * Usage:
  *	gridspace [spacing [spacing [xorig yorig]]]
- *	gridspace off
+ *	gridspace on|off|box|state|help|multiple
  *
  * Results:
  *	None.
@@ -990,34 +1176,63 @@ badusage:
  * ----------------------------------------------------------------------------
  */
 
+#define GRID_BOX	0
+#define GRID_HELP	1
+#define GRID_MULTIPLE	2
+#define GRID_OFF	3
+#define GRID_ON		4
+#define GRID_STATE	5
+#define GRID_TOGGLE	6
+#define GRID_WHAT	7
+
 void
 CmdGrid(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    int xSpacing, ySpacing, xOrig, yOrig;
+    int option, locargc;
+    int xSpacing, ySpacing, xOrig, yOrig, multiple;
     DBWclientRec *crec;
+    char *boxvalues;
+    static char *cmdGridOptions[] =
+    {
+	"box [values]	report the box representing the user grid",
+	"help		print this message",
+	"multiple [m]	set the grid multiple for drawing grids at large scales",
+	"off		turn off the user grid",
+	"on		turn on the user grid",
+	"state		report the state of the user grid",
+	"toggle		toggle the state (on/off) of the user grid",
+	"what		(equivalent to option \"box\")",
+	NULL
+    };
 
     windCheckOnlyWindow(&w, DBWclientID);
     if (w == (MagWindow *) NULL) return;
     crec = (DBWclientRec *) w->w_clientData;
+    locargc = cmd->tx_argc;
 
-    if ((cmd->tx_argc == 4) || (cmd->tx_argc > 5))
+    if (locargc == 1)
+	option = GRID_TOGGLE;
+    else if ((locargc == 2) && !strcmp(cmd->tx_argv[1], "0"))
+	option = GRID_OFF;
+    else
+	option = Lookup(cmd->tx_argv[1], cmdGridOptions);
+
+    /* Process various options with two arguments */
+    switch (option)
     {
-	TxError("Usage: %s [xSpacing [ySpacing [xOrig yOrig]]]]\n",
-		cmd->tx_argv[0]);
-	return;
-    }
+	case GRID_BOX:
+	    if (locargc > 2)
+	    {
+		locargc--;
+		break;
+	    }
 
-    /* Parse various options with two arguments */
+	case GRID_WHAT:
 
-    if (cmd->tx_argc == 2)
-    {
-	if (!strcmp(cmd->tx_argv[1], "box")
-		|| !strcmp(cmd->tx_argv[1], "what"))
-	{
 #ifdef MAGIC_WRAPPER
-	    char *boxvalues = (char *)Tcl_Alloc(50);
+	    boxvalues = (char *)Tcl_Alloc(50);
 	    sprintf(boxvalues, "%d %d %d %d",
 		crec->dbw_gridRect.r_xbot, crec->dbw_gridRect.r_ybot,
 		crec->dbw_gridRect.r_xtop, crec->dbw_gridRect.r_ytop);
@@ -1028,9 +1243,10 @@ CmdGrid(w, cmd)
 		crec->dbw_gridRect.r_xbot, crec->dbw_gridRect.r_ybot,
 		crec->dbw_gridRect.r_xtop, crec->dbw_gridRect.r_ytop);
 #endif
-	}
-	else if (!strcmp(cmd->tx_argv[1], "state"))
-	{
+	    return;
+
+	case GRID_STATE:
+
 #ifdef MAGIC_WRAPPER
 	    Tcl_SetObjResult(magicinterp,
 			Tcl_NewBooleanObj(crec->dbw_flags & DBW_GRID));
@@ -1038,36 +1254,76 @@ CmdGrid(w, cmd)
 	    TxPrintf("Grid is %s\n", 
 			(crec->dbw_flags & DBW_GRID) ? "on" : "off");
 #endif
-	}
-	else if (!strcmp(cmd->tx_argv[1], "help"))
-	{
+	    return;
+
+	case GRID_HELP:
+
 	    TxPrintf("Usage: grid [xSpacing [ySpacing [xOrig yOrig]]]]\n");
 	    TxPrintf("or     grid <option>\n");
-	    TxPrintf("where <option> is one of: on, off, state, box, or help.\n");
-	}
-	else if (!strcmp(cmd->tx_argv[1], "off")
-		|| !strcmp(cmd->tx_argv[1], "0"))
-	{
+	    TxPrintf("where <option> is one of: "
+			"on, off, state, box, what, help, or multiple.\n");
+	    return;
+
+	case GRID_OFF:
+
 	    if (crec->dbw_flags & DBW_GRID)
 	    {
 		crec->dbw_flags &= ~DBW_GRID;
 		WindAreaChanged(w, (Rect *) NULL);
 	    }
-	}
-	else if (!strcmp(cmd->tx_argv[1], "on"))
-	{
+	    return;
+
+	case GRID_ON:
 	    if (!(crec->dbw_flags & DBW_GRID))
 	    {
 		crec->dbw_flags |= DBW_GRID;
 		WindAreaChanged(w, (Rect *) NULL);
 	    }
-	}
-	return;
+	    return;
+
+	case GRID_MULTIPLE:
+	    if (locargc == 2)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp,
+			Tcl_NewIntObj(GrGridMultiple));
+#else
+		if (crec->dbw_multiple == 1)
+		    TxPrintf("Grid multiple drawing is off\n");
+		else
+		    TxPrintf("Grid multiple is %d\n", (int)GrGridMultiple);
+#endif
+	    }
+	    else if (StrIsInt(cmd->tx_argv[2]))
+	    {
+		multiple = atoi(cmd->tx_argv[2]);
+		if (multiple < 1 || multiple > 255)
+		    TxError("Usage: grid multiple <integer value 1-255>\n");
+		GrGridMultiple = (unsigned char)multiple;
+	    }
+	    else if (!strcmp(cmd->tx_argv[2], "off"))
+		GrGridMultiple = (unsigned char)1;
+	    else
+		TxError("Usage: grid multiple <integer value 1-255>\n");
+	    return;
+
+	case GRID_TOGGLE:
+ 	    crec->dbw_flags ^= DBW_GRID;
+	    break;
     }
 
-    if (cmd->tx_argc >= 2)
+    if ((option == GRID_BOX) || (option < 0))
     {
-	xSpacing = cmdParseCoord(w, cmd->tx_argv[1], TRUE, TRUE);
+	int argstart = (option == GRID_BOX) ? 2 : 1;
+
+	if ((locargc == 4) || (locargc > 5))
+	{
+	    TxError("Usage: %s [xSpacing [ySpacing [xOrig yOrig]]]]\n",
+			cmd->tx_argv[0]);
+	    return;
+	}
+
+	xSpacing = cmdParseCoord(w, cmd->tx_argv[argstart], TRUE, TRUE);
 	if (xSpacing <= 0)
 	{
 	    TxError("Grid spacing must be greater than zero.\n");
@@ -1076,19 +1332,19 @@ CmdGrid(w, cmd)
 	ySpacing = xSpacing;
 	xOrig = yOrig = 0;
 
-	if (cmd->tx_argc >= 3)
+	if (locargc >= 3)
 	{
-	    ySpacing = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
+	    ySpacing = cmdParseCoord(w, cmd->tx_argv[argstart + 1], TRUE, FALSE);
 	    if (ySpacing <= 0)
 	    {
 		TxError("Grid spacing must be greater than zero.\n");
 		return;
 	    }
 
-	    if (cmd->tx_argc == 5)
+	    if (locargc == 5)
 	    {
-		xOrig = cmdParseCoord(w, cmd->tx_argv[3], FALSE, TRUE);
-		yOrig = cmdParseCoord(w, cmd->tx_argv[4], FALSE, FALSE);
+		xOrig = cmdParseCoord(w, cmd->tx_argv[argstart + 2], FALSE, TRUE);
+		yOrig = cmdParseCoord(w, cmd->tx_argv[argstart + 3], FALSE, FALSE);
 	    }
 	}
 
@@ -1098,7 +1354,6 @@ CmdGrid(w, cmd)
 	crec->dbw_gridRect.r_ytop = yOrig + ySpacing;
 	crec->dbw_flags |= DBW_GRID;
     }
-    else crec->dbw_flags ^= DBW_GRID;
     WindAreaChanged(w, (Rect *) NULL);
 }
 
@@ -1523,6 +1778,7 @@ flatCopyAllLabels(scx, lab, tpath, targetUse)
     int targetPos;
     CellDef *def;
     char	labelname[1024];
+    char *n, *f, c;
 
     def = targetUse->cu_def;
     if (!GEO_LABEL_IN_AREA(&lab->lab_rect, &(scx->scx_area))) return 0;
@@ -1532,23 +1788,29 @@ flatCopyAllLabels(scx, lab, tpath, targetUse)
     /* Eliminate duplicate labels.  Don't pay any attention to layers
      * in deciding on duplicates:  if text and position match, it's a
      * duplicate.
-     *
-     * (Added 9/10/04) Make sure that the target flattened label is
+     */
+    /* 9/12/05---The utility of eliminating duplicate labels has been
+     * put into question by observing the O(n^2) behavior causing
+     * significant flattening times for layouts with many labels.
+     * Label text will not be duplicated anyway, because each label
+     * contains the heirarchical names.
+     */
+
+    /* (void) DBEraseLabelsByContent(def, &labTargetRect, targetPos,
+        -1, lab->lab_text); */
+
+    /* (Added 9/10/04) Make sure that the target flattened label is
      * not a port; possibly we should retain ports taken from the
      * top level cell, but certainly not any others.
      */
 
-    (void) DBEraseLabelsByContent(def, &labTargetRect, targetPos,
-        -1, lab->lab_text);
-    {
-    	 char *n = tpath->tp_next;
-    	 char *f = tpath->tp_first;
-	 char c = *n;
-	 strcpy(n,lab->lab_text);
-         (void) DBPutLabel(def, &labTargetRect, targetPos,
-			f, lab->lab_type, 0);
-	 *n = c;
-    }
+    n = tpath->tp_next;
+    f = tpath->tp_first;
+    c = *n;
+    strcpy(n, lab->lab_text);
+    DBPutLabel(def, &labTargetRect, targetPos, f, lab->lab_type, 0);
+    *n = c;
+
     return 0;
 }
 
@@ -1560,7 +1822,7 @@ flatCopyAllLabels(scx, lab, tpath, targetUse)
  * Implements flatten command
  *
  * Usage:
- *	flatten destname
+ *	flatten [-<option>] destname
  *
  * Results:
  *	creates new cell def with all the desired info.
@@ -1574,16 +1836,56 @@ CmdFlatten(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-     char		*destname = cmd->tx_argv[1];
+     int		rval, xMask;
+     bool		dolabels;
+     char		*destname;
      CellDef		*newdef;
      CellUse		*newuse;
      SearchContext	scx;
-     CellUse	*flatDestUse;
+     CellUse		*flatDestUse;
      
-    if (cmd->tx_argc != 2)
+    destname = cmd->tx_argv[cmd->tx_argc - 1];
+    xMask = CU_DESCEND_ALL;
+    dolabels = TRUE;
+
+    rval = 0;
+    if (cmd->tx_argc > 2)
     {
-     	  TxError("usage: flatten destcell\n");
-	  return;
+	int i;
+	for (i = 1; i < (cmd->tx_argc - 1); i++)
+	{
+	    if (strncmp(cmd->tx_argv[i], "-no", 3))
+	    {
+	        rval = -1;
+		break;
+	    }
+	    else if (strlen(cmd->tx_argv[i]) > 3)
+	    {
+		switch(cmd->tx_argv[1][3])
+		{
+		    case 'l':
+			dolabels = FALSE;
+			break;
+		    case 's':
+			xMask = CU_DESCEND_NO_SUBCKT;
+			break;
+		    case 'v':
+			xMask = CU_DESCEND_NO_VENDOR;
+			break;
+		    default:
+			TxError("options are: -nolabels, -nosubcircuits -novendor\n");
+			break;
+		}
+	    }
+	}
+    }
+    else if (cmd->tx_argc != 2)
+	rval = -1;
+
+    if (rval != 0)
+    {
+     	TxError("usage: flatten [-<option>...] destcell\n");
+	return;
     }
     /* create the new def */
     if (newdef = DBCellLookDef(destname))
@@ -1597,16 +1899,20 @@ CmdFlatten(w, cmd)
     newuse = DBCellNewUse(newdef, (char *) NULL);
     (void) StrDup(&(newuse->cu_id), "Flattened cell");
     DBSetTrans(newuse, &GeoIdentityTransform);
-    newuse->cu_expandMask = -1;
+    newuse->cu_expandMask = CU_DESCEND_SPECIAL;
     UndoDisable();
     flatDestUse = newuse;
     
     scx.scx_use  = EditCellUse;
     scx.scx_area = EditCellUse->cu_def->cd_bbox;
     scx.scx_trans = GeoIdentityTransform;
-    DBCellCopyAllPaint(&scx,&DBAllButSpaceAndDRCBits,0,flatDestUse);
-    FlatCopyAllLabels(&scx,&DBAllButSpaceAndDRCBits,0,flatDestUse);
-    /* DBCellEnum(EditCellUse->cu_def,flattenyank,&GeoIdentityTransform); */
+
+    DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, xMask, flatDestUse);
+    if (dolabels)
+	FlatCopyAllLabels(&scx, &DBAllTypeBits, xMask, flatDestUse);
+
+    if (xMask != CU_DESCEND_ALL)
+	DBCellCopyAllCells(&scx, xMask, flatDestUse, (Rect *)NULL);
     
     UndoEnable();
 }
