@@ -17,15 +17,13 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/commands/CmdRS.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdRS.c,v 1.5 2006/11/02 23:44:07 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
-#ifdef SYSV
 #include <string.h>
-#endif
+#include <ctype.h>
 
 #include "tcltk/tclmagic.h"
 #include "utils/magic.h"
@@ -129,21 +127,35 @@ CmdSave(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
+    CellDef *locDef;
+
     if (cmd->tx_argc > 2)
     {
 	TxError("Usage: %s [file]\n", cmd->tx_argv[0]);
 	return;
     }
 
-    ASSERT(EditCellUse != (CellUse *) NULL, "CmdSave");
+    /* The "save" command can turn a read-only cell into a writeable	*/
+    /* cell, or vice versa.  We should have more checks on the status	*/
+    /* of the resulting file. . . .  For now, we patch things up so	*/
+    /* that doing "save" on a read-only cell doesn't crash magic.	*/
+
+    if (EditCellUse == NULL)
+    {
+	locDef = ((CellUse *)w->w_surfaceID)->cu_def;
+	locDef->cd_flags &= ~CDNOEDIT;
+    }
+    else
+	locDef = EditCellUse->cu_def;
+
     DBUpdateStamps();
     if (cmd->tx_argc == 2)
     {
 	if (CmdIllegalChars(cmd->tx_argv[1], "[],", "Cell name"))
 	    return;
-	cmdSaveCell(EditCellUse->cu_def, cmd->tx_argv[1], FALSE, TRUE);
+	cmdSaveCell(locDef, cmd->tx_argv[1], FALSE, TRUE);
     }
-    else cmdSaveCell(EditCellUse->cu_def, (char *) NULL, FALSE, TRUE);
+    else cmdSaveCell(locDef, (char *) NULL, FALSE, TRUE);
 }
 
 
@@ -226,6 +238,15 @@ CmdScaleGrid(w, cmd)
 
 	ReduceFraction(&scalen, &scaled);
 
+	/* Check against CIF output style to see if we're violating a	*/
+	/* minimum grid resolution limit.				*/
+
+	if (CIFTechLimitScale(scalen, scaled) != 0)
+	{
+	    TxError("Grid scaling is finer than limit set by the process!\n");
+	    return;
+	}
+
 	/* Scale cifinput and cifoutput */
 
 	CIFTechInputScale(scalen, scaled, TRUE);
@@ -235,6 +256,10 @@ CmdScaleGrid(w, cmd)
 
 	DRCTechScale(scalen, scaled);
 
+	/* Scale plow rules (must come after DRCTechScale) */
+
+	PlowAfterTech();
+
 	/* Scale extract parameters */
 
 	ExtTechScale(scalen, scaled);
@@ -249,12 +274,17 @@ CmdScaleGrid(w, cmd)
 	LefTechScale(scalen, scaled);
 #endif
 
-	/* Scale router parameters */
-
 #ifdef ROUTE_MODULE
-	RtrTechScale(scalen, scaled);
-#endif
+	/* Scale core router parameters */
 
+	RtrTechScale(scalen, scaled);
+
+	/* Scale maze router parameters (must come after DRCTechScale) */
+
+	MZAfterTech();
+	IRAfterTech();
+
+#endif
 	/* Scale all tiles */
 
 	DBScaleEverything(scaled, scalen);
@@ -299,136 +329,6 @@ scalegridusage:
     return;
 }
 
-#ifdef MAGIC_WRAPPER
-/*
- * ----------------------------------------------------------------------------
- *
- * CmdSearch --
- *
- *	This is a low-level search command that allows a hierarchical tile
- *	search to execute a TCL procedure on each tile found.
- *
- *	Use: search [layers] <tcl_procedure>
- *
- *	<tcl_procedure> takes a list containing 4 integer values representing
- *	the bounding box (llx, lly, urx, ury) of the tile, and one value
- *	which is either a string containing the type name of the tile, or a
- *	list of two layers in the case of a split tile, representing LEFT
- *	and RIGHT types, respectively.
- *
- * ----------------------------------------------------------------------------
- */
-
-void
-CmdSearch(w, cmd)
-    MagWindow *w;
-    TxCommand *cmd;
-{
-    SearchContext scx;
-    TileTypeBitMask visitMask;
-    int xMask, windowMask;
-    char *tclproc, *pptr;
-    int cmdTclProcSrFunc();     /* Fwd declaration */
-    CellDef *rootBoxDef;
-    MagWindow *window;
-
-    bzero(&scx, sizeof(SearchContext));
-    if (!ToolGetBox (&rootBoxDef, NULL))
-    {
-	TxError ("Box tool must be present\n");
-	return;
-    }
-    window = ToolGetBoxWindow (&scx.scx_area, &windowMask);
-    if (!window)
-    {
-	TxError ("Box tool does not exist in any window\n");
-	return;
-    }
-    xMask = ((DBWclientRec *)window->w_clientData)->dbw_bitmask;
-    if ((windowMask & ~xMask) != 0)
-    {
-	window = CmdGetRootPoint((Point *) NULL, (Rect *) NULL);
-	xMask = ((DBWclientRec *) window->w_clientData)->dbw_bitmask;
-	if ((windowMask & xMask) == 0)
-	{
-	    TxPrintf("The box is in more than one window;  use the cursor\n");
-	    TxPrintf("to select the one you want to select from.\n");
-	    return;
-	}
-    }
-    scx.scx_use = (CellUse *) window->w_surfaceID;
-    scx.scx_trans = GeoIdentityTransform;
-
-    if ((cmd->tx_argc == 1) || (cmd->tx_argc > 3))
-    {
-	TxError("Usage: search [layers] <tcl_procedure>\n");
-	return;
-    }
-
-    if (cmd->tx_argc == 3)
-    {
-	if (!CmdParseLayers(cmd->tx_argv[1], &visitMask))
-	    return;
-    }
-    else visitMask = DBAllButSpaceAndDRCBits;
-    pptr = cmd->tx_argv[cmd->tx_argc - 1];
-    if (*pptr == '{')           /* in-line procedure */
-    {
-	tclproc = StrDup(NULL, pptr + 1);
-	pptr = tclproc + strlen(tclproc) - 1;
-	if (*pptr == '}') *pptr = '\0';
-    }
-    else
-	tclproc = StrDup(NULL, cmd->tx_argv[cmd->tx_argc - 1]);
-
-    (void) DBTreeSrTiles(&scx, &visitMask, xMask,
-		cmdTclProcSrFunc, (ClientData) tclproc);
-   
-    freeMagic(tclproc);
-}   
-
-/*
- * cmdTclProcSrFunc --
- *  
- * Filter procedure applied to tiles by cmdSearch() above.
- */ 
-  
-int
-cmdTclProcSrFunc(tile, cxp)
-    Tile *tile;
-    TreeContext *cxp;
-{
-    Rect tileRect, rootRect;
-    SearchContext *scx = cxp->tc_scx;
-    char *tclproc = (char *)cxp->tc_filter->tf_arg;
-    char evalstr[512], *eptr;
-    int rval;
-
-    TiToRect(tile, &tileRect);
-    GeoTransRect(&scx->scx_trans, &tileRect, &rootRect);
-    if (GEO_OVERLAP((&tileRect), &cxp->tc_scx->scx_area))
-    {
-	sprintf(evalstr, "%s %d %d %d %d ", tclproc, rootRect.r_xbot,
-		rootRect.r_ybot, rootRect.r_xtop, rootRect.r_ytop);
-	eptr = evalstr + strlen(evalstr);
-#ifdef NONMANHATTAN
-	if (IsSplit(tile))
-	    sprintf(eptr, "{%s %s}\n", DBTypeLongNameTbl[SplitLeftType(tile)],
-			DBTypeLongNameTbl[SplitRightType(tile)]);
-	else
-#endif
-	    sprintf(eptr, "%s\n", DBTypeLongNameTbl[TiGetType(tile)]);
-
-	rval = Tcl_EvalEx(magicinterp, evalstr, -1, 0);
-	if (rval != TCL_OK) return 1;
-    }
-    /* Allow interrupts in case something goes wrong with the Tcl procedure */
-    if (SigInterruptPending) return 1;
-    return 0;
-}   
-
-#endif
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1410,6 +1310,7 @@ CmdSideways(w, cmd)
 	TxError("Usage: %s\n", cmd->tx_argv[0]);
 	return;
     }
+    if (!ToolGetEditBox((Rect *)NULL)) return;
 
     /* To flip the selection sideways, first flip it around the
      * y-axis, then move it back so its lower-left corner is in
@@ -1924,6 +1825,8 @@ CmdStretch(w, cmd)
     {
 	int indx, amountx, amounty;
 
+	if (!ToolGetEditBox((Rect *)NULL)) return;
+
 	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, TRUE);
 	if (indx < 0)
 	    return;
