@@ -17,13 +17,11 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBcellname.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellname.c,v 1.2 2006/04/15 01:04:48 anonymous Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#ifdef SYSV
 #include <string.h>
-#endif
 
 #include "tcltk/tclmagic.h"
 #include "utils/magic.h"
@@ -1822,7 +1820,7 @@ dbGenerateUniqueIdsFunc(use, parentDef)
 	goto setHash;
 
     hedef = HashFind(&dbUniqueDefTable, (char *) use->cu_def);
-    for (suffix = (int) HashGetValue(hedef); ; suffix++)
+    for (suffix = (spointertype) HashGetValue(hedef); ; suffix++)
     {
 	(void) sprintf(name, "%s_%d", use->cu_def->cd_name, suffix);
 	hename = HashLookOnly(&dbUniqueNameTable, name);
@@ -1834,7 +1832,7 @@ dbGenerateUniqueIdsFunc(use, parentDef)
 	TxPrintf("Setting instance-id of cell %s to %s\n",
 		use->cu_def->cd_name, name);
     use->cu_id = StrDup((char **) NULL, name);
-    HashSetValue(hedef, suffix+1);
+    HashSetValue(hedef, (spointertype)suffix + 1);
 
 setHash:
     DBSetUseIdHash(use, parentDef);
