@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellsrch.c,v 1.3 2001/03/29 19:50:05 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellsrch.c,v 1.3 2006/10/04 18:26:56 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -34,6 +34,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellsrch.c,v 1.
 #include "utils/signals.h"
 #include "windows/windows.h"
 #include "utils/main.h"
+#include "mzrouter/mzrouter.h"
 
 /* Quick hack for dbScalePlanes() access to the CIF/GDS paint table */
 
@@ -190,7 +191,7 @@ dbCellTileSrFunc(scx, fp)
     int pNum;
 
     ASSERT(def != (CellDef *) NULL, "dbCellTileSrFunc");
-    if (!DBIsExpand(scx->scx_use, fp->tf_xmask))
+    if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask))
 	return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
 	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
@@ -206,6 +207,7 @@ dbCellTileSrFunc(scx, fp)
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (PlaneMaskHasPlane(fp->tf_planes, pNum))
 	{
+	    context.tc_plane = pNum;
 #ifdef NONMANHATTAN
 	    if (fp->tf_dinfo & TT_DIAGONAL)
 	    {
@@ -310,7 +312,7 @@ dbCellUniqueTileSrFunc(scx, fp)
     int pNum;
 
     ASSERT(def != (CellDef *) NULL, "dbCellUniqueTileSrFunc");
-    if (!DBIsExpand(scx->scx_use, fp->tf_xmask))
+    if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask))
 	return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
 	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
@@ -328,11 +330,11 @@ dbCellUniqueTileSrFunc(scx, fp)
 	{
 	    uMask = DBHomePlaneTypes(pNum);
 	    TTMaskAndMask(&uMask, fp->tf_mask);
-
-	    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
-		    &scx->scx_area, &uMask,
-		    fp->tf_func, (ClientData) &context))
-		return 1;
+	    if (!TTMaskIsZero(&uMask))
+	        if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
+		    	&scx->scx_area, &uMask, fp->tf_func,
+			(ClientData) &context))
+		    return 1;
 	}
 
     /*
@@ -417,7 +419,7 @@ DBNoTreeSrTiles(scx, mask, xMask, func, cdarg)
     int pNum;
 
     ASSERT(def != (CellDef *) NULL, "DBNoTreeSrTiles");
-    if (!DBIsExpand(cellUse, xMask))
+    if (!DBDescendSubcell(cellUse, xMask))
 	return 0;
 
     if ((def->cd_flags & CDAVAILABLE) == 0)
@@ -521,7 +523,7 @@ DBTreeSrLabels(scx, mask, xMask, tpath, func, cdarg)
     int dbCellLabelSrFunc();
 
     ASSERT(def != (CellDef *) NULL, "DBTreeSrLabels");
-    if (!DBIsExpand(cellUse, xMask)) return 0;
+    if (!DBDescendSubcell(cellUse, xMask)) return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
 	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
 
@@ -579,7 +581,7 @@ dbCellLabelSrFunc(scx, fp)
     int result;
 
     ASSERT(def != (CellDef *) NULL, "dbCellLabelSrFunc");
-    if (!DBIsExpand(scx->scx_use, fp->tf_xmask)) return 0;
+    if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask)) return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
 	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
     
@@ -684,7 +686,7 @@ DBTreeSrCells(scx, xMask, func, cdarg)
     TreeContext context;
     TreeFilter filter;
 
-    if (!DBIsExpand(cellUse, xMask))
+    if (!DBDescendSubcell(cellUse, xMask))
 	return 0;
     if ((cellUse->cu_def->cd_flags & CDAVAILABLE) == 0)
 	if (!DBCellRead(cellUse->cu_def, (char *) NULL, TRUE))
@@ -721,11 +723,11 @@ dbTreeCellSrFunc(scx, fp)
     CellUse *use = scx->scx_use;
     int result;
 
-    /* DBIsExpand treats a zero expand mask as "expanded everywhere",
+    /* DBDescendSubcell treats a zero expand mask as "expanded everywhere",
      * whereas we want it to mean "expanded nowhere".  Handle specially.
      */
 
-    if ((!DBIsExpand(use, fp->tf_xmask)) || (fp->tf_xmask == 0))
+    if ((!DBDescendSubcell(use, fp->tf_xmask)) || (fp->tf_xmask == CU_DESCEND_ALL))
 	result = (*fp->tf_func)(scx, fp->tf_arg);
     else
     {
@@ -1310,7 +1312,7 @@ typedef struct LCU1		/* A linked celluse record */
  *    the division step.  Values representing INFINITY are not scaled.
  *
  * Results:
- *	None.
+ *	TRUE if the value does not scale exactly, FALSE otherwise.
  *
  * Side effects:
  *	Integer value passed as pointer is scaled, unless it represents
@@ -1319,7 +1321,7 @@ typedef struct LCU1		/* A linked celluse record */
  * ----------------------------------------------------------------------------
  */
 
-void
+bool
 DBScaleValue(v, n, d)
     int *v, n, d;
 {
@@ -1342,6 +1344,7 @@ DBScaleValue(v, n, d)
 	if ((dlong)(*v) != llv)
 	    TxError("ERROR: ARITHMETIC OVERFLOW in DBScaleValue()!\n");
     }
+    return (((*v) % d) != 0);
 }
 
 /*
@@ -1354,7 +1357,7 @@ DBScaleValue(v, n, d)
  *    the division step.
  *
  * Results:
- *	None.
+ *	TRUE if the point does not scale exactly, FALSE otherwise.
  *
  * Side effects:
  *	Point structure pointed to by the first argument is scaled.
@@ -1362,13 +1365,15 @@ DBScaleValue(v, n, d)
  * ----------------------------------------------------------------------------
  */
 
-void
+bool
 DBScalePoint(p, n, d)
     Point *p;
     int n, d;
 {
-    DBScaleValue(&p->p_x, n, d);
-    DBScaleValue(&p->p_y, n, d);
+    bool result;
+    result = DBScaleValue(&p->p_x, n, d);
+    result |= DBScaleValue(&p->p_y, n, d);
+    return result;
 }
 
 /*
@@ -1385,7 +1390,8 @@ DBScalePoint(p, n, d)
  * Side Effects:
  *	Every single tile, label, and cell is altered.
  *	Geometry may be irrevocably distorted if transformed measurements
- *	would be sub-integer.
+ *	would be sub-integer.  The MODIFIED flag is set on any cell for
+ *	which any internal geometry does not scale precisely.
  *
  * ----------------------------------------------------------------------------
  */
@@ -1422,6 +1428,9 @@ DBScaleEverything(scalen, scaled)
 	lcd = lcd->cd_next;
     }
 
+    /* Recovery of global plane pointers */
+    MZAttachHintPlanes();
+
     SigEnableInterrupts();
 }
 
@@ -1433,6 +1442,7 @@ struct scaleArg {
    int pnum;
    Plane *ptarget;
    bool doCIF;
+   bool modified;
 };
 
 /*
@@ -1446,7 +1456,7 @@ struct scaleArg {
  * ----------------------------------------------------------------------------
  */
 
-void
+bool
 dbScalePlane(oldplane, newplane, pnum, scalen, scaled, doCIF)
     Plane *oldplane, *newplane;
     int pnum;
@@ -1461,8 +1471,11 @@ dbScalePlane(oldplane, newplane, pnum, scalen, scaled, doCIF)
     arg.ptarget = newplane;
     arg.pnum = pnum;
     arg.doCIF = doCIF;
+    arg.modified = FALSE;
     (void) DBSrPaintArea((Tile *) NULL, oldplane, &TiPlaneRect,
 		&DBAllButSpaceBits, dbTileScaleFunc, (ClientData) &arg);
+
+    return arg.modified;
 }
 
 /*
@@ -1490,8 +1503,10 @@ dbTileScaleFunc(tile, scvals)
 
     TiToRect(tile, &targetRect);
 
-    DBScalePoint(&targetRect.r_ll, scvals->scalen, scvals->scaled);
-    DBScalePoint(&targetRect.r_ur, scvals->scalen, scvals->scaled);
+    if (DBScalePoint(&targetRect.r_ll, scvals->scalen, scvals->scaled))
+	scvals->modified = TRUE;
+    if (DBScalePoint(&targetRect.r_ur, scvals->scalen, scvals->scaled))
+	scvals->modified = TRUE;
 
     /* Tile scaled out of existence! */
     if ((targetRect.r_xtop - targetRect.r_xbot == 0) ||
@@ -1520,6 +1535,72 @@ dbTileScaleFunc(tile, scvals)
     return 0;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBSrCellUses --
+ *
+ *   Do function "func" for each cell use in cellDef, passing "arg" as
+ *   client data.  Unlike DBEnumCell, this routine first collects a linked
+ *   list of cell uses, then performs the function on the list, so that
+ *   the search cannot be corrupted by, removing or reallocating the use
+ *   structure from the cell def.  Function "func" takes 2 arguments:
+ *
+ *	int func(Celluse *use, ClientData arg) {}
+ *
+ * Results:
+ *	0 one successful completion of the search, 1 on error.
+ *
+ * Side Effects:
+ *	Whatever "func" does.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+DBSrCellUses(cellDef, func, arg)
+    CellDef *cellDef;	/* Pointer to CellDef to search for uses. */
+    int (*func)();	/* Function to apply for each cell use.	  */
+    ClientData arg;	/* data to be passed to function func().  */
+{
+    int dbCellUseEnumFunc();
+    int retval;
+    CellUse *use;
+    LinkedCellUse *luhead, *lu;
+
+    /* DBCellEnum() attempts to read unavailable celldefs.  We don't	*/
+    /* want to do that here, so check CDAVAILABLE flag first.	  	*/
+
+    if ((cellDef->cd_flags & CDAVAILABLE) == 0) return 0;
+
+    /* Enumerate all unique cell uses, and scale their position,	*/
+    /* transform, and array information.				*/
+
+    luhead = NULL;
+
+    retval = DBCellEnum(cellDef, dbCellUseEnumFunc, (ClientData) &luhead);
+
+    lu = luhead;
+    while (lu != NULL)
+    {
+	use = lu->cellUse;
+	if ((*func)(use, arg)) {
+	   retval = 1;
+	   break;
+	}
+	lu = lu->cu_next;
+    }
+
+    /* Free this linked cellUse structure */
+    lu = luhead;
+    while (lu != NULL)
+    {
+	freeMagic((char *)lu);
+	lu = lu->cu_next;
+    }
+    return retval;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1547,7 +1628,10 @@ dbScaleCell(cellDef, scalen, scaled)
     /* DBCellEnum() attempts to read unavailable celldefs.  We don't	*/
     /* want to do that here, so check CDAVAILABLE flag first.	  	*/
 
-    if ((cellDef->cd_flags & CDAVAILABLE) == 0) goto donecell;
+    if ((cellDef->cd_flags & CDAVAILABLE) == 0)
+	goto donecell;
+    else
+	cellDef->cd_flags |= CDBOXESCHANGED;
 
     /* Enumerate all unique cell uses, and scale their position,	*/
     /* transform, and array information.				*/
@@ -1624,13 +1708,17 @@ dbScaleCell(cellDef, scalen, scaled)
 	if (cellDef->cd_planes[pNum] == NULL) continue;
 	newplane = DBNewPlane((ClientData) TT_SPACE);
 	DBClearPaintPlane(newplane);
-	dbScalePlane(cellDef->cd_planes[pNum], newplane, pNum,
-		scalen, scaled, FALSE);
+	if (dbScalePlane(cellDef->cd_planes[pNum], newplane, pNum,
+		scalen, scaled, FALSE))
+	    cellDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
         DBFreePaintPlane(cellDef->cd_planes[pNum]);
 	TiFreePlane(cellDef->cd_planes[pNum]);
         cellDef->cd_planes[pNum] = newplane;
     }
 
+    /* Check consistency of several global pointers	*/
+    /* WARNING:  This list may not be complete!		*/
+
     /* Also scale the position of all labels */
 
     if (cellDef->cd_labels)
@@ -1649,9 +1737,6 @@ donecell:
     DBScalePoint(&cellDef->cd_bbox.r_ll, scalen, scaled);
     DBScalePoint(&cellDef->cd_bbox.r_ur, scalen, scaled);
 
-    /* Perhaps we should not do this if CD_AVAILABLE is not set? */
-    cellDef->cd_flags &= ~(CDMODIFIED|CDBOXESCHANGED|CDSTAMPSCHANGED);
-
     return 0;
 }
 
