@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBconnect.c,v 1.7 2001/11/29 20:08:51 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBconnect.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -301,7 +301,7 @@ dbSrConnectFunc(tile, csa)
     int i;
     TileTypeBitMask *connectMask;
     TileType loctype, checktype;
-    unsigned int planes;
+    PlaneMask planes;
 
     TiToRect(tile, &tileArea);
 
@@ -639,7 +639,7 @@ dbcConnectFunc(tile, cx)
 #ifdef NONMANHATTAN
     TileType dinfo = 0;
 #endif
-    int pNum;
+    int pNum = cx->tc_plane;
     CellDef *def;
 
     TiToRect(tile, &tileArea);
@@ -682,7 +682,6 @@ dbcConnectFunc(tile, cx)
      * since any processing that is needed was already done before.
      */
 	
-    pNum = DBPlane(loctype);
     connectMask = &csa2->csa2_connect[loctype];
 
     /* In the case of contact bits, the types underneath
@@ -696,9 +695,13 @@ dbcConnectFunc(tile, cx)
 
 	TTMaskSetOnlyType(&notConnectMask, loctype);
 
-	/* Differenct contact types may share residues (6/18/04) */
+	/* Different contact types may share residues (6/18/04) */
+	/* Use TTMaskIntersect(), not TTMaskEqual()---types	*/
+	/* which otherwise stack may be in separate cells	*/
+	/* (12/1/05)						*/
+
 	for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
-	    if (TTMaskEqual(rMask, DBResidueMask(ctype)))
+	    if (TTMaskIntersect(rMask, DBResidueMask(ctype)))
 		TTMaskSetType(&notConnectMask, ctype);
 
 	/* The mask of contact types must include all stacked contacts */
