@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtcontact.c,v 1.2 2001/01/12 22:12:21 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtcontact.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -46,7 +46,7 @@ TileTypeBitMask DBLayerTypeMaskTbl[NT];
 
 /* Filled in after contact types have been generated */
 TileTypeBitMask DBPlaneTypes[PL_MAXTYPES];
-int 	        DBTypePlaneMaskTbl[NT];
+PlaneMask        DBTypePlaneMaskTbl[NT];
 
 /* --------------------- Data local to this file ---------------------- */
 
@@ -347,8 +347,11 @@ dbTechAddOneStackedContact(type1, type2)
     /* Both types must be contacts */
     if (!lim->l_isContact || !lin->l_isContact) return -1;
 
-    /* Contacts do not stack if they share all the same planes */
-    if (lim->l_pmask == lin->l_pmask) return -1;
+    /* Contacts do not stack if they share more than one plane */
+    /* if (lim->l_pmask == lin->l_pmask) return -1; */
+    if (((lim->l_pmask & lin->l_pmask) & ((lim->l_pmask & lin->l_pmask) - 1))
+		!= 0)
+	return -1;
 
     TTMaskAndMask3(&ttshared, &lim->l_residues, &lin->l_residues);
 
@@ -485,7 +488,8 @@ dbTechContactResidues(argc, argv, contactType)
     char **argv;
     TileType contactType;
 {
-    int       homePlane, residuePlane, nresidues, pMask;
+    int       homePlane, residuePlane, nresidues;
+    PlaneMask pMask;
     TileType  residueType, imageType;
     bool      residueOnHome;
     LayerInfo *lp;
@@ -716,7 +720,8 @@ DBHomePlaneTypes(pNum)
     int pNum;
 {
     TileTypeBitMask uMask, *rMask;
-    int m, n, p;
+    int m, n;
+    PlaneMask p;
     LayerInfo *lc, *lr;
 
     TTMaskZero(&uMask);
@@ -895,6 +900,51 @@ DBResidueMask(type)
     return (&li->l_residues);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBFullResidueMask --
+ *
+ *   Get the residue mask of the specified type.  For stacking contacts,
+ *   decompose the contact residues into their component residue layers.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	"rmask" is a pointer to a TileTypeBitMask.  The result is placed
+ *	in this location.
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBFullResidueMask(type, rmask)
+    TileType type;
+    TileTypeBitMask *rmask;
+{
+    TileType t;
+    TileTypeBitMask *lmask;
+    LayerInfo *li, *lr;
+
+    li = &dbLayerInfo[type];
+    lmask = &li->l_residues;
+    TTMaskZero(rmask);
+
+    if (type < DBNumUserLayers)
+    {
+	TTMaskSetMask(rmask, &li->l_residues);
+    }
+    else
+    {
+	for (t = DBNumUserLayers; t < DBNumTypes; t++)
+	    if (TTMaskHasType(lmask, t))
+	    {
+		lr = &dbLayerInfo[t];
+		TTMaskSetMask(rmask, &lr->l_residues);
+	    }
+    }
+}
+
 
 /*
  * ----------------------------------------------------------------------------
