@@ -20,14 +20,12 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBtechname.c,v 1.2 2005/06/01 21:41:40 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtechname.c,v 1.3 2006/04/23 01:50:22 anonymous Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include <ctype.h>
-#ifdef SYSV
 #include <string.h>
-#endif
+#include <ctype.h>
 
 #include "tcltk/tclmagic.h"
 #include "utils/magic.h"
@@ -73,7 +71,7 @@ DBTechNameType(typename)
 
     slash = index(typename, '/');
     if (slash != NULL) *slash = 0;
-    type = (TileType) dbTechNameLookup(typename, &dbTypeNameLists);
+    type = (TileType)(spointertype) dbTechNameLookup(typename, &dbTypeNameLists);
     if (slash == NULL) return type;
     *slash = '/';
     if (type < 0) return type;
@@ -81,7 +79,7 @@ DBTechNameType(typename)
     /* There's a plane qualification.  Make sure the type exists */
     /* on the indicated plane.  If not, return an error.	 */
 
-    plane = (int) dbTechNameLookup(slash + 1, &dbPlaneNameLists);
+    plane = (spointertype) dbTechNameLookup(slash + 1, &dbPlaneNameLists);
     if (plane < 0) return -2;
 
     lp = &dbLayerInfo[type];
@@ -118,7 +116,7 @@ DBTechNameTypes(typename, bitmask)
     TTMaskZero(bitmask);
     slash = index(typename, '/');
     if (slash != NULL) *slash = 0;
-    type = (TileType) dbTechNameLookup(typename, &dbTypeNameLists);
+    type = (TileType)(spointertype) dbTechNameLookup(typename, &dbTypeNameLists);
     if (type < 0)
     {
 	HashEntry *he;
@@ -145,7 +143,7 @@ DBTechNameTypes(typename, bitmask)
 
     /* There's a plane qualification.  Locate the image. */
 
-    plane = (int) dbTechNameLookup(slash + 1, &dbPlaneNameLists);
+    plane = (spointertype) dbTechNameLookup(slash + 1, &dbPlaneNameLists);
     if (plane < 0) return -2;
 
     lp = &dbLayerInfo[type];
@@ -220,7 +218,7 @@ int
 DBTechNamePlane(planename)
     char *planename;	/* The name of the plane */
 {
-    return ((int) dbTechNameLookup(planename, &dbPlaneNameLists));
+    return ((spointertype) dbTechNameLookup(planename, &dbPlaneNameLists));
 }
 
 /*
@@ -292,7 +290,7 @@ DBTypeShortName(type)
 	    tbl != &dbTypeNameLists;
 	    tbl = tbl->sn_next)
     {
-	if (tbl->sn_value == (ClientData) type && tbl->sn_primary)
+	if (tbl->sn_value == (ClientData)(pointertype) type && tbl->sn_primary)
 	    return (tbl->sn_name);
     }
 
@@ -312,7 +310,7 @@ DBPlaneShortName(pNum)
 	    tbl != &dbPlaneNameLists;
 	    tbl = tbl->sn_next)
     {
-	if (tbl->sn_value == (ClientData) pNum && tbl->sn_primary)
+	if (tbl->sn_value == (ClientData)(pointertype) pNum && tbl->sn_primary)
 	    return (tbl->sn_name);
     }
 
@@ -403,7 +401,7 @@ DBTechPrintTypes(dolist)
 	for (p = dbTypeNameLists.sn_next; p != &dbTypeNameLists;
 		p = p->sn_next)
 	{
-	    if (((TileType) p->sn_value) == i)
+	    if (((TileType)(spointertype) p->sn_value) == i)
 	    {
 		if (dolist) 
 		{
@@ -439,7 +437,7 @@ DBTechPrintTypes(dolist)
 	    for (p = dbTypeNameLists.sn_next; p != &dbTypeNameLists;
 		p = p->sn_next)
 	    {
-		if (((TileType) p->sn_value) == dtp->dt_type)
+		if (((TileType)(spointertype) p->sn_value) == dtp->dt_type)
 		{
 		    if (dolist) 
 		    {
@@ -497,7 +495,7 @@ DBTechPrintCanonicalType(layername)
 	for (p = dbTypeNameLists.sn_next; p != &dbTypeNameLists;
 		p = p->sn_next)
 	{
-	    if (((TileType) p->sn_value) == i)
+	    if (((TileType)(spointertype) p->sn_value) == i)
 	    {
 		if (first) keepname = p->sn_name;
 		else if (strlen(p->sn_name) > strlen(keepname))
@@ -666,7 +664,8 @@ DBTechNoisyNameMask(layers, mask)
 		    }
 
 		    t = DBTechNoisyNameType(p);
-		    if (t >= 0) m2 = DBLayerTypeMaskTbl[t];
+		    if (t >= 0)
+			m2 = DBLayerTypeMaskTbl[t];
 
 		    /* Include all types which have t as a residue */
 		    if (allResidues)
