@@ -18,14 +18,12 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBtechtype.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtechtype.c,v 1.2 2006/10/04 19:38:33 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include <ctype.h>
-#ifdef SYSV
 #include <string.h>
-#endif
+#include <ctype.h>
 
 #include "utils/magic.h"
 #include "utils/geometry.h"
@@ -232,6 +230,7 @@ DBTechInitType()
 	}
 	DBTypeLongNameTbl[dtp->dt_type] = cp;
 	DBPlane(dtp->dt_type) = dtp->dt_plane;
+        TTMaskSetOnlyType(&DBLayerTypeMaskTbl[dtp->dt_type], dtp->dt_type);
     }
 
     /* Hash table of layer aliases, free'ing the allocated type masks */
@@ -359,7 +358,8 @@ DBTechAddType(sectionName, argc, argv)
     if (!strcmp(argv[0], "alias"))
     {
 	HashEntry *he;
-	TileTypeBitMask *amask;
+	TileType atype;
+	TileTypeBitMask *amask, tmask;
 
 	/* First check that the alias name does not shadow an existing type */
 	if (DBTechNameType(argv[1]) >= 0)
@@ -368,12 +368,26 @@ DBTechAddType(sectionName, argc, argv)
 	    return TRUE;
 	}
 
+	/* Next, check the type list.  If there is only one type, then	*/
+	/* add the name to the type's names as usual, instead of adding	*/
+	/* it to the alias hash table, which is meant for names that	*/
+	/* map to multiple types.					*/
+
+	DBTechNoisyNameMask(argv[2], &tmask);
+	if ((atype = DBTechNameType(argv[2])) >= 0)
+	    if (TTMaskEqual(&DBLayerTypeMaskTbl[atype], &tmask))
+	    {
+		DBTechAddNameToType(argv[1], atype);
+		return TRUE;
+	    }
+
 	he = HashFind(&DBTypeAliasTable, argv[1]);
 	amask = (TileTypeBitMask *)HashGetValue(he);
 	if (amask == NULL)
 	{
 	    amask = (TileTypeBitMask *)mallocMagic(sizeof(TileTypeBitMask));
-	    DBTechNoisyNameMask(argv[2], amask);
+	    TTMaskZero(amask);
+	    TTMaskSetMask(amask, &tmask);
 	    HashSetValue(he, amask);
 	}
 	else
@@ -394,6 +408,7 @@ DBTechAddType(sectionName, argc, argv)
 
 	DBTypeLongNameTbl[DBNumTypes] = cp;
 	DBPlane(DBNumTypes) = pNum;
+	TTMaskSetOnlyType(&DBLayerTypeMaskTbl[DBNumTypes], DBNumTypes);
 	DBNumTypes++;
     }
 
