@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtiles.c,v 1.3 2001/03/29 19:50:06 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtiles.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -110,7 +110,8 @@ DBSrPaintNMArea(hintTile, plane, ttype, rect, mask, func, arg)
     Point start;
     Tile *tp, *tpnew;
     TileType tpt;
-    int rheight, rwidth, rmax, f1, f2, f3, f4;
+    int rheight, rwidth, rmax;
+    dlong f1, f2, f3, f4;
     bool ignore_sides;
 
     /* If the search area is not diagonal, return the result of the	*/
@@ -141,20 +142,20 @@ nm_enum:
 	rwidth = rect->r_xtop - rect->r_xbot;
 	rmax = MAX(rheight, rwidth);
 	f1 = (BOTTOM(tp) > MINFINITY + 2) ?
-		((rect->r_ytop - BOTTOM(tp)) * rwidth) : INFINITY - 2;
+		((dlong)(rect->r_ytop - BOTTOM(tp)) * rwidth) : DLONG_MAX;
 	f2 = (TOP(tp) < INFINITY - 2) ?
-		((TOP(tp) - rect->r_ybot) * rwidth) : INFINITY - 2;
+		((dlong)(TOP(tp) - rect->r_ybot) * rwidth) : DLONG_MAX;
 
 	if (ttype & TT_SIDE)
 	{
 	    /* Outside-of-triangle check---ignore sub-integer slivers */
 	    if (RIGHT(tp) < INFINITY - 2)
 	    {
-		f3 = (rect->r_xtop - RIGHT(tp)) * rheight;
+		f3 = (dlong)(rect->r_xtop - RIGHT(tp)) * rheight;
 		f3 += rmax;
 	    }
 	    else
-		f3 = MINFINITY + 2;
+		f3 = DLONG_MIN;
 	    if ((ttype & TT_DIRECTION) ? (f2 < f3) : (f1 < f3))
 		goto enum_next;
 	}
@@ -163,11 +164,11 @@ nm_enum:
 	    /* Outside-of-triangle check---ignore sub-integer slivers */
 	    if (LEFT(tp) > MINFINITY + 2)
 	    {
-		f4 = (LEFT(tp) - rect->r_xbot) * rheight;
+		f4 = (dlong)(LEFT(tp) - rect->r_xbot) * rheight;
 		f4 += rmax;
 	    }
 	    else
-		f4 = MINFINITY + 2;
+		f4 = DLONG_MIN;
 	    if ((ttype & TT_DIRECTION) ? (f1 < f4) : (f2 < f4))
 		goto enum_next;
 	}
@@ -191,15 +192,15 @@ nm_enum:
 	    if ((tpt & TT_DIRECTION) == (ttype & TT_DIRECTION))
 	    {
 		f3 = (LEFT(tp) > MINFINITY + 2) ?
-			((rect->r_xtop - LEFT(tp)) * rheight) : INFINITY - 2;
+			((dlong)(rect->r_xtop - LEFT(tp)) * rheight) : DLONG_MAX;
 		f4 = (RIGHT(tp) < INFINITY - 2) ?
-			((RIGHT(tp) - rect->r_xbot) * rheight) : INFINITY - 2;
+			((dlong)(RIGHT(tp) - rect->r_xbot) * rheight) : DLONG_MAX;
 
 		if (ttype & TT_SIDE)
 		{
 		    /* Ignore sub-integer slivers */
-		    if (f4 != INFINITY - 2) f4 -= rmax;
-		    if (f3 != INFINITY - 2) f3 += rmax;
+		    if (f4 != DLONG_MAX) f4 -= rmax;
+		    if (f3 != DLONG_MAX) f3 += rmax;
 		    
 		    if (ttype & TT_DIRECTION)
 		    {
@@ -217,8 +218,8 @@ nm_enum:
 		else
 		{
 		    /* Ignore sub-integer slivers */
-		    if (f4 != INFINITY - 2) f4 += rmax;
-		    if (f3 != INFINITY - 2) f3 -= rmax;
+		    if (f4 != DLONG_MAX) f4 += rmax;
+		    if (f3 != DLONG_MAX) f3 -= rmax;
 		    
 		    if (ttype & TT_DIRECTION)
 		    {
@@ -243,17 +244,17 @@ nm_enum:
 	    rwidth = RIGHT(tp) - LEFT(tp);
 	    rmax = MAX(rheight, rwidth);
 	    f1 = (TOP(tp) < INFINITY - 2) ?
-			((TOP(tp) - rect->r_ybot) * rwidth) : INFINITY - 2;
+			((dlong)(TOP(tp) - rect->r_ybot) * rwidth) : DLONG_MAX;
 	    f2 = (BOTTOM(tp) > MINFINITY + 2) ?
-			((rect->r_ytop - BOTTOM(tp)) * rwidth) : INFINITY - 2;
+			((dlong)(rect->r_ytop - BOTTOM(tp)) * rwidth) : DLONG_MAX;
 	    f3 = (RIGHT(tp) < INFINITY - 2) ?
-			((RIGHT(tp) - rect->r_xtop) * rheight) : INFINITY - 2;
+			((dlong)(RIGHT(tp) - rect->r_xtop) * rheight) : DLONG_MAX;
 	    f4 = (LEFT(tp) > MINFINITY + 2) ?
-			((rect->r_xbot - LEFT(tp)) * rheight) : INFINITY - 2;
+			((dlong)(rect->r_xbot - LEFT(tp)) * rheight) : DLONG_MAX;
 
 	    /* ignore sub-integer slivers */
-	    if (f4 < INFINITY - 2) f4 += rmax;
-	    if (f3 < INFINITY - 2) f3 += rmax;
+	    if (f4 < DLONG_MAX) f4 += rmax;
+	    if (f3 < DLONG_MAX) f3 += rmax;
 
 	    if (SplitDirection(tp) ? (f1 < f4) : (f2 < f4))
 		ignore_sides |= IGNORE_LEFT;
@@ -405,20 +406,21 @@ enumerate:
 
 	    /* Watch for calculations involving (M)INFINITY     */
 
-	    int theight, twidth, f1, f2, f3, f4;
+	    int theight, twidth;
+	    dlong f1, f2, f3, f4;
 
 	    theight = TOP(tp) - BOTTOM(tp);
 	    twidth = RIGHT(tp) - LEFT(tp);
 	    f1 = (rect->r_ybot > MINFINITY + 2) ?
-		(TOP(tp) - rect->r_ybot) * twidth : INFINITY - 2;
+		(dlong)(TOP(tp) - rect->r_ybot) * twidth : DLONG_MAX;
 	    f2 = (rect->r_ytop < INFINITY - 2) ?
-		(rect->r_ytop - BOTTOM(tp)) * twidth : INFINITY - 2;
+		(dlong)(rect->r_ytop - BOTTOM(tp)) * twidth : DLONG_MAX;
 
 	    if (TTMaskHasType(mask, SplitLeftType(tp)))
 	    {
 		/* !Outside-of-triangle check */
 		f4 = (rect->r_xbot > MINFINITY + 2) ?
-			(rect->r_xbot - LEFT(tp)) * theight : MINFINITY + 2;
+			(dlong)(rect->r_xbot - LEFT(tp)) * theight : DLONG_MIN;
 		if (SplitDirection(tp) ? (f1 > f4) : (f2 > f4))
 		{
 		    TiSetBody(tp, (ClientData)((TileType)TiGetBody(tp)
@@ -431,7 +433,7 @@ enumerate:
 	    {
 		/* !Outside-of-triangle check */
 		f3 = (rect->r_xtop < INFINITY - 2) ?
-			(RIGHT(tp) - rect->r_xtop) * theight : MINFINITY + 2;
+			(dlong)(RIGHT(tp) - rect->r_xtop) * theight : DLONG_MIN;
 		if (SplitDirection(tp) ? (f2 > f3) : (f1 > f3))
 		{
 		    TiSetBody(tp, (ClientData)((TileType)TiGetBody(tp)
@@ -566,20 +568,21 @@ enumerate:
 
 	    /* Watch for calculations involving (M)INFINITY     */
 
-	    int theight, twidth, f1, f2, f3, f4;
+	    int theight, twidth;
+	    dlong f1, f2, f3, f4;
 
 	    theight = TOP(tp) - BOTTOM(tp);
 	    twidth = RIGHT(tp) - LEFT(tp);
 	    f1 = (rect->r_ybot > MINFINITY + 2) ?
-		(TOP(tp) - rect->r_ybot) * twidth : INFINITY - 2;
+		(TOP(tp) - rect->r_ybot) * twidth : DLONG_MAX;
 	    f2 = (rect->r_ytop < INFINITY - 2) ?
-		(rect->r_ytop - BOTTOM(tp)) * twidth : INFINITY - 2;
+		(rect->r_ytop - BOTTOM(tp)) * twidth : DLONG_MAX;
 
 	    if (TTMaskHasType(mask, SplitLeftType(tp)))
 	    {
 		/* !Outside-of-triangle check */
 		f4 = (rect->r_xbot > MINFINITY + 2) ?
-			(rect->r_xbot - LEFT(tp)) * theight : MINFINITY + 2;
+			(rect->r_xbot - LEFT(tp)) * theight : DLONG_MIN;
 		if (SplitDirection(tp) ? (f1 > f4) : (f2 > f4))
 		{
 		    TiSetBody(tp, (ClientData)((TileType)TiGetBody(tp)
@@ -593,7 +596,7 @@ enumerate:
 	    {
 		/* !Outside-of-triangle check */
 		f3 = (rect->r_xtop < INFINITY - 2) ?
-			(RIGHT(tp) - rect->r_xtop) * theight : MINFINITY + 2;
+			(RIGHT(tp) - rect->r_xtop) * theight : DLONG_MIN;
 		if (SplitDirection(tp) ? (f2 > f3) : (f1 > f3))
 		{
 		    TiSetBody(tp, (ClientData)((TileType)TiGetBody(tp)
