@@ -17,7 +17,7 @@
  *
  * Needs to include: magic.h, tile.h, database.h
  *
- * rcsid $Header: /ufs/repository/magic/database/databaseInt.h,v 1.4 2001/04/16 14:31:40 tim Exp $
+ * rcsid $Header: /usr/cvsroot/magic-7.5/database/databaseInt.h,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $
  */
 
 #ifndef _DATABASEINT_H
@@ -148,7 +148,7 @@ typedef struct
     TileTypeBitMask	 l_residues;
 
     /* Mask of connected planes, including this one */
-    int			 l_pmask;
+    PlaneMask		 l_pmask;
 } LayerInfo;
 
 extern LayerInfo dbLayerInfo[];
@@ -157,7 +157,7 @@ extern int dbNumContacts;
 extern int dbNumImages;
 
 /* Macros for above table */
-#define	PlaneMask(t)		dbLayerInfo[t].l_pmask
+#define	LayerPlaneMask(t)	dbLayerInfo[t].l_pmask
 #define	IsContact(t)		dbLayerInfo[t].l_isContact
 
 /* --------------------- Composition rule tables ---------------------- */
