@@ -13,15 +13,13 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/ext2spice/ext2spice.c,v 1.4 2001/12/04 21:44:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/ext2spice/ext2spice.c,v 1.4 2006/09/01 18:37:05 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>		/* for atof() */
-#include <ctype.h>
-#ifdef SYSV
 #include <string.h>
-#endif
+#include <ctype.h>
 
 #include "tcltk/tclmagic.h"
 #include "utils/magic.h"
@@ -247,7 +245,8 @@ Exttospice_Init(interp)
 {
     /* Sanity checks! */
     if (interp == NULL) return TCL_ERROR;
-    if (Tcl_PkgRequire(interp, "Tclmagic", "7.3", 0) == NULL) return TCL_ERROR;
+    if (Tcl_PkgRequire(interp, "Tclmagic", MAGIC_VERSION, 0) == NULL)
+	return TCL_ERROR;
     if (Tcl_InitStubs(interp, "8.1", 0) == NULL) return TCL_ERROR;
 
     TxPrintf("Auto-loading EXTTOSPICE module\n");
@@ -263,7 +262,7 @@ Exttospice_Init(interp)
     if (WindReplaceCommand(DBWclientID, "ext2spice", CmdExtToSpice) < 0)
 	return TCL_ERROR;
 
-    Tcl_PkgProvide(interp, "Exttospice", "7.3");
+    Tcl_PkgProvide(interp, "Exttospice", MAGIC_VERSION);
     return TCL_OK;
 }
 
@@ -559,7 +558,7 @@ CmdExtToSpice(w, cmd)
 	    LocCapThreshold = 2;
 	    LocResistThreshold = INFINITE_THRESHOLD;
 	    EFTrimFlags = 0;
-	    EFScale = 0;
+	    EFScale = 0.0;
 	    if (EFArgTech)
 	    {
 		freeMagic(EFArgTech);
@@ -604,7 +603,7 @@ runexttospice:
     EFCapThreshold = LocCapThreshold;
 
     /* Process command line arguments */
-
+    
     inName = EFArgs(argc, argv, &err_result, spcmainArgs, (ClientData) NULL);
     if (err_result == TRUE)
     {
@@ -717,8 +716,8 @@ runexttospice:
 	      ( (esFormat == SPICE3) ? "SPICE3" : "HSPICE" ),  
 	    inName, EFTech);
     if ( esFormat == HSPICE ) 
-    	fprintf(esSpiceF,".option scale=%gu\n\n", ((float) EFScale)/100.0);
-    else esScale = EFScale/100.0;
+    	fprintf(esSpiceF,".option scale=%gu\n\n", EFScale / 100.0);
+    else esScale = EFScale / 100.0;
 
     /* Convert the hierarchical description to a flat one */
     flatFlags = EF_FLATNODES;
@@ -874,8 +873,8 @@ main(argc, argv)
 	      ( (esFormat == SPICE3) ? "SPICE3" : "HSPICE" ),  
 	    inName, EFTech);
     if ( esFormat == HSPICE ) 
-    	fprintf(esSpiceF,".option scale=%gu\n\n", ((float) EFScale)/100.0);
-    else esScale = EFScale/100.0;
+    	fprintf(esSpiceF,".option scale=%gu\n\n", EFScale / 100.0);
+    else esScale = EFScale / 100.0;
 
     /* Convert the hierarchical description to a flat one */
     flatFlags = EF_FLATNODES;
@@ -1600,6 +1599,20 @@ spcdevVisit(dev, hierName, trans)
     }
     fprintf(esSpiceF, "%c", devchar);
 
+    /* Flag shorted devices */
+    switch(dev->dev_class)
+    {
+	case DEV_MOSFET:
+	case DEV_FET:
+	    if (source == drain)
+		fprintf(esSpiceF, "** SOURCE/DRAIN TIED\n");
+
+	default:
+	    if (gate == source)
+		fprintf(esSpiceF, "** SHORTED DEVICE\n");
+	    break;
+    }
+
     /* Device index is taken from gate attributes if attached;	*/
     /* otherwise, the device is numbered in sequence.		*/
 
@@ -2450,13 +2463,13 @@ int nodeHspiceName(s)
     {
         snum = esSbckNum++;
 	he = HashFind(&subcktNameTable, s);
-	HashSetValue(he, (ClientData) snum);
+	HashSetValue(he, (ClientData)(pointertype) snum);
 #ifndef UNSORTED_SUBCKT
 	DQPushRear(&subcktNameQueue, he);
 #endif
     }
     else 
-	snum = (int) HashGetValue(he);
+	snum = (spointertype) HashGetValue(he);
     sprintf(map, "x%d/%s", snum, sf);
 
 topLevel:
