@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtLength.c,v 1.2 2001/01/12 22:12:37 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtLength.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -626,7 +626,8 @@ extPathPairDistance(lab1, lab2, pMin, pMax)
 {
     struct extPathArg epa;
     TileTypeBitMask mask;
-    int pMask, pNum;
+    PlaneMask pMask;
+    int pNum;
     Rect r;
 
     /* Skip if either type is space (sanity check) */
@@ -770,7 +771,8 @@ extPathFlood(tile, p, distance, epa)
 {
     TileType type = TiGetType(tile);
     Label *lab2 = epa->epa_lab2;
-    int pMask, pNum, newdistance;
+    int pNum, newdistance;
+    PlaneMask pMask;
     Tile *tp;
     char mesg[512];
     Point p2;
@@ -838,7 +840,8 @@ extPathFlood(tile, p, distance, epa)
     /* Try connections to other planes */
     if (DBIsContact(type))
     {
-	int pMask = DBConnPlanes[type];
+	int saveplane = epa->epa_pNum;
+	PlaneMask pMask = DBConnPlanes[type];
 	pMask &= ~(PlaneNumToMaskBit(epa->epa_pNum));
 
 	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
@@ -855,11 +858,11 @@ extPathFlood(tile, p, distance, epa)
 		if (tp->ti_client == (ClientData) MINFINITY
 			&& DBConnectsTo(type, TiGetType(tp)))
 		{
-		    struct extPathArg newepa = *epa;
-		    newepa.epa_pNum = pNum;
-		    extPathFlood(tp, p, distance, newepa);
+		    epa->epa_pNum = pNum;
+		    extPathFlood(tp, p, distance, epa);
 		}
 	    }
+	epa->epa_pNum = saveplane;
     }
 
     /*
@@ -873,25 +876,26 @@ extPathFlood(tile, p, distance, epa)
      */
     if (pMask = DBAllConnPlanes[type])
     {
+	int saveplane = epa->epa_pNum;
 	struct extPathFloodArg epfa;
-	struct extPathArg newepa = *epa;
 	Rect biggerArea;
 
 	TITORECT(tile, &epfa.epfa_srcArea);
 	GEO_EXPAND(&epfa.epfa_srcArea, 1, &biggerArea);
 	epfa.epfa_distance = distance;
-	epfa.epfa_epa = &newepa;
+	epfa.epfa_epa = epa;
 	epfa.epfa_srcPoint = p;
 	epfa.epfa_srcTile = tile;
 	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
 	    if (pNum != epa->epa_pNum && PlaneMaskHasPlane(pMask, pNum))
 	    {
-		newepa.epa_pNum = pNum;
+		epa->epa_pNum = pNum;
 		(void) DBSrPaintClient((Tile *) NULL,
 			extPathDef->cd_planes[pNum], &biggerArea,
 			&DBConnectTbl[type], (ClientData) MINFINITY,
 			extPathFloodFunc, (ClientData) &epfa);
 	    }
+	epa->epa_pNum = saveplane;
     }
 }
 
