@@ -23,11 +23,9 @@ static char sccsid[] = "@(#)ExtTech.c	4.8 MAGIC (Berkeley) 10/26/85";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include <math.h>
-#ifdef SYSV
-#include <string.h>
-#endif
 #include <stdlib.h>		/* for strtod() */
+#include <string.h>
+#include <math.h>
 #include <ctype.h>		/* for isspace() */
 
 #include "tcltk/tclmagic.h"
@@ -598,7 +596,7 @@ extTechStyleInit(style)
     style->exts_sideCoupleHalo = 0;
 
     style->exts_stepSize = 100;
-    style->exts_unitsPerLambda = 100;
+    style->exts_unitsPerLambda = 100.0;
     style->exts_resistScale = 1000;
     style->exts_capScale = 1000.0;
     style->exts_numResistClasses = 0;
@@ -701,7 +699,7 @@ ExtLoadStyle(stylename)
     invext = TechSectionGetMask("extract", NULL);
     TechLoad(NULL, invext);
 
-    extTechFinalStyle(ExtCurStyle);
+    /* extTechFinalStyle(ExtCurStyle); */  /* Taken care of by TechLoad() */
     ExtTechScale(DBLambda[0], DBLambda[1]);
 }
 
@@ -1349,7 +1347,7 @@ ExtTechLine(sectionName, argc, argv)
 		if (TTMaskHasType(&types1, t))
 		{
 		    he = HashFind(&ExtCurStyle->exts_transResist[t], argv[2]);
-		    HashSetValue(he, val);
+		    HashSetValue(he, (spointertype)val);
 		    if (isLinear)
 			ExtCurStyle->exts_linearResist[t] = val;
 		}
@@ -1378,7 +1376,7 @@ ExtTechLine(sectionName, argc, argv)
 	    }
 	    break;
 	case LAMBDA:
-	    ExtCurStyle->exts_unitsPerLambda = atoi(argv[1]);
+	    ExtCurStyle->exts_unitsPerLambda = (float)atof(argv[1]);
 	    break;
 	case OVERC:
 	    DBTechNoisyNameMask(argv[2], &types2);
@@ -1482,7 +1480,7 @@ ExtTechLine(sectionName, argc, argv)
 		      TechError("Must specify plane for sideoverlap to space\n");
 		 }
 		 cp++;
-		 p3 =  (int) dbTechNameLookup(cp, &dbPlaneNameLists);
+		 p3 =  (spointertype) dbTechNameLookup(cp, &dbPlaneNameLists);
 		 if (p3 < 0)
 		 {
 		      TechError("Unknown overlap plane %s\n",argv[3]);
@@ -1633,7 +1631,7 @@ ExtTechLine(sectionName, argc, argv)
 	    }
 	    break ;
 	case PLANEORDER: {
-	    int pnum = (int) dbTechNameLookup(argv[1], &dbPlaneNameLists);
+	    int pnum = (spointertype) dbTechNameLookup(argv[1], &dbPlaneNameLists);
 	    int pos = atoi(argv[2]);
 
 	    if ( ExtCurStyle->exts_planeOrderStatus == noPlaneOrder ) {
@@ -1842,10 +1840,13 @@ ExtTechScale(scalen, scaled)
     int i, j;
     float sqn, sqd;
 
+    if (style == NULL) return;
+
     sqn = (float)(scalen * scalen);
     sqd = (float)(scaled * scaled);
 
-    DBScaleValue(&style->exts_unitsPerLambda, scalen, scaled);
+    style->exts_unitsPerLambda = style->exts_unitsPerLambda * (float)scalen
+		/ (float)scaled;
     DBScaleValue(&style->exts_sideCoupleHalo, scaled, scalen);
     DBScaleValue(&style->exts_stepSize, scaled, scalen);
 
