@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/garouter/gaMaze.c,v 1.2 2001/01/12 22:12:40 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/garouter/gaMaze.c,v 1.6 2006/08/27 00:39:46 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -51,7 +51,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/garouter/gaMaze.c,v 1.2 20
 /*-------------- Global data only referenced within this file ---------------*/
 /* Parameter settings passed to mzrouter (initialized when first needed.) */
 
-MazeParameters *gaMazeParms; 
+MazeParameters *gaMazeParms = NULL;
 
 /* Top Level Cell used by mzrouter during stem generation.
  * Contains a fence, to limit the scope of the search, 
@@ -90,16 +90,10 @@ gaMazeInit(routeUse)
 {
     UndoDisable();
 
-    /* setup maze parms - if necessary */
-    if(!gaMazeParms)
-    {
-	bool gaMazeInitParms();
+    /* setup maze parms */
 
-	if(gaMazeInitParms()==FALSE)
-	{
-	    return FALSE;
-	}
-    }
+    if (GAMazeInitParms() == FALSE)
+	return FALSE;
 
     /* allocate gaMazeTop - if necessary */
     if(!gaMazeTopUse)
@@ -129,7 +123,7 @@ gaMazeInit(routeUse)
 /*
  * ----------------------------------------------------------------------------
  *
- * gaMazeInitParms -- Initialize ga maze routing parameters.
+ * GAMazeInitParms -- Initialize ga maze routing parameters.
  *
  * Called by gaMazeInit the first time it is invoked.
 
@@ -142,8 +136,13 @@ gaMazeInit(routeUse)
  * ----------------------------------------------------------------------------
  */
 bool
-gaMazeInitParms()
+GAMazeInitParms()
 {
+    if (gaMazeParms != NULL)
+    {
+	MZFreeParameters(gaMazeParms);
+	gaMazeParms = NULL;
+    }
 
     /* Initialize to copy of default garouter  parameters */
     gaMazeParms = MZCopyParms(MZFindStyle("garouter"));
@@ -155,7 +154,7 @@ gaMazeInitParms()
     }
 
     /* optimize for this application */
-    gaMazeParms->mp_expandDests = TRUE;
+    gaMazeParms->mp_expandEndpoints = TRUE;
     gaMazeParms->mp_topHintsOnly = TRUE;
     gaMazeParms->mp_bloomLimit = MAZE_TIMEOUT;
 
@@ -195,8 +194,9 @@ gaMazeRoute(routeUse, terminalLoc, pinPoint, pinLayerMask, side, writeFlag)
     NLTermLoc *terminalLoc;	/* Terminal to connect to - somewhere in
 				 * interior of cell */
     Point *pinPoint;		/* Point to connect from (on edge of cell) */
-    TileTypeBitMask pinLayerMask;	/* layer at pin (ONLY ONE BIT SHOULD
-					 * BE SET)
+    TileTypeBitMask pinLayerMask;	/* layer at pin (no more than one
+					 * bit should correspond to a known
+					 * routing layer)
 					 */
     int side;			/* side of cell destPoint lies on */
     bool writeFlag;	        /* If non-null, paint back result (into
@@ -213,6 +213,7 @@ gaMazeRoute(routeUse, terminalLoc, pinPoint, pinLayerMask, side, writeFlag)
 	gaMazeBounds(terminalLoc, pinPoint, &routeBounds);
 
 	/* Enforce route bounds with fence */
+
 	UndoDisable();
 	DBPaint(gaMazeTopDef, &routeBounds, TT_FENCE);
 	DBReComputeBbox(gaMazeTopDef);
@@ -270,7 +271,7 @@ gaMazeRoute(routeUse, terminalLoc, pinPoint, pinLayerMask, side, writeFlag)
 	RoutePath *path;
 
 	/* search for path */
-	path = MZRoute(); 
+	path = MZRoute(NULL); 	/* Note: we could make use of the result code... */
 	if(SigInterruptPending) goto abort;
 
 	/* If no path found, abort */
@@ -382,10 +383,22 @@ gaMazeBounds(terminalLoc, pinPoint, r)
 	    }
 	}
 
-	/* grow to top and right by max active width */
+	/* Grow to top and right by max active width */
+
+	/* NOTE:  Changed by Tim, 7/30/06.  1x max active width is WAY	*/
+	/* too constraining, and causes MANY routes to fail.  We only	*/
+	/* want to avoid hapless searching far outside the local space	*/
+	/* that might cause routes to be placed in the way of other	*/
+	/* routes.							*/
+
 	{
-	    r->r_xtop += width;
-	    r->r_ytop += width;
+	    /* r->r_xtop += width; */
+	    /* r->r_ytop += width; */
+
+	    r->r_xtop += (2 * width);
+	    r->r_ytop += (2 * width);
+	    r->r_xbot -= (2 * width);
+	    r->r_ybot -= (2 * width);
 	}
     }
 
