@@ -19,7 +19,7 @@
 
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/graphics/grClip.c,v 1.8 2001/10/19 15:40:10 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/graphics/grClip.c,v 1.2 2006/06/16 16:16:12 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -29,6 +29,7 @@ static char rcsid[]="$Header: /ufs/repository/magic/graphics/grClip.c,v 1.8 2001
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
+#include "utils/styles.h"
 #include "database/database.h"
 #include "windows/windows.h"
 #include "graphics/graphics.h"
@@ -46,15 +47,13 @@ extern void GrClipTriangle();
  */
 
 global Rect GrCrossRect = {-GR_CROSSSIZE, -GR_CROSSSIZE, GR_CROSSSIZE, GR_CROSSSIZE};
-
 global int GrNumClipBoxes = 0;	/* for benchmarking */
-
 global int grCurDStyle;
+global unsigned char GrGridMultiple = 1;
 
 /* A rectangle that is one square of the grid */
 static Rect *grGridRect;
 
-
 /*
  * ----------------------------------------------------------------------------
  * GrSetStuff --
@@ -183,6 +182,7 @@ bool grClipAddFunc(box, cd)
 }
 
 
+void
 grObsBox(r)
     Rect *r;
 {
@@ -206,7 +206,7 @@ grObsBox(r)
     {
 	LinkedRect *oldarea;
 	if (grCurFill == GR_STGRID)
-	    (void) (*grDrawGridPtr)(grGridRect, grCurOutline, &((*areas)->r_r));
+	    (*grDrawGridPtr)(grGridRect, grCurOutline, &((*areas)->r_r));
 	else
 	    (*grFillRectPtr)(&((*areas)->r_r));
 	oldarea = *areas;
@@ -880,10 +880,17 @@ GrBox(MagWindow *mw, Transform *trans, Tile *tile)
 
     if (grCurFill == GR_STCROSS)
     {
+	Rect rnc;
+	/* don't clip contact diagonals */
 	if (needClip || needObscure)
 	{
-	    GrClipLine(r.r_xbot, r.r_ybot, r.r_xtop, r.r_ytop);
-	    GrClipLine(r.r_xbot, r.r_ytop, r.r_xtop, r.r_ybot);
+	    WindSurfaceToScreenNoClip(mw, &r2, &rnc);
+/*
+	    (*grDrawLinePtr)(rnc.r_xbot, rnc.r_ybot, rnc.r_xtop, rnc.r_ytop);
+	    (*grDrawLinePtr)(rnc.r_xbot, rnc.r_ytop, rnc.r_xtop, rnc.r_ybot);
+*/
+	    GrClipLine(rnc.r_xbot, rnc.r_ybot, rnc.r_xtop, rnc.r_ytop);
+	    GrClipLine(rnc.r_xbot, rnc.r_ytop, rnc.r_xtop, rnc.r_ybot);
 	}
 	else
 	{
@@ -1028,7 +1035,7 @@ GrFastBox(prect)
 	else
 	{
 	    if (grCurFill == GR_STGRID)
-		(void) (*grDrawGridPtr)(grGridRect, grCurOutline, &clipr);
+		(*grDrawGridPtr)(grGridRect, grCurOutline, &clipr);
 	    else
 		(void) (*grFillRectPtr)(&clipr);
 	}
