@@ -78,7 +78,9 @@ groglDrawGrid (prect, outline, clip)
 
     xsize = prect->r_xtop - prect->r_xbot;
     ysize = prect->r_ytop - prect->r_ybot;
-    if (!xsize || !ysize || GRID_TOO_SMALL(xsize, ysize, GR_NUM_GRIDS))
+    if (!xsize || !ysize)
+	return FALSE;
+    if (GRID_TOO_SMALL(xsize, ysize, GR_NUM_GRIDS))
 	return FALSE;
     
     xstart = prect->r_xbot % xsize;
@@ -95,7 +97,6 @@ groglDrawGrid (prect, outline, clip)
     hi = clip->r_ytop;
     for (x = xstart; x < (clip->r_xtop + 1) << SUBPIXELBITS; x += xsize)
     {
-        if (snum >= GR_NUM_GRIDS) break;
 	shifted = x >> SUBPIXELBITS;
 	glVertex2i(shifted, low);
 	glVertex2i(shifted, hi);
@@ -107,7 +108,6 @@ groglDrawGrid (prect, outline, clip)
     hi = clip->r_xtop;
     for (y = ystart; y < (clip->r_ytop + 1) << SUBPIXELBITS; y += ysize)
     {
-        if (snum >= GR_NUM_GRIDS) break;
 	shifted = y >> SUBPIXELBITS;
 	glVertex2i(low, shifted);
 	glVertex2i(hi, shifted);
