@@ -16,13 +16,14 @@
  *     *********************************************************************
  *
  *
- * rcsid "$Header: /ufs/repository/magic/graphics/graphicsInt.h,v 1.3 2001/03/29 19:50:10 tim Exp $"
+ * rcsid "$Header: /usr/cvsroot/magic-7.5/graphics/graphicsInt.h,v 1.2 2006/06/16 16:16:12 tim Exp $"
  */
 
 #ifndef _GRAPHICSINT_H
 #define _GRAPHICSINT_H
 
 #include "windows/windows.h"
+#include "utils/magic.h"
 
 
 extern int grNumBitPlanes;
@@ -50,8 +51,6 @@ extern void grSimpleLock(), grSimpleUnlock();
 extern void grNoLock();
 #define	GR_CHECK_LOCK()	{if (grLockedWindow == NULL) grNoLock();} 
 
-#define GR_NUM_STIPPLES	32
-
 /* The size of the crosses drawn for degenerate box outlines: */
 #define GR_CROSSSIZE 3
 
@@ -77,9 +76,8 @@ extern char *grCursorType;
  */
 extern int grCurDStyle;
 
-/* Device-dependent limit on # of stipples (must be <= GR_NUM_STIPPLES): */
-
-extern int grMaxStipples;
+/* Number of stipples in the style file */
+extern int grNumStipples;
 
 /* Used to setup current color, etc. */
 extern bool grDriverInformed;
