@@ -23,13 +23,14 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/irouter/irCommand.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/irouter/irCommand.c,v 1.6 2006/10/21 04:36:06 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "tcltk/tclmagic.h"
 #include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
@@ -88,6 +89,8 @@ extern SubCmdTableE irSubcommands[];
  *
  *      If file is null, parameter value is written to magic text window via
  *      TxPrintf
+ *
+ *	If file is (FILE *)1, parameter value is returned as a Tcl object
  *	
  * ----------------------------------------------------------------------------
  */
@@ -201,6 +204,101 @@ irSetNoisyAutoInt(parm, valueS, file)
  * ----------------------------------------------------------------------------
  */
 
+#ifdef MAGIC_WRAPPER
+
+/* irLSetActive --  */
+Tcl_Obj *
+irLSetActive(rL,s,file)
+    RouteLayer *rL;
+    char *s;
+    FILE *file;
+{
+    if (file == (FILE *)1)
+	return Tcl_NewBooleanObj(rL->rl_routeType.rt_active);
+    SetNoisyBool(&(rL->rl_routeType.rt_active), s, file);
+    return NULL;
+}
+
+/* irLSetWidth -- */
+Tcl_Obj *
+irLSetWidth(rL,s,file)
+    RouteLayer *rL;
+    char *s;
+    FILE *file;
+{
+    if (file == (FILE *)1)
+	return Tcl_NewIntObj(rL->rl_routeType.rt_width);
+    SetNoisyInt(&(rL->rl_routeType.rt_width),s,file);
+    return NULL;
+}
+
+/* irLSetHCost -- */
+Tcl_Obj *
+irLSetHCost(rL,s,file)
+    RouteLayer *rL;
+    char *s;
+    FILE *file;
+{
+    if (file == (FILE *)1)
+	return Tcl_NewIntObj(rL->rl_hCost);
+    SetNoisyInt(&(rL->rl_hCost),s,file);
+    return NULL;
+}
+	
+/* irLSetVCost -- */
+Tcl_Obj *
+irLSetVCost(rL,s,file)
+    RouteLayer *rL;
+    char *s;
+    FILE *file;
+{
+    if (file == (FILE *)1)
+	return Tcl_NewIntObj(rL->rl_vCost);
+    SetNoisyInt(&(rL->rl_vCost),s,file);
+    return NULL;
+}
+
+/* irLSetJogCost -- */
+Tcl_Obj *
+irLSetJogCost(rL,s,file)
+    RouteLayer *rL;
+    char *s;
+    FILE *file;
+{
+    if (file == (FILE *)1)
+	return Tcl_NewIntObj(rL->rl_jogCost);
+    SetNoisyInt(&(rL->rl_jogCost),s,file);
+    return NULL;
+}
+
+/* irLSetHintCost  -- */
+Tcl_Obj *
+irLSetHintCost(rL,s,file)
+    RouteLayer *rL;
+    char *s;
+    FILE *file;
+{
+    if (file == (FILE *)1)
+	return Tcl_NewIntObj(rL->rl_hintCost);
+    SetNoisyInt(&(rL->rl_hintCost),s,file);
+    return NULL;
+}
+
+/* irLSetOverCost  -- */
+Tcl_Obj *
+irLSetOverCost(rL,s,file)
+    RouteLayer *rL;
+    char *s;
+    FILE *file;
+{
+    if (file == (FILE *)1)
+	return Tcl_NewIntObj(rL->rl_overCost);
+    SetNoisyInt(&(rL->rl_overCost),s,file);
+    return NULL;
+}
+
+#else
+
 /* irLSetActive --  */
 void
 irLSetActive(rL,s,file)
@@ -262,6 +360,18 @@ irLSetHintCost(rL,s,file)
     SetNoisyInt(&(rL->rl_hintCost),s,file);
 }
 
+/* irLSetOverCost  -- */
+void
+irLSetOverCost(rL,s,file)
+    RouteLayer *rL;
+    char *s;
+    FILE *file;
+{
+    SetNoisyInt(&(rL->rl_overCost),s,file);
+}
+
+#endif
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -281,6 +391,49 @@ irLSetHintCost(rL,s,file)
  * ----------------------------------------------------------------------------
  */
 	
+#ifdef MAGIC_WRAPPER
+
+/* irCSetActive -- */
+Tcl_Obj *
+irCSetActive(rC,s, file)
+    RouteContact *rC;
+    char *s;
+    FILE *file;
+{
+    if (file == (FILE *)1)
+	return Tcl_NewBooleanObj(rC->rc_routeType.rt_active);
+    SetNoisyBool(&(rC->rc_routeType.rt_active),s,file);
+    return NULL;
+}
+
+/* irCSetWidth --  */
+Tcl_Obj *
+irCSetWidth(rC,s,file)
+    RouteContact *rC;
+    char *s;
+    FILE *file;
+{
+    if (file == (FILE *)1)
+	return Tcl_NewIntObj(rC->rc_routeType.rt_width);
+    SetNoisyInt(&(rC->rc_routeType.rt_width),s,file);
+    return NULL;
+}
+	
+/* irCSetCost -- */
+Tcl_Obj *
+irCSetCost(rC,s,file)
+    RouteContact *rC;
+    char *s;
+    FILE *file;
+{
+    if (file == (FILE *)1)
+	return Tcl_NewIntObj(rC->rc_cost);
+    SetNoisyInt(&(rC->rc_cost),s,file);
+    return NULL;
+}
+
+#else
+
 /* irCSetActive -- */
 void
 irCSetActive(rC,s, file)
@@ -311,6 +464,9 @@ irCSetCost(rC,s,file)
     SetNoisyInt(&(rC->rc_cost),s,file);
 }
 
+
+#endif
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -395,13 +551,13 @@ irWzdSetEstimate(s,file)
     SetNoisyBool(&(irMazeParms->mp_estimate),s,file);
 }
 
-/* irWzdSetExpandDests-- */
+/* irWzdSetExpandEndpoints-- */
 void
-irWzdSetExpandDests(s, file)
+irWzdSetExpandEndpoints(s, file)
 char *s;
 FILE *file;
 {
-    SetNoisyBool(&(irMazeParms->mp_expandDests),s,file);
+    SetNoisyBool(&(irMazeParms->mp_expandEndpoints),s,file);
 }
 
 /* irWzdSetPenalty -- */
@@ -568,7 +724,11 @@ FILE *file;
 static struct
 {
     char	*cP_name;	/* name of parameter */
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj	*(*cP_proc)();	/* Procedure processing this parameter */
+#else
     void    	(*cP_proc)();	/* Procedure processing this parameter */
+#endif
 } cParms[] = {
     "active",	irCSetActive,
     "width",	irCSetWidth,
@@ -579,8 +739,8 @@ static struct
 /* NEXTVALUE - returns pointer to next value arg (string). */
 #define NEXTVALUE \
 ( \
-  (cmd->tx_argc <= 4) ? NULL : \
-    (nV_i >= cmd->tx_argc-1) ? cmd->tx_argv[nV_i=4] : cmd->tx_argv[++nV_i] \
+  (argc <= 4) ? NULL : \
+    (nV_i >= argc-1) ? cmd->tx_argv[nV_i=4] : cmd->tx_argv[++nV_i] \
 )
 
 void
@@ -591,47 +751,86 @@ irContactsCmd(w, cmd)
     TileType tileType;
     RouteContact *rC;
 
+    int argc = cmd->tx_argc;
     int which, n;
-    int nV_i = cmd->tx_argc-1; 	/* Used by NEXTVALUE, must be initialized */
+    int nV_i; 			/* Used by NEXTVALUE, must be initialized */
+    bool doList = FALSE;
+
+#ifdef MAGIC_WRAPPER
+    /* Check for "-list" option */
+    if (!strncmp(cmd->tx_argv[argc - 1], "-list", 5))
+    {
+	doList = TRUE;
+	argc--;
+    }
+#endif
+
+    nV_i = argc - 1;
     
     /* Process by case */
-    if(cmd->tx_argc == 2 ||
-	(cmd->tx_argc == 3 && (strcmp(cmd->tx_argv[2],"*")==0)) ||
-	(cmd->tx_argc >= 4 && 
+    if(argc == 2 ||
+	(argc == 3 && (strcmp(cmd->tx_argv[2],"*")==0)) ||
+	(argc >= 4 && 
 	    (strcmp(cmd->tx_argv[2],"*")==0) &&
 	    (strcmp(cmd->tx_argv[3],"*")==0)))
     {
 	/* PROCESS ALL PARMS FOR ALL CONTACT TYPES */
 
-	/* Print Contact Heading */
-	TxPrintf("%-12.12s ", "contact");
-	for(n=0; cParms[n].cP_name; n++)
+#ifdef MAGIC_WRAPPER
+	if (doList)
 	{
-	    TxPrintf("%8.8s ",cParms[n].cP_name);
-	}
-	TxPrintf("\n");
+	    Tcl_Obj *alllist, *rlist, *rname, *robj;
+	    alllist = Tcl_NewListObj(0, NULL);
 
-	TxPrintf("%-12.12s ", irRepeatChar(strlen("contact"),'-'));
-	for(n=0; cParms[n].cP_name; n++)
-	{
-	    TxPrintf("%8.8s ",irRepeatChar(strlen(cParms[n].cP_name),'-'));
+	    /* Process contact parms */
+	    for (rC = irRouteContacts; rC != NULL; rC = rC->rc_next)
+	    {
+		rlist = Tcl_NewListObj(0, NULL);
+		rname = Tcl_NewStringObj(
+			DBTypeLongNameTbl[rC->rc_routeType.rt_tileType], -1);
+		Tcl_ListObjAppendElement(magicinterp, rlist, rname);
+		for (n = 0; cParms[n].cP_name; n++)
+		{
+		    robj = (*cParms[n].cP_proc)(rC, NEXTVALUE, (FILE *)1);
+		    Tcl_ListObjAppendElement(magicinterp, rlist, robj);
+		}
+		Tcl_ListObjAppendElement(magicinterp, alllist, rlist);
+	    }
+	    Tcl_SetObjResult(magicinterp, alllist);
 	}
-	TxPrintf("\n");
-
-	/* Process contact parms */
-	for (rC=irRouteContacts; rC!= NULL; rC=rC->rc_next)
+	else
+#endif
 	{
-	    TxPrintf("%-12.12s ",
-		DBTypeLongNameTbl[rC->rc_routeType.rt_tileType]);
+	    /* Print Contact Heading */
+	    TxPrintf("%-12.12s ", "contact");
 	    for(n=0; cParms[n].cP_name; n++)
 	    {
-		(*cParms[n].cP_proc)(rC,NEXTVALUE,NULL);
+		TxPrintf("%8.8s ",cParms[n].cP_name);
 	    }
 	    TxPrintf("\n");
+
+	    TxPrintf("%-12.12s ", irRepeatChar(strlen("contact"),'-'));
+	    for(n=0; cParms[n].cP_name; n++)
+	    {
+		TxPrintf("%8.8s ",irRepeatChar(strlen(cParms[n].cP_name),'-'));
+	    }
+	    TxPrintf("\n");
+
+	    /* Process contact parms */
+	    for (rC=irRouteContacts; rC!= NULL; rC=rC->rc_next)
+	    {
+		TxPrintf("%-12.12s ",
+			DBTypeLongNameTbl[rC->rc_routeType.rt_tileType]);
+		for(n=0; cParms[n].cP_name; n++)
+		{
+		    (*cParms[n].cP_proc)(rC,NEXTVALUE,NULL);
+		}
+		TxPrintf("\n");
+	    }
 	}
     }
-    else if(cmd->tx_argc==3 ||
-	(cmd->tx_argc >= 4 && (strcmp(cmd->tx_argv[3],"*")==0)))
+    else if(argc==3 ||
+	(argc >= 4 && (strcmp(cmd->tx_argv[3],"*")==0)))
     {
 	/* PROCESS ALL PARMS ASSOCIATED WITH CONTACT */
 	/* convert layer string to tileType */
@@ -676,7 +875,7 @@ irContactsCmd(w, cmd)
 	    return;
 	}
     }
-    else if(cmd->tx_argc>=4 && strcmp(cmd->tx_argv[2],"*")==0)
+    else if(argc>=4 && strcmp(cmd->tx_argv[2],"*")==0)
     {
 	/* PROCESS A COLUMN (THE VALUES OF A PARAMETER FOR ALL CONTACTS) */
      
@@ -729,7 +928,7 @@ irContactsCmd(w, cmd)
 	    }
 	}
     }
-    else if(cmd->tx_argc>=4)
+    else if(argc>=4)
     {
 	/* PROCESS PARAMETER ASSOCIATED WITH CONTACT */
 	/* convert layer string to tileType */
@@ -786,7 +985,7 @@ irContactsCmd(w, cmd)
     }
 
     /* Give warning if number of parm values didn't come out even */
-    if (nV_i != cmd->tx_argc-1)
+    if (nV_i != argc-1)
     {
 	TxError("Warning:  Number of parameter values didn't match number of parameters.\n");
     }
@@ -893,7 +1092,11 @@ irHelpCmd(w, cmd)
 static struct
 {
     char	*lP_name;	/* name of parameter */
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj	*(*lP_proc)();	/* procedure processing this parameter */
+#else
     void	(*lP_proc)();	/* procedure processing this parameter */
+#endif
 } lParms[] = {
     "active",	irLSetActive,
     "width",	irLSetWidth,
@@ -901,14 +1104,15 @@ static struct
     "vCost",	irLSetVCost,
     "jogCost",	irLSetJogCost,
     "hintCost", irLSetHintCost,
+    "overCost", irLSetOverCost,
     0
 };
 
 /* NEXTVALUE - returns pointer to next value arg (string). */
 #define NEXTVALUE \
 ( \
-  (cmd->tx_argc <= 4) ? NULL : \
-    (nV_i >= cmd->tx_argc-1) ? cmd->tx_argv[nV_i=4] : cmd->tx_argv[++nV_i] \
+  (argc <= 4) ? NULL : \
+    (nV_i >= argc-1) ? cmd->tx_argv[nV_i=4] : cmd->tx_argv[++nV_i] \
 )
 
     /*ARGSUSED*/	
@@ -919,49 +1123,87 @@ irLayersCmd(w, cmd)
 {
     TileType tileType;
     RouteLayer *rL;
+    bool doList = FALSE;
 
+    int argc = cmd->tx_argc;
     int which, n;
-    int nV_i = cmd->tx_argc-1; 	/* Used by NEXTVALUE, must be initialized */
+    int nV_i;		/* Used by NEXTVALUE, must be initialized */
+
+#ifdef MAGIC_WRAPPER
+    /* Check for "-list" option */
+    if (!strncmp(cmd->tx_argv[argc - 1], "-list", 5))
+    {
+	doList = TRUE;
+	argc--;
+    }
+#endif
+
+    nV_i = argc - 1;
     
     /* Process by case */
-    if(cmd->tx_argc == 2 ||
-	(cmd->tx_argc == 3 && (strcmp(cmd->tx_argv[2],"*")==0)) ||
-	(cmd->tx_argc >= 4 && 
+    if(argc == 2 ||
+	(argc == 3 && (strcmp(cmd->tx_argv[2],"*")==0)) ||
+	(argc >= 4 && 
 	    (strcmp(cmd->tx_argv[2],"*")==0) &&
 	    (strcmp(cmd->tx_argv[3],"*")==0)))
     {
 	/* PROCESS ALL PARMS FOR ALL ROUTE LAYERS */
 
-	/* Print Route Layer Heading */
-	TxPrintf("%-12.12s ", "layer");
-	for(n=0; lParms[n].lP_name; n++)
+#ifdef MAGIC_WRAPPER
+	if (doList)
 	{
-	    TxPrintf("%8.8s ",lParms[n].lP_name);
-	}
-	TxPrintf("\n");
+	    Tcl_Obj *alllist, *rlist, *robj, *rname;
+	    alllist = Tcl_NewListObj(0, NULL);
 
-	TxPrintf("%-12.12s ", irRepeatChar(strlen("layer"),'-'));
-	for(n=0; lParms[n].lP_name; n++)
-	{
-	    TxPrintf("%8.8s ", irRepeatChar(strlen(lParms[n].lP_name),'-'));
+	    /* Process parms for each route layer */
+	    for (rL = irRouteLayers; rL != NULL; rL = rL->rl_next)
+	    {
+		rlist = Tcl_NewListObj(0, NULL);
+		rname = Tcl_NewStringObj(
+			DBTypeLongNameTbl[rL->rl_routeType.rt_tileType], -1);
+		Tcl_ListObjAppendElement(magicinterp, rlist, rname);
+		for (n = 0; lParms[n].lP_name; n++)
+		{
+		    robj = (*lParms[n].lP_proc)(rL, NEXTVALUE, (FILE *)1);
+		    Tcl_ListObjAppendElement(magicinterp, rlist, robj);
+		}
+	        Tcl_ListObjAppendElement(magicinterp, alllist, rlist);
+	    }
+	    Tcl_SetObjResult(magicinterp, alllist);
 	}
-	TxPrintf("\n");
-
-	/* Process parms for each route layer */
-	for (rL=irRouteLayers; rL!= NULL; rL=rL->rl_next)
+	else
+#endif
 	{
-	    TxPrintf("%-12.12s ", 
-		DBTypeLongNameTbl[rL->rl_routeType.rt_tileType]);
+	    /* Print Route Layer Heading */
+	    TxPrintf("%-12.12s ", "layer");
 	    for(n=0; lParms[n].lP_name; n++)
 	    {
-		(*lParms[n].lP_proc)(rL,NEXTVALUE,NULL);
+		TxPrintf("%8.8s ",lParms[n].lP_name);
 	    }
 	    TxPrintf("\n");
-	}
 
+	    TxPrintf("%-12.12s ", irRepeatChar(strlen("layer"),'-'));
+	    for(n=0; lParms[n].lP_name; n++)
+	    {
+		TxPrintf("%8.8s ", irRepeatChar(strlen(lParms[n].lP_name),'-'));
+	    }
+	    TxPrintf("\n");
+
+	    /* Process parms for each route layer */
+	    for (rL=irRouteLayers; rL!= NULL; rL=rL->rl_next)
+	    {
+		TxPrintf("%-12.12s ", 
+			DBTypeLongNameTbl[rL->rl_routeType.rt_tileType]);
+		for(n=0; lParms[n].lP_name; n++)
+		{
+		    (*lParms[n].lP_proc)(rL,NEXTVALUE,NULL);
+		}
+		TxPrintf("\n");
+	    }
+	}
     }
-    else if(cmd->tx_argc==3 ||
-	(cmd->tx_argc >= 4 && (strcmp(cmd->tx_argv[3],"*")==0)))
+    else if(argc==3 ||
+	(argc >= 4 && (strcmp(cmd->tx_argv[3],"*")==0)))
     {
 	/* PROCESS ALL PARMS ASSOCIATED WITH ROUTE LAYER */
 	/* convert layer string to tileType */
@@ -1006,7 +1248,7 @@ irLayersCmd(w, cmd)
 	    return;
 	}
     }
-    else if(cmd->tx_argc>=4 && strcmp(cmd->tx_argv[2],"*")==0)
+    else if(argc>=4 && strcmp(cmd->tx_argv[2],"*")==0)
     {
 	/* PROCESS A COLUMN (THE VALUES OF A PARAMETER FOR ALL LAYERS) */
      
@@ -1061,7 +1303,7 @@ irLayersCmd(w, cmd)
 
 	}
     }
-    else if(cmd->tx_argc>=4)
+    else if(argc>=4)
     {
 	/* PROCESS PARAMETER ASSOCIATED WITH ROUTE LAYER */
 	/* convert layer string to tileType */
@@ -1117,7 +1359,7 @@ irLayersCmd(w, cmd)
     }
 
     /* Give warning if number of parm values didn't come out even */
-    if (nV_i != cmd->tx_argc-1)
+    if (nV_i != argc-1)
     {
 	TxError("Warning:  Number of parameter values didn't match number of parameters.\n");
     }
@@ -1153,6 +1395,9 @@ static char* rOptions[] = {
     "-slabel",	/* 6 */
     "-slayers", /* 7 */
     "-spoint",	/* 8 */
+#ifdef MAGIC_WRAPPER
+    "-timeout",	/* 9 */
+#endif
     NULL
 };
 
@@ -1171,6 +1416,7 @@ irRouteCmd(w, cmd)
     int destType;		/* type of destination specifier */
     Point startPt;
     Rect destRect;
+    int irResult = MZ_NO_ACTION;
     int i;			/* index of arg being processed */
     int argc = cmd->tx_argc;
     char **argv = cmd->tx_argv;
@@ -1243,48 +1489,48 @@ irRouteCmd(w, cmd)
 		    TxError("Incomplete coordinates.\n");
 		    goto leaveClean;
 		}
-		if(!StrIsInt(argv[i]))
+		if(!StrIsNumeric(argv[i]))
 		{
-		    TxError("Noninteger coordinate: %s\n",argv[i]);
+		    TxError("Non coordinate: %s\n",argv[i]);
 		    goto leaveClean;
 		}
-		destRect.r_xbot = atoi(argv[i]);
+		destRect.r_xbot = cmdParseCoord(w, argv[i], FALSE, TRUE);
 
 		if(++i>=argc)
 		{
 		    TxError("Incomplete coordinates.\n");
 		    goto leaveClean;
 		}
-		if(!StrIsInt(argv[i]))
+		if(!StrIsNumeric(argv[i]))
 		{
-		TxError("Noninteger coordinate: %s\n",argv[i]);
+		TxError("Nonnumeric coordinate: %s\n",argv[i]);
 		goto leaveClean;
 		}
-		destRect.r_ybot = atoi(argv[i]);
+		destRect.r_ybot = cmdParseCoord(w, argv[i], FALSE, FALSE);
 
 		if(++i>=argc)
 		{
 		    TxError("Incomplete coordinates.\n");
 		    goto leaveClean;
 		}
-		if(!StrIsInt(argv[i]))
+		if(!StrIsNumeric(argv[i]))
 		{
-		    TxError("Noninteger coordinate: %s\n",argv[i]);
+		    TxError("Nonnumeric coordinate: %s\n",argv[i]);
 		    goto leaveClean;
 		}
-		destRect.r_xtop = atoi(argv[i]);
+		destRect.r_xtop = cmdParseCoord(w, argv[i], FALSE, TRUE);
 
 		if(++i>=argc)
 		{
 		    TxError("Incomplete coordinates.\n");
 		    goto leaveClean;
 		}
-		if(!StrIsInt(argv[i]))
+		if(!StrIsNumeric(argv[i]))
 		{
-		TxError("Noninteger coordinate: %s\n",argv[i]);
+		TxError("Nonnumeric coordinate: %s\n",argv[i]);
 		goto leaveClean;
 		}
-		destRect.r_ytop = atoi(argv[i]);
+		destRect.r_ytop = cmdParseCoord(w, argv[i], FALSE, FALSE);
 
 		destRectArg = &destRect;
 		break;	    
@@ -1337,26 +1583,43 @@ irRouteCmd(w, cmd)
 		    TxError("Incomplete coordinates.\n");
 		    goto leaveClean;
 		}
-		if(!StrIsInt(argv[i]))
+		if(!StrIsNumeric(argv[i]))
 		{
-		    TxError("Noninteger coordinate: %s\n",argv[i]);
+		    TxError("Nonnumeric coordinate: %s\n",argv[i]);
 		    goto leaveClean;
 		}
-		startPt.p_x = atoi(argv[i]);
+		startPt.p_x = cmdParseCoord(w, argv[i], FALSE, TRUE);
 		if(++i>=argc)
 		{
 		    TxError("Incomplete coordinates.\n");
 		    goto leaveClean;
 		}
-		if(!StrIsInt(argv[i]))
+		if(!StrIsNumeric(argv[i]))
 		{
-		TxError("Noninteger coordinate: %s\n",argv[i]);
+		TxError("Nonnumeric coordinate: %s\n",argv[i]);
 		goto leaveClean;
 		}
-		startPt.p_y = atoi(argv[i]);
+		startPt.p_y = cmdParseCoord(w, argv[i], FALSE, FALSE);
 
 		startPtArg = &startPt;
 		break;
+#ifdef MAGIC_WRAPPER
+	    case 9:
+		if(++i>=argc)
+		{
+		    TxError("No timeout value given.\n");
+		    goto leaveClean;
+		}
+		if(!StrIsInt(argv[i]))
+		{
+		    TxError("Noninteger timeout value: %s\n",argv[i]);
+		    goto leaveClean;
+		}
+		SigRemoveTimer();
+		SigTimerInterrupts();
+		SigSetTimer(atoi(argv[i]));
+		break;
+#endif
 	    default:
 		/* shouldn't happen */
 		ASSERT(FALSE,"irRouteCmd");
@@ -1368,8 +1631,35 @@ irRouteCmd(w, cmd)
     }
 
     /* We're done parsing the command, call irRoute to do the real work */
-    irRoute(w,startType,startPtArg,startLabel,startLayers,
-	    destType,destRectArg,destLabel,destLayers);
+    irResult = irRoute(w, startType, startPtArg, startLabel, startLayers,
+			destType, destRectArg, destLabel, destLayers);
+
+#ifdef MAGIC_WRAPPER
+    SigTimerDisplay();
+
+    /* Set Tcl Result to irResult */
+    switch (irResult)
+    {
+	case MZ_SUCCESS:
+	    Tcl_SetResult(magicinterp, "Route success", 0);
+	    break;
+	case MZ_CURRENT_BEST:
+	    Tcl_SetResult(magicinterp, "Route best before interrupt", 0);
+	    break;
+	case MZ_FAILURE:
+	    Tcl_SetResult(magicinterp, "Route failure", 0);
+	    break;
+	case MZ_UNROUTABLE:
+	    Tcl_SetResult(magicinterp, "Route unroutable", 0);
+	    break;
+	case MZ_INTERRUPTED:
+	    Tcl_SetResult(magicinterp, "Route interrupted", 0);
+	    break;
+	case MZ_ALREADY_ROUTED:
+	    Tcl_SetResult(magicinterp, "Route already routed", 0);
+	    break;
+    }
+#endif
 
 leaveClean:
     ListDealloc(startLayers);
@@ -1700,9 +1990,9 @@ irSpacingsCmd(w, cmd)
 
 	    /* convert value-string to integer */
 	    s = cmd->tx_argv[argI+1];
-	    if (StrIsInt(s))
+	    if (StrIsNumeric(s))
 	    {
-		value = atoi(s);
+		value = cmdParseCoord(w, s, TRUE, FALSE);
 		if (value < -1)
 		{
 		    TxError("\nBad spacing value: %d\n",value);
@@ -1894,7 +2184,7 @@ static struct
     "bloom",		irWzdSetBloom,
     "boundsIncrement",	irWzdSetBoundsIncrement,
     "estimate",		irWzdSetEstimate,
-    "expandDests",	irWzdSetExpandDests,
+    "expandEndpoints",	irWzdSetExpandEndpoints,
     "penalty",		irWzdSetPenalty,
     "penetration",	irWzdSetPenetration,
     "window",		irWzdSetWindow,
@@ -2248,19 +2538,41 @@ IRCommand(w, cmd)
 
     if(cmd->tx_argc == 1)
     {
+	int irResult;
+
 	/* No subcommand specified - so just route from cursor to box
 	 * No endpts or layers explicitly given on cmd, so args are
 	 * NULL below.
 	 */
-	irRoute(w,
-		ST_CURSOR,
-		(Point*)NULL,
-		(char*)NULL,
-		(List*)NULL,     
-		DT_BOX,
-		(Point*)NULL,
-		(char*)NULL,
-		(List*)NULL);
+	irResult = irRoute(w, ST_CURSOR,
+		(Point*)NULL, (char*)NULL, (List*)NULL,     
+		DT_BOX, (Point*)NULL, (char*)NULL, (List*)NULL);
+
+#ifdef MAGIC_WRAPPER
+	/* Set Tcl Result to irResult */
+	switch (irResult)
+	{
+	    case MZ_SUCCESS:
+		Tcl_SetResult(magicinterp, "Route success", 0);
+		break;
+	    case MZ_CURRENT_BEST:
+		Tcl_SetResult(magicinterp, "Route best before interrupt", 0);
+		break;
+	    case MZ_FAILURE:
+		Tcl_SetResult(magicinterp, "Route failure", 0);
+		break;
+	    case MZ_UNROUTABLE:
+		Tcl_SetResult(magicinterp, "Route unroutable", 0);
+		break;
+	    case MZ_INTERRUPTED:
+		Tcl_SetResult(magicinterp, "Route interrupted", 0);
+		break;
+	    case MZ_ALREADY_ROUTED:
+		Tcl_SetResult(magicinterp, "Route already routed", 0);
+		break;
+	}
+#endif
+
     }
     else
     {
