@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/irouter/irRoute.c,v 1.2 2001/01/12 22:12:53 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/irouter/irRoute.c,v 1.7 2006/09/24 23:35:58 tim Exp $";
 #endif  /* not lint */
 
 /* --- Includes --- */
@@ -61,6 +61,7 @@ typedef struct labelSearchData
 {
     Rect 	lsd_locRect;	/* set to labels location by filter func */
     char	*lsd_name;	/* label name to search for */
+    TileType	lsd_type;	/* layer that label is attached to */
     int 	lsd_result;	/* code giving result of search */
 } LabelSearchData;
 
@@ -79,7 +80,7 @@ typedef struct labelSearchData
  * calls the maze router to make a connection.
  *
  * Results:
- *	None.
+ *	Passes back the result code of MZRoute() (see mzrouter.h for codes)
  *
  * Side effects:
  *	Paint route into editcell.
@@ -87,7 +88,7 @@ typedef struct labelSearchData
  * ----------------------------------------------------------------------------
  */
 
-void
+int
 irRoute(cmdWindow, startType, argStartPt, argStartLabel, argStartLayers, 
 		destType, argDestRect, argDestLabel, argDestLayers)
     MagWindow *cmdWindow;	/* window route command issued to */
@@ -108,6 +109,8 @@ irRoute(cmdWindow, startType, argStartPt, argStartLabel, argStartLayers,
     Rect destRect;
     List *destLayers = NULL;
     RoutePath *path = NULL;	/* resulting path */
+    TileType startLayer = TT_SPACE;
+    int mzResult = MZ_NO_ACTION;
 
     /* determine routeUse and expansionMask for this route. */
     {
@@ -134,7 +137,7 @@ irRoute(cmdWindow, startType, argStartPt, argStartLabel, argStartLayers,
 	if(window==NULL)
 	{
 	    TxError("Point to a layout window first.\n");
-	    return;
+	    return mzResult;
 	}
 
 	/* Set expansion mask to window route cmd issued to.  Used 
@@ -160,7 +163,7 @@ irRoute(cmdWindow, startType, argStartPt, argStartLabel, argStartLayers,
 	if (!EditCellUse || EditRootDef != routeUse->cu_def)
 	{
 	    TxError("Nothing being edited in route window.\n");
-	    return;
+	    return mzResult;
 	}
     }
 
@@ -174,6 +177,7 @@ irRoute(cmdWindow, startType, argStartPt, argStartLabel, argStartLayers,
 	startPt = irGetStartPoint(startType, 
 				  argStartPt, 
 				  argStartLabel,
+				  &startLayer,
 				  routeUse);
 
 	/* check for failure */
@@ -194,7 +198,7 @@ irRoute(cmdWindow, startType, argStartPt, argStartLabel, argStartLayers,
 	{
 	    RouteLayer *rL;
 		
-	    for(rL=irRouteLayers; rL!=NULL; rL=rL->rl_next)
+	    for (rL = irRouteLayers; rL != NULL; rL = rL->rl_next)
 	    {
 		if(rL->rl_routeType.rt_active)
 		{
@@ -240,10 +244,12 @@ irRoute(cmdWindow, startType, argStartPt, argStartLabel, argStartLayers,
     /* dest is defined by rectangle */
     {
 	Rect irGetDestRect();
+	TileType destLayer = TT_SPACE;
 
 	destRect = irGetDestRect(destType, 
 				 argDestRect, 
 				 argDestLabel,
+				 &destLayer,
 				 routeUse);
 
 	/* check for failure */
@@ -252,12 +258,29 @@ irRoute(cmdWindow, startType, argStartPt, argStartLabel, argStartLayers,
 	/* set a maze router dest area for extent of destRect
 	 * on each permitted dest layer.
 	 */
-	if(argDestLayers == NULL)
-	/* no layer arg specified, permit all active route layers */
+	if (destLayer != TT_SPACE)
 	{
 	    RouteLayer *rL;
+
+	    /* layer type returned by irGetDestRect */
+
+	    for(rL = irRouteLayers; rL != NULL; rL = rL->rl_next)
+	    {
+		if (rL->rl_routeType.rt_active &&
+			TTMaskHasType(&(DBConnectTbl[destLayer]),
+                                   rL->rl_routeType.rt_tileType))
+		{
+		    MZAddDest(&destRect, rL->rl_routeType.rt_tileType);
+		    break;
+		}
+	    }
+	}
+	else if (argDestLayers == NULL)
+	{
+	    /* no layer arg specified, permit all active route layers */
+	    RouteLayer *rL;
 	    
-	    for(rL=irRouteLayers; rL!=NULL; rL=rL->rl_next)
+	    for (rL = irRouteLayers; rL != NULL; rL = rL->rl_next)
 	    {
 		if(rL->rl_routeType.rt_active)
 		{
@@ -283,8 +306,30 @@ irRoute(cmdWindow, startType, argStartPt, argStartLabel, argStartLayers,
 	}	    
     }
 
-    /* Determine OK start layers */
+    if (startLayer != TT_SPACE)
     {
+	RouteLayer *rL;
+
+	/* layer type returned by irGetStartPoint */
+
+	for (rL = irRouteLayers; rL != NULL; rL = rL->rl_next)
+	{
+	    if (rL->rl_routeType.rt_active &&
+			TTMaskHasType(&(DBConnectTbl[startLayer]),
+                                   rL->rl_routeType.rt_tileType))
+
+	    {
+		MZAddStart(&startPt, rL->rl_routeType.rt_tileType);
+		break;
+	    }
+	}
+    }
+    else
+    {
+	/* Determine OK start layers */
+
+	List *l;
+
 	startLayers = irChooseEndPtLayers(
 					  routeUse, 
 					  expansionMask, 
@@ -298,11 +343,8 @@ irRoute(cmdWindow, startType, argStartPt, argStartLabel, argStartLayers,
 	    TxPrintf("----- startLayers:\n");
 	    MZPrintRLListNames(startLayers);
 	}
-    }
 
-    /* Set maze router start point(s) - one for each ok layer */
-    {
-	List *l;
+	/* Set maze router start point(s) - one for each ok layer */
 
 	for(l=startLayers; l!=NULL;l=LIST_TAIL(l))
 	{
@@ -314,7 +356,8 @@ irRoute(cmdWindow, startType, argStartPt, argStartLabel, argStartLayers,
     }
 
     /* Do the Route */
-    path = MZRoute(); /* If MZRoute is interrupted it returns best path
+    path = MZRoute(&mzResult);
+		      /* If MZRoute is interrupted it returns best path
 		       * found so far.
 		       */
 
@@ -408,6 +451,7 @@ irRoute(cmdWindow, startType, argStartPt, argStartLabel, argStartLayers,
 	if(SigInterruptPending) goto abort;
 
 	TxPrintf("Done Routing.\n");
+	TxFlushOut();
     }
     else
     {
@@ -429,7 +473,7 @@ abort:
 	MZClean();
     }
 
-    return;
+    return mzResult;
 }
 
 
@@ -444,17 +488,18 @@ abort:
  *	Returns point.
  *
  * Side effects:
- *	None.
+ *	Returns start layer type in startLayerPtr.
  *
  * ---------------------------------------------------------------------
  */
 
 Point
-irGetStartPoint(startType, argStartPt, argStartLabel, routeUse)
+irGetStartPoint(startType, argStartPt, argStartLabel, startLayerPtr, routeUse)
     int startType;		/* how start is specified */
     Point *argStartPt;		/* location to route from 
 				 * (in edit cell coords) */
     char *argStartLabel;	/* label to route from */
+    TileType *startLayerPtr;	/* layer type (returned value) */
     CellUse *routeUse;		/* toplevel cell visible to router */
 {
     Point startPt;
@@ -513,10 +558,7 @@ irGetStartPoint(startType, argStartPt, argStartLabel, routeUse)
 
 	    if (lSD.lsd_result == LSR_NOTUNIQUE)
 	    {
-		TxError("Start label not unique:  ");
-		TxError("more than one '%s' label selected!\n", 
-			argStartLabel);
-		goto abort;
+		TxError("Warning: Start label '%s' not unique.\n", argStartLabel);
 	    }
 	    else if (lSD.lsd_result == LSR_NOTFOUND)
 	    /* No selected label matched, so search all labels */
@@ -531,9 +573,7 @@ irGetStartPoint(startType, argStartPt, argStartLabel, routeUse)
 	
 		if (lSD.lsd_result == LSR_NOTUNIQUE)
 		{
-		    TxError("Start label '%s' not unique:  select one.\n",
-			    argStartLabel);
-		    goto abort;
+		    TxError("Warning: Start label '%s' not unique.\n", argStartLabel);
 		}
 		else if (lSD.lsd_result == LSR_NOTFOUND)
 		{
@@ -544,6 +584,7 @@ irGetStartPoint(startType, argStartPt, argStartLabel, routeUse)
 	    }
 
 	    startPt = lSD.lsd_locRect.r_ll;
+	    if (startLayerPtr) *startLayerPtr = lSD.lsd_type;
 	}
 	break;
 
@@ -575,17 +616,18 @@ abort:
  *	Returns rect.
  *
  * Side effects:
- *	None.
+ *	Returns layer type in destLayerPtr.
  *
  * ---------------------------------------------------------------------
  */
 
 Rect
-irGetDestRect(destType, argDestRect, argDestLabel, routeUse)
+irGetDestRect(destType, argDestRect, argDestLabel, destLayerPtr, routeUse)
     int destType;		/* how dest is specified */
     Rect *argDestRect;		/* location to route to 
 				 * (in edit cell coords) */
     char *argDestLabel;		/* label to route to */
+    TileType *destLayerPtr;	/* layer type (returned value) */
     CellUse *routeUse;		/* toplevel cell visible to router */
 {
     Rect destRect;
@@ -619,9 +661,7 @@ irGetDestRect(destType, argDestRect, argDestLabel, routeUse)
 
 	    if (lSD.lsd_result == LSR_NOTUNIQUE)
 	    {
-		TxError("Destination label not unique:  ");
-		TxError("more than one '%s' label selected!\n", argDestLabel);
-		goto abort;
+		TxError("Warning: Destination label '%s' not unique.\n", argDestLabel);
 	    }
 	    else if (lSD.lsd_result == LSR_NOTFOUND)
 	    {
@@ -634,10 +674,8 @@ irGetDestRect(destType, argDestRect, argDestLabel, routeUse)
 
 		if (lSD.lsd_result == LSR_NOTUNIQUE)
 		{
-		    TxError("Destination label '%s' not unique:  ",
-			    argDestLabel);
-		    TxError("select one.\n");
-		    goto abort;
+		    TxError("Warning: Destination label '%s' not unique.\n",
+				argDestLabel);
 		}
 		else if (lSD.lsd_result == LSR_NOTFOUND)
 		{
@@ -648,6 +686,7 @@ irGetDestRect(destType, argDestRect, argDestLabel, routeUse)
 	    }
 
 	    destRect = lSD.lsd_locRect;
+	    if (destLayerPtr) *destLayerPtr = lSD.lsd_type;
 	}
 	break;
 
@@ -710,30 +749,33 @@ abort:
  */
 
 int
-irSelLabelsFunc(label, cellDef, transform, lsd)
+irSelLabelsFunc(label, cellUse, transform, clientData)
     Label *label;
-    CellDef *cellDef;
+    CellUse *cellUse;
     Transform *transform;
-    ClientData lsd;
+    ClientData clientData;
 {
-    if (strcmp(((LabelSearchData *)lsd)->lsd_name, label->lab_text)!=0)
+    LabelSearchData *lsd = (LabelSearchData *)clientData;
+    CellDef *cellDef = cellUse->cu_def;
+
+    if (strcmp(lsd->lsd_name, label->lab_text) != 0)
     {
 	/* this label doesn't match, continue search */
 	return 0;
     }
-    else if (((LabelSearchData *)lsd)->lsd_result == LSR_FOUND)
+    else if (lsd->lsd_result == LSR_FOUND)
     {
 	/* second match, set result and terminate search */
-	((LabelSearchData *)lsd)->lsd_result = LSR_NOTUNIQUE;
+	lsd->lsd_result = LSR_NOTUNIQUE;
 	return 1;
     }
     else
     {
 	/* first match, set location, result, and continue search */
 	GeoTransRect(transform,
-		&(label->lab_rect),
-		&(((LabelSearchData *)lsd)->lsd_locRect));
-	((LabelSearchData *)lsd)->lsd_result = LSR_FOUND;
+		&(label->lab_rect), &(lsd->lsd_locRect));
+	lsd->lsd_result = LSR_FOUND;
+	lsd->lsd_type = label->lab_type;
 	return 0;
     }
 }
@@ -757,23 +799,28 @@ irSelLabelsFunc(label, cellDef, transform, lsd)
  */
 
 int
-irAllLabelsFunc(rect, name, label, lsd)
+irAllLabelsFunc(rect, name, label, clientData)
     Rect *rect;
     char *name;
     Label *label;
-    ClientData lsd;
+    ClientData clientData;
 {
-    if (((LabelSearchData *)lsd)->lsd_result == LSR_FOUND)
+    LabelSearchData *lsd = (LabelSearchData *)clientData;
+
+    if (lsd->lsd_result == LSR_FOUND)
     {
+	if (GEO_SAMERECT(lsd->lsd_locRect, *rect)) return 0;
+
 	/* second match, so set result and terminate search */
-	((LabelSearchData *)lsd)->lsd_result = LSR_NOTUNIQUE;
+	lsd->lsd_result = LSR_NOTUNIQUE;
 	return 1;
     }
     else
     {
 	/* first match, so set location, result, and continue search */
-	((LabelSearchData *)lsd)->lsd_locRect = *rect;
-	((LabelSearchData *)lsd)->lsd_result = LSR_FOUND;
+	lsd->lsd_locRect = *rect;
+	lsd->lsd_type = label->lab_type;
+	lsd->lsd_result = LSR_FOUND;
 	return 0;
     }
 }
