@@ -35,12 +35,12 @@
  *		+---------------------------+	^
  *		|			    |	| s
  *		|	+---------------+   |	v
- *		|	|		|   |
- *		|	|     tile	|   |
- *		|	|		|   |
+ *		|	|. . . . . . . .|   |
+ *		|	| . . tile. . .	|   |
+ *		|	|. . . . . . . .|   |
  *		|	+---------------+   |	^
  *		|			    |	|
- *		|			    |	| s+w-1
+ *		|   blockage		    |	| s+w-1
  *		|			    |	|
  *		+---------------------------+	v
  *
@@ -49,11 +49,42 @@
  *
  * There are two blockage planes for each routeType.  One is merged into
  * maximal horizontal strips, and the other
- * into maximal vertical strips.
+ * into maximal vertical strips.  (To be done---there's no reason to
+ * have separate routines for maximal vertical strips;  just use max
+ * horizontal strips routines with x and y appropriately swapped).
+ *
+ * Start and destination nodes are treated differently, by considering
+ * the areas in which a route's lower left corner can be placed that will
+ * let the route terminate without DRC errors.  This is the following:
+ *
+ *		    w-1		        w-1	
+ *		  <----->	      <---->
+ *		        +-------------+-----+	^
+ *		        |. . . . . . .|. . .|	| w-1
+ *		        | . . . . . . | . . |	| 
+ *		  +-----+. . . . . . .+-----+	v
+ *		  |	| . . . . . . . . . |
+ *		  |	|. . .tile . . . . .|
+ *		  |	| . . . . . . . . . |
+ *		  +-----+-------------+-----+	^
+ *		        | 	      |	     	|
+ *		        |	      |	     	| w-1
+ *		        +-------------+      	v
+ *
+ * The upper-right corner of the tile is a keep-out area, where a route
+ * would violate width rules.  The rest of the tile is valid for placement
+ * of a route's lower left corner, as well as two extension areas to the
+ * left and bottom.
+ *
+ * (to be done)---Both start and dest areas should not trim back the
+ * upper corners of tiles that are connected to other start/dest tiles,
+ * since that would leave unnecessary holes in the routable areas.
+ * Not doing this tends to cause troubles when the start/dest nodes are
+ * broken up into many tiles.
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/mzrouter/mzBlock.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/mzrouter/mzBlock.c,v 1.16 2006/10/21 04:36:07 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -77,8 +108,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/mzro
 #include "mzrouter/mzInternal.h"
 
 /* Forward declarations */
-extern void mzPaintBlock();
-extern void mzPaintSameNodeBlock();
+extern void mzPaintBlockType();
 
 
 /*
@@ -96,8 +126,8 @@ extern void mzPaintSameNodeBlock();
  * types of block tiles.   Multiple block types are needed to handle route 
  * termination at the destination node properly.
  *
- * TT_BLOCKED and TT_SAMENODE_BLOCK are generated from the mask data
- * (SAMENODE_BLOCK areas are only blocked by the destination node itself).
+ * TT_BLOCKED and TT_SAMENODE are generated from the mask data
+ * (SAMENODE areas are only blocked by the destination node itself).
  *
  * In addition to blockage planes for route layers, there are
  * blockage planes for contacts, assumed to connect two planes.  
@@ -119,6 +149,8 @@ mzBuildMaskDataBlocks(buildArea)
     Rect *buildArea;	/* Area over which blockage planes will be built */
 {
     Rect searchArea;
+    int pNum;
+    int mzPaintSameNodeFunc();	/* Fwd declaration */
 
     /* search area  = build area + context */
     searchArea.r_xbot = buildArea->r_xbot - mzContextRadius;
@@ -126,6 +158,16 @@ mzBuildMaskDataBlocks(buildArea)
     searchArea.r_xtop = buildArea->r_xtop + mzContextRadius;
     searchArea.r_ytop = buildArea->r_ytop + mzContextRadius;
 
+    /* Paint SAMENODE on all destination areas		 */
+    /* (all areas that were painted into mzDestAreasUse->cu_def) */
+
+    for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+    {
+	DBSrPaintArea((Tile *)NULL, mzDestAreasUse->cu_def->cd_planes[pNum],
+		&searchArea, &DBAllButSpaceAndDRCBits,
+		mzPaintSameNodeFunc, (ClientData)buildArea);
+    }
+
     /* Build the blockage planes on all layers, ignore unexpanded subcells
      * (unexpanded in reference window)
      */
@@ -153,6 +195,7 @@ mzBuildMaskDataBlocks(buildArea)
      *     while DBTreeSrCells() takes a 0 mask to mean all subcells are
      *     unexpanded.
      */
+
     if(mzCellExpansionMask != 0)
     {
 	int mzBlockSubcellsFunc();
@@ -178,7 +221,7 @@ mzBuildMaskDataBlocks(buildArea)
  *
  * Filter function called via DBTreeSrTiles on behalf of mzBuildBlock()
  * above, for each solid tile in the area of interest.  Paints TT_BLOCKED
- * (TT_SAMENODE_BLOCKED if the tile is marked) areas on each of the planes 
+ * (TT_SAMENODE if the tile is marked) areas on each of the planes 
  * affected by this tile.
  *
  * Results:
@@ -205,20 +248,7 @@ mzBuildBlockFunc(tile, cxp)
     GEOCLIP(&r, &scx->scx_area);
     GEOTRANSRECT(&scx->scx_trans, &r, &rDest);
 
-    if((int)(tile->ti_client) == MINFINITY)
-    /* tile not part of dest node, paint normal blocks onto affected
-     * planes.
-     * ( area is bloated by appropriate spacing on each affected plane)
-     */
-    {
-	mzPaintBlock(&rDest, TiGetType(tile), buildArea);
-    }
-    else
-    /* tile is part of dest node, paint samenode block areas */
-    {
-	mzPaintSameNodeBlock(&rDest, TiGetType(tile), buildArea);
-    }
-
+    mzPaintBlockType(&rDest, TiGetType(tile), buildArea, TT_BLOCKED);
     return (0);
 }
 
@@ -255,97 +285,58 @@ mzBlockSubcellsFunc(scx, cdarg)
     r = scx->scx_use->cu_def->cd_bbox;
     GEOTRANSRECT(&scx->scx_trans, &r, &rDest);
 
-    if((int)(scx->scx_use->cu_client) == MINFINITY)
+    if((int)(scx->scx_use->cu_client) != MZ_EXPAND_DEST)
     /* cell over part of dest node, paint normal blocks onto affected
      * planes.
      * (area is bloated by appropriate spacing on each affected plane)
      */
     {
-	mzPaintBlock(&rDest, TT_SUBCELL, buildArea);
+	mzPaintBlockType(&rDest, TT_SUBCELL, buildArea, TT_BLOCKED);
     }
     else
     /* cell is over part of dest node, paint samenode blocks */
     {
-	mzPaintSameNodeBlock(&rDest, TT_SUBCELL, buildArea);
+	mzPaintBlockType(&rDest, TT_SUBCELL, buildArea, TT_SAMENODE);
     }
 
     return (0);
 }
 
-
 /*
  * ----------------------------------------------------------------------------
  *
- * mzPaintBlock --
+ * mzPaintSameNodeFunc --
  *
- * Add TT_BLOCKED paint appropriate to data tile of given location
- * and type to blockage planes of all relevant routeTypes.
- *
- * The blockage tiles painted are bloated versions of the data tiles.
+ * Add TT_SAMENODE paint into the route layer block plane for each
+ * tile found in mzDestAreasUse->cu_def
  *
  * Results:
- *	None.
- *
- * Side effects:
- *	Modifies blockage planes in RouteType structures.
+ *	Always return 0 to keep the search going.
  *
  * ----------------------------------------------------------------------------
  */
 
-void
-mzPaintBlock(r, type, buildArea)
-    Rect *r;
-    TileType type;
-    Rect *buildArea;
+int
+mzPaintSameNodeFunc(Tile *t, Rect *buildArea)
 {
-    RouteType *rT;
+    Rect r;
+    TileType ttype;
 
-    /* process routeTypes  */
-    for (rT=mzActiveRTs; rT!=NULL; rT=rT->rt_nextActive)
-    {
-	Rect rblock;
+    ttype = TiGetType(t);
+    TiToRect(t, &r);
 
-	/* if there is a constraint between RouteType and type, 
-	 * paint a block 
-	 */
-	if (rT->rt_bloatBot[type]>=0)
-	{
-	    /* Compute blockage rectangle */
-	    int bot = rT->rt_bloatBot[type];
-	    int top = rT->rt_bloatTop[type];
-	    rblock.r_xbot = r->r_xbot - bot;
-	    rblock.r_ybot = r->r_ybot - bot;
-	    rblock.r_xtop = r->r_xtop + top;
-	    rblock.r_ytop = r->r_ytop + top;
+    mzPaintBlockType(&r, ttype, buildArea, TT_SAMENODE);
 
-	    /* clip to build area */
-	    GEOCLIP(&rblock, buildArea);
-
-	    /* and paint it */
-	    {
-		DBPaintPlane(rT->rt_hBlock, 
-			     &rblock, 
-			     mzBlockPaintTbl[TT_BLOCKED],
-			     (PaintUndoInfo *) NULL);
-		DBPaintPlaneVert(rT->rt_vBlock, 
-				 &rblock, 
-				 mzBlockPaintTbl[TT_BLOCKED],
-				 (PaintUndoInfo *) NULL);
-	    }
-
-	}
-    }
-
-    return;
+    return 0;
 }
 
 
 /*
  * ----------------------------------------------------------------------------
  *
- * mzPaintSameNodeBlock --
+ * mzPaintBlockType --
  *
- * Add TT_SAMENODE_BLOCK paint appropriate to data tile of given location
+ * Add "blockType" paint appropriate to data tile of given location
  * and type to blockage planes of all relevant routeTypes.
  *
  * The blockage tiles painted are bloated versions of the data tiles.
@@ -360,46 +351,152 @@ mzPaintBlock(r, type, buildArea)
  */
 
 void
-mzPaintSameNodeBlock(r, type, buildArea)
+mzPaintBlockType(r, type, buildArea, blockType)
     Rect *r;
     TileType type;
     Rect *buildArea;
+    TileType blockType;
 {
     RouteType *rT;
+    TileType locBlockType;
 
     /* process routeTypes  */
     for (rT=mzActiveRTs; rT!=NULL; rT=rT->rt_nextActive)
     {
 	Rect rblock;
 
-	/* if there is a constraint between RouteType and type, 
-	 * paint a block 
-	 */
-	if (rT->rt_bloatBot[type]>=0)
+	/* Added by Tim 7/28/06 --- Ignore those planes that don't interact! */
+
+	if (!(DBTypePlaneMaskTbl[type] & DBTypePlaneMaskTbl[rT->rt_tileType]))
+	    continue;
+
+	/* if there is a constraint between RouteType and type, paint a block */
+
+	locBlockType = blockType;
+
+	if (rT->rt_bloatBot[type] >= 0)
 	{
-	    /* Compute blockage rectangle */
-	    int bot = rT->rt_bloatBot[type];
-	    int top = rT->rt_bloatTop[type];
-	    rblock.r_xbot = r->r_xbot - bot;
-	    rblock.r_ybot = r->r_ybot - bot;
-	    rblock.r_xtop = r->r_xtop + top;
-	    rblock.r_ytop = r->r_ytop + top;
+	    int bot, top;
 
-	    /* clip to build area */
-	    GEOCLIP(&rblock, buildArea);
+	    /* If we're painting TT_BLOCKED, make sure we're not painting */
+	    /* on top of a start or dest area (has TT_SAMENODE)	  */
 
-	    /* and paint it */
+	    if (blockType != TT_SAMENODE)
 	    {
-		DBPaintPlane(rT->rt_hBlock, 
-			     &rblock, 
-			     mzBlockPaintTbl[TT_SAMENODE_BLOCK],
-			     (PaintUndoInfo *) NULL);
-		DBPaintPlaneVert(rT->rt_vBlock, 
-				 &rblock, 
-				 mzBlockPaintTbl[TT_SAMENODE_BLOCK],
-				 (PaintUndoInfo *) NULL);
+		RouteContact *rC;
+		Tile *tp = rT->rt_hBlock->pl_hint;
+		GOTOPOINT(tp, &r->r_ll);
+		if (TiGetType(tp) == TT_SAMENODE)
+		{
+		    /* This can fail to paint a proper blockage on a	*/
+		    /* contact plane when the terminal is in one plane	*/
+		    /* and the blocking route is on the other.  So, if	*/
+		    /* painting into a contact plane, check that the	*/
+		    /* tile in the residue layer that is "type" is also	*/
+		    /* covered in TT_SAMENODE, or this is a block.	*/
+
+		    if (DBIsContact(rT->rt_tileType))
+		    {
+			for (rC = mzRouteContacts; rC != NULL; rC = rC->rc_next)
+			{
+			    if (rC->rc_routeType.rt_tileType == rT->rt_tileType)
+			    {
+				Tile *tp2;
+				if (rC->rc_rLayer1->rl_routeType.rt_tileType == type)
+				{
+				    tp2 = rC->rc_rLayer1->
+						rl_routeType.rt_hBlock->pl_hint;
+				    GOTOPOINT(tp2, &r->r_ll);
+				    if (TiGetType(tp) == TT_SAMENODE) break;
+				}
+				else if (rC->rc_rLayer2->rl_routeType.rt_tileType
+						== type)
+				{
+				    tp2 = rC->rc_rLayer2->
+						rl_routeType.rt_hBlock->pl_hint;
+				    GOTOPOINT(tp2, &r->r_ll);
+				    if (TiGetType(tp) == TT_SAMENODE) break;
+				}
+			    }
+			}
+			if (rC != NULL) continue;
+		    }
+		    else
+			continue;
+		}
+	    }
+	    else
+	    {
+		/* If the tile type is a contact of the same 
+		 * type as the route type, draw a block layer, not SAMENODE.
+		 * This prevents the maze router from attempting to place
+		 * a contact too close to an existing one, causing a DRC
+		 * error.
+		 */
+		if ((DBIsContact(type)) && (rT->rt_tileType == type))
+		    locBlockType = TT_BLOCKED;
 	    }
 
+	    /* Compute blockage rectangle */
+	    bot = rT->rt_bloatBot[type];
+	    top = rT->rt_bloatTop[type];
+
+	    if (locBlockType != TT_SAMENODE)
+	    {
+		rblock.r_xbot = r->r_xbot - bot;
+		rblock.r_ybot = r->r_ybot - bot;
+
+		rblock.r_xtop = r->r_xtop + top;
+		rblock.r_ytop = r->r_ytop + top;
+	    }
+	    else
+	    {
+		int wless = bot - top + 1;
+
+		/* SAMENODE:  valid route can start in one of	*/
+		/* two overlapping rectangles as computed	*/
+		/* below.  The left and bottom sides are	*/
+		/* extended by the route width, but the		*/
+		/* corners are prohibited.			*/
+
+		rblock.r_xbot = r->r_xbot - wless;
+		rblock.r_ybot = r->r_ybot;
+
+		rblock.r_xtop = r->r_xtop;
+		rblock.r_ytop = r->r_ytop - wless;
+
+		GEOCLIP(&rblock, buildArea);
+		if (!GEO_RECTNULL(&rblock))
+		{
+		    DBPaintPlane(rT->rt_hBlock, &rblock, 
+				mzBlockPaintTbl[blockType],
+				(PaintUndoInfo *) NULL);
+		    DBPaintPlaneVert(rT->rt_vBlock, &rblock, 
+				mzBlockPaintTbl[blockType],
+				(PaintUndoInfo *) NULL);
+		}
+
+		rblock.r_xbot = r->r_xbot;
+		rblock.r_ybot = r->r_ybot - wless;
+
+		rblock.r_xtop = r->r_xtop - wless;
+		rblock.r_ytop = r->r_ytop;
+	    }
+
+	    /* clip to build area */
+
+	    GEOCLIP(&rblock, buildArea);
+	    if (!GEO_RECTNULL(&rblock))
+	    {
+		/* and paint it */
+
+		DBPaintPlane(rT->rt_hBlock, &rblock, 
+				mzBlockPaintTbl[locBlockType],
+				(PaintUndoInfo *) NULL);
+		DBPaintPlaneVert(rT->rt_vBlock, &rblock, 
+				mzBlockPaintTbl[locBlockType],
+				(PaintUndoInfo *) NULL);
+	    }
 	}
     }
 
@@ -490,12 +587,19 @@ mzBuildFenceBlocksFunc(tile, buildArea)
     RouteType *rT;
     int d;
     Rect r, rAdjusted; 
+    TileType tt = TiGetType(tile);
 
     /* Get boundary of tile */
     TITORECT(tile, &r);
 
     for (rT=mzActiveRTs; rT!=NULL; rT=rT->rt_nextActive)
     {
+	/* Added by Tim 7/28/06 --- Ignore those planes that don't interact! */
+	/* Revised 8/22/06---Fence tiles are supposed to block all layers!   */
+
+	// if (!(DBTypePlaneMaskTbl[tt] & DBTypePlaneMaskTbl[rT->rt_tileType]))
+	//    continue;
+
 	/* adjust to compensate for wire width */
 	d = rT->rt_effWidth - 1;
 	rAdjusted.r_xbot = r.r_xbot - d;
@@ -735,6 +839,10 @@ List *mzWalkList;
 void
 mzBuildDestAreaBlocks()
 {
+    int mzDestAreaFunc();
+    int mzDestWalksFunc();
+    int mzCWalksFunc();
+    SearchContext scx;
 
     /* initialize global walk list */
     mzWalkList = NULL;
@@ -742,18 +850,6 @@ mzBuildDestAreaBlocks()
     /* Compute bounding box for dest areas cell */
     DBReComputeBbox(mzDestAreasUse->cu_def);
 
-    /* Trim top and right edge of dest areas to make sure routes terminate
-     * entirely inside dest area.
-     */
-    {
-	RouteLayer *rL;
-
-	for(rL=mzActiveRLs; rL!=NULL; rL=rL->rl_nextActive)
-	{
-	    mzTrimDestAreas(rL);
-	}
-    }
-
     /* Process dest areas in dest area cell one by one.  
      *     - generates normal blockage info.
      *     - paints dest area into appropriate blockage planes.
@@ -763,29 +859,25 @@ mzBuildDestAreaBlocks()
      * Walks are not actually painted yet because existing 
      * walks can interfere with the generation of new ones.
      */
-    {
-	int mzDestAreaFunc();
-	SearchContext scx;
 
-	scx.scx_area = mzBoundingRect;
-	scx.scx_trans = GeoIdentityTransform;
-	scx.scx_use = mzDestAreasUse;
+    scx.scx_area = mzBoundingRect;
+    scx.scx_trans = GeoIdentityTransform;
+    scx.scx_use = mzDestAreasUse;
 	
-	/* clip area to bounding box to avoid overflow during transforms */
-	GEOCLIP(&(scx.scx_area),&(mzDestAreasUse->cu_def->cd_bbox));
+    /* clip area to bounding box to avoid overflow during transforms */
+    GEOCLIP(&(scx.scx_area), &(mzDestAreasUse->cu_def->cd_bbox));
 
-	/* process, one dest area at a time */
-	(void) DBTreeSrTiles(
-			     &scx, 
-			     &DBAllButSpaceAndDRCBits, 
-			     0, 
-			     mzDestAreaFunc, 
-			     (ClientData) NULL);
-    }
+    (void) DBTreeSrTiles( &scx, &DBAllButSpaceAndDRCBits, 
+			     0, mzDestAreaFunc, (ClientData) NULL);
+
+    (void) DBTreeSrTiles( &scx, &DBAllButSpaceAndDRCBits, 
+			     0, mzDestWalksFunc, (ClientData) NULL);
+
+    (void) DBTreeSrTiles(&scx, &DBAllButSpaceAndDRCBits, 
+			     0, mzCWalksFunc, (ClientData) NULL);
+
+    /* Paint walks into blockage planes (and dealloc walk list) */
 
-    /* Actually paint walks into blockage planes
-     * (and dealloc walk list)
-     */
     {
 	List *l;
 
@@ -793,6 +885,12 @@ mzBuildDestAreaBlocks()
 	{
 	    Walk *walk = (Walk *) LIST_FIRST(l);
 
+	    /* Diagnostic---something is wrong here! */
+	    if (walk->w_type > TT_MAXROUTETYPES)
+	    {
+		TxError("Fatal: Bad destination walk!\n");
+	 	continue;	/* Try to go on anyway */
+	    }
 		
 	    DBPaintPlane(walk->w_rT->rt_hBlock, 
 			 &(walk->w_rect),
@@ -807,32 +905,6 @@ mzBuildDestAreaBlocks()
 
 	ListDeallocC(mzWalkList);
     }
-
-    /* Finally generate CONTACT_WALKS for all dest areas. 
-     *
-     * CONTACT_WALK(s) are painted only directly above and below
-     * actual dest areas where contacts are possible.
-     */
-    {
-	int mzCWalksFunc();
-	SearchContext scx;
-
-	scx.scx_area = mzBoundingRect;
-	scx.scx_trans = GeoIdentityTransform;
-	scx.scx_use = mzDestAreasUse;
-	
-	/* clip area to bounding box to avoid overflow during transforms */
-	GEOCLIP(&(scx.scx_area),&(mzDestAreasUse->cu_def->cd_bbox));
-
-	/* process, one dest area at a time */
-	(void) DBTreeSrTiles(
-			     &scx, 
-			     &DBAllButSpaceAndDRCBits, 
-			     0, 
-			     mzCWalksFunc, 
-			     (ClientData) NULL);
-    }
-
     return;
 }
 
@@ -842,11 +914,12 @@ mzBuildDestAreaBlocks()
  *
  * mzDestAreaFunc --
  *
- * Pregen blockage info around dest area, and then do special dest area 
- * processing.
+ * Pregenerate blockage info around the destination area
  *
  * Results:
- *	Returns 0 always.
+ *	Returns 0 on success.  In case we are stymied by an incomplete
+ *	techfile entry for the maze router, return with 1 to stop the
+ *	whole process.
  *
  * Side effects:
  *	Generates blockage info.
@@ -868,32 +941,101 @@ mzDestAreaFunc(tile, cxp)
     TITORECT(tile, &r);
     GEOTRANSRECT(&scx->scx_trans, &r, &rect);
 
-    /* gen normal blockage info around this area */
     mzExtendBlockBoundsR(&rect);
-    
-    /* find route type for this dest area */
-    {
-	rT = mzActiveRTs;
-	while ((rT->rt_tileType != type) && (rT!=NULL))
-	{
-	    rT = rT->rt_nextActive;
-	}
-	ASSERT(rT!=NULL,"mzDestAreaFunc");
-    }
-	
-    /* paint dest area into blockage planes of appropriate route type */
-    {
-	DBPaintPlane(rT->rt_hBlock, 
-		     &rect, 
-		     mzBlockPaintTbl[TT_DEST_AREA],
-		     (PaintUndoInfo *) NULL);
-	DBPaintPlaneVert(rT->rt_vBlock, 
-			 &rect,
-			 mzBlockPaintTbl[TT_DEST_AREA],
-			 (PaintUndoInfo *) NULL);
-    }
 
-     /* Gen alignments and walks for dest areas.
+    /* find route type for this dest area */
+
+    rT = mzActiveRTs;
+    while ((rT != NULL) && (rT->rt_tileType != type))
+	rT = rT->rt_nextActive;
+
+    /* This can happen if the techfile does not completely define */
+    /* the gaRoute info. . .					  */
+    if (rT == NULL)
+	return 1;
+	
+    /* Draw the routable destination area (see comments at top of file). */
+
+    r.r_xtop = rect.r_xtop - rT->rt_width;
+    r.r_ytop = rect.r_ytop;
+    r.r_xbot = rect.r_xbot;
+    r.r_ybot = rect.r_ybot - rT->rt_width;
+
+    /* paint dest area into blockage planes of appropriate route type */
+    DBPaintPlane(rT->rt_hBlock, &r, mzBlockPaintTbl[TT_DEST_AREA],
+			(PaintUndoInfo *) NULL);
+
+    DBPaintPlaneVert(rT->rt_vBlock, &r, mzBlockPaintTbl[TT_DEST_AREA],
+			(PaintUndoInfo *) NULL);
+
+    r.r_xtop = rect.r_xtop;
+    r.r_ytop = rect.r_ytop - rT->rt_width;
+    r.r_xbot = rect.r_xbot - rT->rt_width;
+    r.r_ybot = rect.r_ybot;
+
+    /* paint dest area into blockage planes of appropriate route type */
+    DBPaintPlane(rT->rt_hBlock, &r, mzBlockPaintTbl[TT_DEST_AREA],
+			(PaintUndoInfo *) NULL);
+
+    DBPaintPlaneVert(rT->rt_vBlock, &r, mzBlockPaintTbl[TT_DEST_AREA],
+			(PaintUndoInfo *) NULL);
+
+    /* continue with next dest area */
+    return 0;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * mzDestWalksFunc --
+ *
+ * Generate walks around each destination area tile, showing the router
+ * the direct "walk" into the destination area from points around the
+ * perimeter.  This prevents the routing algorithm from having to blindly
+ * find the exact destination area;  it only has to get close enough to
+ * be told how to paint a path directly to the destination node.
+ *
+ * Results:
+ *	Returns 0 on success.  In case we are stymied by an incomplete
+ *	techfile entry for the maze router, return with 1 to stop the
+ *	whole process.
+ *
+ * Side effects:
+ *	Generates walk tiles in the blockage planes
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+mzDestWalksFunc(tile, cxp)
+    Tile *tile;
+    TreeContext *cxp;
+{
+    SearchContext *scx = cxp->tc_scx;
+    TileType type = TiGetType(tile);
+    Rect r, rect;
+    RouteType *rT;
+    int mzHWalksFunc();
+    int mzVWalksFunc();
+    int mzCWalksFunc();
+    TileTypeBitMask destAreaMask;
+
+    /* Transform to result coordinates */
+    TITORECT(tile, &r);
+    GEOTRANSRECT(&scx->scx_trans, &r, &rect);
+
+    /* find route type for this dest area */
+
+    rT = mzActiveRTs;
+    while ((rT != NULL) && (rT->rt_tileType != type))
+	rT = rT->rt_nextActive;
+
+    /* This can happen if the techfile does not completely define */
+    /* the gaRoute info. . .					  */
+    if (rT == NULL)
+	return 1;
+	
+     /* Generate alignments and walks for dest areas.
       *
       * Since the above dest area may be partially blocked, the blockage
       * planes are searched for DEST_AREA tiles underneath
@@ -903,27 +1045,23 @@ mzDestAreaFunc(tile, cxp)
       * have been calculated.
       *
       */
-    {
-	int mzHWalksFunc();
-	int mzVWalksFunc();
-	TileTypeBitMask destAreaMask;
-	
-	TTMaskSetOnlyType(&destAreaMask, TT_DEST_AREA);
 
-	DBSrPaintArea(NULL,	/* no hint tile */
-		      rT->rt_hBlock,
-		      &rect, 
-		      &destAreaMask,
-		      mzHWalksFunc, 
+    TTMaskSetOnlyType(&destAreaMask, TT_DEST_AREA);
+
+    DBSrPaintArea(NULL,	/* no hint tile */
+		      rT->rt_hBlock, &rect, 
+		      &destAreaMask, mzHWalksFunc, 
 		      (ClientData) rT);
 
-	DBSrPaintArea(NULL,	/* no hint tile */
-		      rT->rt_vBlock,
-		      &rect, 
-		      &destAreaMask,
-		      mzVWalksFunc, 
+    DBSrPaintArea(NULL,	/* no hint tile */
+		      rT->rt_vBlock, &rect, 
+		      &destAreaMask, mzVWalksFunc, 
+		      (ClientData) rT);
+
+    DBSrPaintArea(NULL,	/* no hint tile */
+		      rT->rt_hBlock, &rect, 
+		      &destAreaMask, mzCWalksFunc, 
 		      (ClientData) rT);
-    }
 
     /* continue with next dest area */
     return 0;
@@ -968,8 +1106,7 @@ mzHWalksFunc(tile, cdarg)
 	/* Build walks for blocks to left of tile */
 	while(BOTTOM(tLeft)<TOP(tile))
 	{
-
-	    if(TiGetType(tLeft)==TT_SAMENODE_BLOCK)
+	    if(TiGetType(tLeft)==TT_SAMENODE)
 	    {
 		walk = (Walk *) mallocMagic((unsigned) (sizeof(Walk)));
 		walk->w_rT = rT; 
@@ -995,7 +1132,7 @@ mzHWalksFunc(tile, cdarg)
 	/* Build walks for blocks to right of tile */
 	while(TOP(tRight)>BOTTOM(tile))
 	{
-	   if(TiGetType(tRight)==TT_SAMENODE_BLOCK)
+	   if(TiGetType(tRight)==TT_SAMENODE)
 	   {
 	       walk = (Walk *) mallocMagic((unsigned) (sizeof(Walk)));
 	       walk->w_rT = rT; 
@@ -1057,7 +1194,7 @@ mzVWalksFunc(tile, cdarg)
 	/* Build walks for blocks to below tile */
 	while(LEFT(tBelow)<RIGHT(tile))
 	{
-	    if(TiGetType(tBelow)==TT_SAMENODE_BLOCK)
+	    if(TiGetType(tBelow)==TT_SAMENODE)
 	    {
 		walk = (Walk *) mallocMagic((unsigned) (sizeof(Walk)));
 		walk->w_rT = rT;
@@ -1084,7 +1221,7 @@ mzVWalksFunc(tile, cdarg)
 	/* Build walks for blocks above tile */
 	while(RIGHT(tAbove)>LEFT(tile))
 	{
-	    if(TiGetType(tAbove)==TT_SAMENODE_BLOCK)
+	    if(TiGetType(tAbove)==TT_SAMENODE)
 	    {
 		walk = (Walk *) mallocMagic((unsigned) (sizeof(Walk)));
 		walk->w_rT = rT;
@@ -1106,84 +1243,21 @@ mzVWalksFunc(tile, cdarg)
     return(0);
 }
 
-
-/*
- * ----------------------------------------------------------------------------
- *
- * mzCWalksFunc --
- *
- * Paint contact walks.
- *
- * Results:
- *	Returns 0 always.
- *
- * Side effects:
- *	Generates blockage info.
- *
- * ----------------------------------------------------------------------------
- */
-
-int
-mzCWalksFunc(tile, cxp)
-    Tile *tile;
-    TreeContext *cxp;
-{
-    SearchContext *scx = cxp->tc_scx;
-    TileType type = TiGetType(tile);
-    Rect r, rect;
-    RouteType *rT;
-
-    /* Transform to result coordinates */
-    TITORECT(tile, &r);
-    GEOTRANSRECT(&scx->scx_trans, &r, &rect);
-    
-    /* find route type for this dest area */
-    {
-	rT = mzActiveRTs;
-	while ((rT->rt_tileType != type) && (rT!=NULL))
-	{
-	    rT = rT->rt_nextActive;
-	}
-	ASSERT(rT!=NULL,"mzDestAreaFunc");
-    }
-
-    /* Generate CONTACT_WALK(s) above and below dest areas.
-     *
-     * CONTACT_WALK(s) are painted only directly above and below
-     * actual dest areas where contacts are possible.
-     */
-    {
-	int mzCWalksFunc2();
-	TileTypeBitMask destAreaMask;
-	
-	TTMaskSetOnlyType(&destAreaMask, TT_DEST_AREA);
-
-	DBSrPaintArea(NULL,	/* no hint tile */
-		      rT->rt_hBlock,
-		      &rect, 
-		      &destAreaMask,
-		      mzCWalksFunc2, 
-		      (ClientData) rT);
-    }
-
-    /* continue with next dest area */
-    return 0;
-}
-
 /* 
- *  Struc to pass data between mzCWalksFunc2 and mzCWalksFunc3
+ *  Struc to pass data between mzCWalksFunc and mzCWalksFunc2
  */
 typedef struct walkContactFuncData
 {
     Rect		*wd_bounds;	/* dest area bounds */
     RouteLayer 		*wd_rL;		/* Route layer of walk_contact */
+    int			wd_walk;	/* TT_ABOVE_WALK or TT_BELOW_WALK */
 } WalkContactFuncData;
 
 
 /*
  * ----------------------------------------------------------------------------
  *
- * mzCWalksFunc2 --
+ * mzCWalksFunc --
  *
  * Search dest area for regions where contacts are ok, and paint
  * CONTACT_WALK(s) there in adjacent layers.
@@ -1198,60 +1272,62 @@ typedef struct walkContactFuncData
  */
 
 int
-mzCWalksFunc2(tile, cdarg)
+mzCWalksFunc(tile, cdarg)
     Tile *tile;
     ClientData cdarg;
 {
     RouteType *rT = (RouteType *) cdarg; /* RouteType of this dest area */
     RouteContact *rC;	
     Rect rect;
+    int walkType;
 
     /* set rect to boundary of this dest area */
     TITORECT(tile, &rect);
 
     /* process contact types that can connect to this dest area */
-    for(rC=mzRouteContacts; rC!=NULL; rC=rC->rc_next)
+    for (rC = mzRouteContacts; rC != NULL; rC = rC->rc_next)
     {
 	RouteLayer *rLOther = NULL; 
 
 	/* skip inactive contact types */
-	if(!(rC->rc_routeType.rt_active)) continue;
+	if (!(rC->rc_routeType.rt_active)) continue;
 
 	/* Find layer connecting to dest area FROM */
-	if(&(rC->rc_rLayer1->rl_routeType) == rT)
+	if (&(rC->rc_rLayer1->rl_routeType) == rT)
 	{
 	    rLOther = rC->rc_rLayer2;
+	    walkType = TT_ABOVE_WALK;
 	}
-	else if(&(rC->rc_rLayer2->rl_routeType) == rT)
+	else if (&(rC->rc_rLayer2->rl_routeType) == rT)
 	{
 	    rLOther = rC->rc_rLayer1;
+	    walkType = TT_BELOW_WALK;
 	}
 
 	/* If current contact type (RC) permits connection to dest area, 
 	 * insert CONTACT_WALK(s) into blockage planes for other layer.
 	 */
-	if(rLOther)
+	if (rLOther)
 	{
 	    /* process contact-OK tiles in blockage plane for this contact */
-	    {
-		int mzCWalksFunc3();
-		WalkContactFuncData wD;
-		TileTypeBitMask contactOKMask;
+	    int mzCWalksFunc2();
+	    WalkContactFuncData wD;
+	    TileTypeBitMask contactOKMask;
 
-		TTMaskSetOnlyType(&contactOKMask, TT_SPACE);
-		TTMaskSetType(&contactOKMask, TT_SAMENODE_BLOCK);
+	    TTMaskSetOnlyType(&contactOKMask, TT_SPACE);
+	    TTMaskSetType(&contactOKMask, TT_SAMENODE);
 
-		wD.wd_bounds = &rect; 
-		wD.wd_rL = rLOther;
+	    wD.wd_bounds = &rect; 
+	    wD.wd_rL = rLOther;
+	    wD.wd_walk = walkType;
 
-		/* search for OK places for contact, and paint them */
-		DBSrPaintArea(NULL,	/* no hint tile */
+	    /* search for OK places for contact, and paint them */
+	    DBSrPaintArea(NULL,	/* no hint tile */
 			      rC->rc_routeType.rt_hBlock,
 			      &rect, 
 			      &contactOKMask,
-			      mzCWalksFunc3, 
+			      mzCWalksFunc2, 
 			      (ClientData) &wD);
-	    }
 	}
     }
     
@@ -1263,7 +1339,7 @@ mzCWalksFunc2(tile, cdarg)
 /*
  * ----------------------------------------------------------------------------
  *
- * mzCWalksFunc3 --
+ * mzCWalksFunc2 --
  *
  * Called for tiles on contact plane where contact is OK that overlap a
  * dest area.  We paint a CONTACT_WALK on the blockage planes for the
@@ -1279,27 +1355,25 @@ mzCWalksFunc2(tile, cdarg)
  */
 
 int
-mzCWalksFunc3(tile, cdarg)
+mzCWalksFunc2(tile, cdarg)
     Tile *tile;
     ClientData cdarg;
 {
     WalkContactFuncData *wD = (WalkContactFuncData *) cdarg;
     Rect rect;
+    Walk *walk;
 
     /* rect = tile clipped dest area bounds. */
     TITORECT(tile, &rect);
     GEOCLIP(&rect, wD->wd_bounds);
 
-    /* paint contact_walk into blockage planes */
+    /* Build walks for contacts above or below the tile */
     {
-	DBPaintPlane(wD->wd_rL->rl_routeType.rt_hBlock, 
-		     &rect, 
-		     mzBlockPaintTbl[TT_CONTACT_WALK],
-		     (PaintUndoInfo *) NULL);
-	DBPaintPlaneVert(wD->wd_rL->rl_routeType.rt_vBlock, 
-			 &rect,
-			 mzBlockPaintTbl[TT_CONTACT_WALK],
-			 (PaintUndoInfo *) NULL);
+	walk = (Walk *) mallocMagic((unsigned) (sizeof(Walk)));
+	walk->w_rT = &(wD->wd_rL->rl_routeType);
+	walk->w_type = wD->wd_walk;
+	walk->w_rect = rect;
+	LIST_ADD(walk, mzWalkList);
     }
 
     /* return 0 - to continue search */
