@@ -20,7 +20,7 @@
  *     *********************************************************************
  *
  *
- * rcsid $Header: /ufs/repository/magic/mzrouter/mzInternal.h,v 1.2 2001/01/12 22:13:02 jsolomon Exp $
+ * rcsid $Header: /usr/cvsroot/magic-7.5/mzrouter/mzInternal.h,v 1.7 2006/10/14 21:52:36 tim Exp $
  */
 
 #ifndef _MZINTERNAL_H
@@ -82,10 +82,14 @@ extern ClientData mzDebugID;
  * quadrant with respect to destination point).
  */
 #define DEF_ESTIMATE 1
-/* If set, routes may terminate at any geometry that is electrically connected
- * to specified dest regions.
+/* If set, routes may start or terminate at any geometry that is
+ * electrically connected to specified start or dest regions.
  */
-#define DEF_EXPAND_DESTS 0
+#define DEF_EXPAND_ENDPOINTS 1
+#define MZ_EXPAND_START	     1	/* ClientData type for start tiles */
+#define MZ_EXPAND_DEST	     0  /* ClientData type for dest tiles */
+#define MZ_EXPAND_NONE	     MINFINITY  /* Normal ClientData type */
+
 /* If set only hints in toplevel cell are recognized - speeds up processing
  * of hint planes when there are lots of expanded subcells.
  */
@@ -133,27 +137,34 @@ extern CellUse *mzResultUse;
 
 /* BLOCKAGE PLANES - AND CELL FOR DISPLAY */
 /* Higher numbered blockage types ALWAYS take priority during painting */
-    /* Block due to destnode only */
-#define TT_SAMENODE_BLOCK (1 + TT_OFF)  
+
+    /* Start or destination terminal */
+#define TT_SAMENODE	  (1 + TT_OFF)
+    /* Block due to start or destination terminal */
+#define TT_SAMENODE_BLOCK (2 + TT_OFF)
     /* Can reach dest via contact */
-#define TT_CONTACT_WALK	  (2 + TT_OFF)
+#define TT_ABOVE_WALK	  (3 + TT_OFF)
+#define TT_BELOW_WALK	  (4 + TT_OFF)
     /* Approach to dest area */
-#define TT_LEFT_WALK	  (3 + TT_OFF)  
-#define TT_RIGHT_WALK     (4 + TT_OFF)
-#define TT_TOP_WALK       (5 + TT_OFF)
-#define TT_BOTTOM_WALK    (6 + TT_OFF)
+#define TT_LEFT_WALK	  (5 + TT_OFF)  
+#define TT_RIGHT_WALK     (6 + TT_OFF)
+#define TT_TOP_WALK       (7 + TT_OFF)
+#define TT_BOTTOM_WALK    (8 + TT_OFF)
     /* route destination area */
-#define TT_DEST_AREA      (7 + TT_OFF)  
+#define TT_DEST_AREA      (9 + TT_OFF)  
     /* Can't route in this space */
-#define	TT_BLOCKED	  (8 + TT_OFF)	
-extern PaintResultType mzBlockPaintTbl[TT_MAXTYPES][TT_MAXTYPES];
+#define	TT_BLOCKED	  (10 + TT_OFF)	
+
+#define TT_MAXROUTETYPES (1 + TT_BLOCKED)
+
+extern PaintResultType mzBlockPaintTbl[TT_MAXROUTETYPES][TT_MAXROUTETYPES];
 extern CellDef *mzBlockDef ;
 
 /* BOUNDS PLANE */
     /* Blockage planes generated here */
 #define TT_INBOUNDS	(1 + TT_OFF)	
 #define TT_GENBLOCK	(2 + TT_OFF)
-extern PaintResultType mzBoundsPaintTbl[TT_MAXTYPES][TT_MAXTYPES];
+extern PaintResultType mzBoundsPaintTbl[TT_MAXROUTETYPES][TT_MAXROUTETYPES];
 extern Plane *mzHBoundsPlane;
 extern Plane *mzVBoundsPlane;
 
@@ -161,12 +172,13 @@ extern Plane *mzVBoundsPlane;
 #define TT_EST_SUBCELL 	(1 + TT_OFF)
 #define TT_EST_FENCE 	(2 + TT_OFF)
 #define TT_EST_DEST	(3 + TT_OFF)
-extern PaintResultType mzEstimatePaintTbl[TT_MAXTYPES][TT_MAXTYPES];
+extern PaintResultType mzEstimatePaintTbl[TT_MAXROUTETYPES][TT_MAXROUTETYPES];
 extern Plane *mzEstimatePlane;
 
 /* HINT TYPES */
 extern TileTypeBitMask mzHintTypesMask;     /* map of hint, fence
 						and rotate types */
+extern TileTypeBitMask mzStartTypesMask;    /* mask of valid types for a route start */
 
 /* HINT (magnet) PLANES */
 extern Plane *mzHHintPlane;
@@ -217,22 +229,11 @@ extern RoutePage *mzCurPage;
 	    ? mzAllocRPath() \
 	    : (RoutePath *) (&mzCurPage->rpp_array[mzCurPage->rpp_free++]))
 
-/*---------------------- Start Terminal ------------------------------------ */
-/* A start terminal is a point and a tiletype to start on.
- * This struc is used to gather list of legal start points.
- */
-typedef struct
-{
-    Point 	cp_point;	/* (x, y) */
-    TileType    cp_type;	/* layer to start on */
-} ColoredPoint;
-
-
-/*---------------------- Dest Terminal ------------------------------------ */
-/* A destination terminal is a rectangle and a type to connect on. 
- * This struc is used to create lists of legal destination areas.
- * (This structure also used in other situations requiring rects with
- *  an associated tiletype.)
+/*------------------- Start and Dest Terminals ---------------------------- */
+/* A start or destination terminal is a rectangle and a type to connect
+ * on. This struct is used to create lists of legal start and destination
+ * areas.  (This structure also used in other situations requiring rects
+ * with an associated tiletype.)
  */
 typedef struct
 {
@@ -281,19 +282,30 @@ extern Heap mzMinCostCompleteHeap;  /* completed paths */
 extern RoutePath *mzAllocRPath();
 extern char *mzCost2String();
 extern char *mzDICost2String();
-/* BY NP */
-extern dlong mzEstimatedCost();
 extern RouteType *mzFindRouteType();
 extern void mzNLClear();
 extern int *mzNLGetContainingInterval();
 extern void mzNLInit();
 extern void mzNLInsert();
-extern RoutePath *mzSearch();
 extern void mzTechFinal();
+extern int mzPaintContact();
 extern TileTypeBitMask mzTouchingTypes();
 
+extern void mzAddPoint(RoutePath *, Point *, RouteLayer *, int, int, dlong *);
+extern void mzWalkRight(RoutePath *);
+extern void mzWalkLeft(RoutePath *);
+extern void mzWalkUp(RoutePath *);
+extern void mzWalkDown(RoutePath *);
+extern void mzWalkContact(RoutePath *);
+extern dlong mzEstimatedCost(Point *);
+extern RoutePath *mzSearch();
+
+
 /* --------------------- Variables Global to MazeRouter ----------------- */
 
+/* Use a maximum cost which we can add to without overflowing */
+#define COST_MAX        (DLONG_MAX >> 2)
+
 /* Parameter sets for mzrouting - from tech file. */
 extern MazeStyle *mzStyles;
 
@@ -354,9 +366,13 @@ extern int mzBoundsIncrement;
  */
 extern int mzEstimate;
 /* If set, routes may terminate at any geometry that is 
- * electrically connected to specified dest areas.
+ * electrically connected to specified dest areas (default TRUE)
  */
 extern int mzExpandDests;
+/* If set, routes may start at any geometry that is 
+ * electrically connected to specified start areas (default TRUE)
+ */
+extern int mzExpandStarts;
 /* If set, only hints in toplevel cell are recognized */
 extern int mzTopHintsOnly;
 /* Maximum distance route will extend into blocked area to connect to dest. */
@@ -373,18 +389,15 @@ extern int mzVerbosity;
 extern int mzBloomLimit;
 
 /* minimum estimated total cost for initial paths */
-/* BY NP */
 extern dlong mzMinInitialCost;
 
 /* Parameters controlling search */
 extern RouteFloat mzPenalty;
-/* BY NP */
 extern dlong mzWRate;
 extern dlong mzBloomDeltaCost;
 extern dlong mzWWidth;
 
 /* Statistics */
-/* BY NP */
 extern dlong mzInitialEstimate;  /* Initial estimated cost of route */
 extern int mzNumBlooms;
 extern int mzNumOutsideBlooms;	/* num blooms from outside window */
@@ -399,13 +412,11 @@ extern int mzReportInterval;    /* frequency that # of paths etc.
 extern int mzPathsTilReport;	/* counts down to next path report */
 
 /* Variables controling search */
-/* BY NP */
 extern dlong mzWInitialMinToGo;
 extern dlong mzWInitialMaxToGo;
 extern dlong mzBloomMaxCost;	
 
 /* Search status */
-/* BY NP */
 extern dlong mzWindowMinToGo; /* Window location */
 extern dlong mzWindowMaxToGo;
 
