@@ -36,7 +36,7 @@
  * toward the goal over time. Normally the partial-path of least 
  * estimated-total-cost WITHIN
  * the window is chosen for expansion, although a further path (i.e. one
- * beyond the window) that has exceptially low estimated-total-cost is
+ * beyond the window) that has exceptionally low estimated-total-cost is
  * sometimes chosen instead.  When the trailing edge of the window reaches
  * the goal, the search is deemed complete and the lowest cost complete
  * path found is returned.  However if no complete-path has been found
@@ -91,15 +91,15 @@
  *                    paths nearer to completion - a startegy that is
  *                    adequate in many situations.
  *
- * mzBloomStack - paths in current local foucs = all expansions of a given
+ * mzBloomStack - paths in current local focus = all expansions of a given
  *                partial-path that do not exceed a given estimated-cost
  *                (also see stacks below)
  *
  * mzStraightStack - focus paths being extended in straight line.
  *
- * mzDownHillStack - focus paths followed only unitl cost increases.
+ * mzDownHillStack - focus paths followed only until cost increases.
  *
- * mzWalkStack - paths insise walk (i.e. inside blocked area adjacent to
+ * mzWalkStack - paths inside walk (i.e. inside blocked area adjacent to
  *               destination.
  *
  * If the walk stack is not empty the next path is taken from there (thus
@@ -123,7 +123,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/mzrouter/mzSearch.c,v 1.2 2001/01/12 22:13:02 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/mzrouter/mzSearch.c,v 1.9 2006/10/21 04:36:07 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -148,17 +148,10 @@ static char rcsid[] = "$Header: /ufs/repository/magic/mzrouter/mzSearch.c,v 1.2
 
 /* Forward declarations */
 
-extern void mzExtendPath();
 extern void mzExtendViaContacts();
-extern void mzAddPoint();
 extern void mzBloomInit();
 extern void mzMakeStatReport();
-extern void mzExtendInitPath();
-extern void mzAddInitialContacts();
-extern void mzStartRight();
-extern void mzStartLeft();
-extern void mzStartUp();
-extern void mzStartDown();
+extern void mzExtendPath(RoutePath *);
 
 
 /*
@@ -183,9 +176,9 @@ extern void mzStartDown();
  *
  * ----------------------------------------------------------------------------
  */
-/* BY NP */
 RoutePath *
-mzSearch()
+mzSearch(mzResult)
+    int *mzResult;
 {
     RoutePath *path;		/* solution */
     bool morePartialPaths = TRUE;
@@ -482,21 +475,21 @@ mzSearch()
 
 		while((maxToGoTop=HeapRemoveTop(&mzMaxToGoHeap,
 			  &maxToGoTopBuf)) != NULL && 
-		      (maxToGoTop->he_union.hu_dint >= mzWindowMinToGo))
+		      (maxToGoTop->he_union.hu_dlong >= mzWindowMinToGo))
 		{
 		    if (DebugIsSet(mzDebugID, mzDebMaze))
 		    {
 			mzPrintPathHead((RoutePath*)(maxToGoTop->he_id));
 		    }
 
-		    HeapAddDInt(&mzMinCostHeap, 
+		    HeapAddDLong(&mzMinCostHeap, 
 			      ((RoutePath *)(maxToGoTop->he_id))->rp_cost, 
 			      (char *) (maxToGoTop->he_id));
 		}
 		if(maxToGoTop!=NULL)
 		{
-		    HeapAddDInt(&mzMaxToGoHeap,
-			      maxToGoTop->he_union.hu_dint, 
+		    HeapAddDLong(&mzMaxToGoHeap,
+			      maxToGoTop->he_union.hu_dlong, 
 			      (char *) (maxToGoTop->he_id));
 		}
 	    }
@@ -523,7 +516,7 @@ mzSearch()
 		    adjCost = (dlong)((RoutePath *)(minCostTop->he_id))->rp_togo;
 		    adjCost = (dlong)(adjCost * mzPenalty.rf_mantissa);
 		    adjCost = adjCost >> mzPenalty.rf_nExponent;
-		    adjCost += minCostTop->he_union.hu_dint;
+		    adjCost += minCostTop->he_union.hu_dlong;
 
 		    if (DebugIsSet(mzDebugID, mzDebMaze))
 		    {
@@ -532,14 +525,14 @@ mzSearch()
 		    }
 
 		    /* add to adjusted cost heap */
-		    HeapAddDInt(&mzMinAdjCostHeap, adjCost,
+		    HeapAddDLong(&mzMinAdjCostHeap, adjCost,
 			      (char *) (minCostTop->he_id));
 		}
 
 		if(minCostTop!=NULL)
 		{
-		    HeapAddDInt(&mzMinCostHeap,
-			      minCostTop->he_union.hu_dint, 
+		    HeapAddDLong(&mzMinCostHeap,
+			      minCostTop->he_union.hu_dlong, 
 			      (char *) (minCostTop->he_id));
 		}
 	    }
@@ -577,7 +570,7 @@ mzSearch()
 		if(minAdjCostTop)
 		{
 		    TxPrintf("  Heap-key adjCost:  %.0f\n", 
-			     (double)(minAdjCostTop->he_union.hu_dint));
+			     (double)(minAdjCostTop->he_union.hu_dlong));
 		}
 		else
 		{
@@ -606,7 +599,7 @@ mzSearch()
 		    TxPrintf("WINDOW-CORRECTED ADJCOST:  %.0f\n",
 			 	(double)(adjCost));
 		}
-		if(minCostTop->he_union.hu_dint <= adjCost)
+		if(minCostTop->he_union.hu_dlong <= adjCost)
 		{
 		    if (DebugIsSet(mzDebugID, mzDebMaze))
 		    {
@@ -688,10 +681,24 @@ mzSearch()
 
 	if(HeapRemoveTop(&mzMinCostCompleteHeap,&heEntry))
 	{
+	    if (mzResult)
+	    {
+		if (SigInterruptPending)
+		    *mzResult = MZ_CURRENT_BEST;
+		else
+		    *mzResult = MZ_SUCCESS;
+	    }
 	    return (RoutePath *)(heEntry.he_id);
 	}
 	else
 	{
+	    if (mzResult)
+	    {
+		if (SigInterruptPending)
+		    *mzResult = MZ_INTERRUPTED;
+		else
+		    *mzResult = MZ_FAILURE;
+	    }
 	    return NULL;
 	}
     }
@@ -782,7 +789,7 @@ mzExtendPath(path)
  * mzExtendViaContacts --
  *
  * Spread from a point to other planes reachable from it,
- * by using contacts.  Stacked contacts are not allowed.
+ * by using contacts.  Stacked contacts are allowed.
  *
  * Results	None.
  *
@@ -791,24 +798,66 @@ mzExtendPath(path)
  *
  * ----------------------------------------------------------------------------
  */
-/* BY NP */
 void
 mzExtendViaContacts(path)
     RoutePath *path;
 {
-    Point *p = &path->rp_entry;
+    Point p = path->rp_entry, *lastCpos = NULL;
     RouteLayer *rLayer = path->rp_rLayer; 
+    RouteContact *rC;
     List *cL;
     RouteLayer *newRLayer;
+    Tile *tp;
+    TileType type, lastCtype = TT_SPACE;
+    RoutePath *spath;
+    int bendDist = 0;
 
     /* DEBUG - trace calls to this routine */
     if (DebugIsSet(mzDebugID, mzDebMaze))
 	TxPrintf("EXTENDING WITH CONTACTS\n");
 
+    /* Check what the last contact was and remember its	*/
+    /* position.  Do not allow two contacts of the same	*/
+    /* type within a DRC error distance of each other.	*/
+
+    for (spath = path; spath && spath->rp_back && (spath->rp_orient != 'O');
+		spath = spath->rp_back);
+    if (spath->rp_back)
+    {
+	lastCpos = &spath->rp_entry;
+	rC = MZGetContact(spath, spath->rp_back);
+	lastCtype = rC->rc_routeType.rt_tileType;
+    }
+
+    /* Check where the last route bend is.  Don't allow a	*/
+    /* contact within a DRC error distance of a	bend.		*/
+
+    if (path)
+    {
+	if (path->rp_orient == 'V')
+	{
+	    for (spath = path->rp_back; spath && spath->rp_orient == 'V';
+			spath = spath->rp_back);
+	    if (spath && spath->rp_orient == 'H')
+		bendDist = spath->rp_entry.p_y - p.p_y;
+	    if (bendDist < 0)
+		bendDist += rLayer->rl_routeType.rt_width;
+	}
+	else if (path->rp_orient == 'H')
+	{
+	    for (spath = path->rp_back; spath && spath->rp_orient == 'H';
+			spath = spath->rp_back);
+	    if (spath && spath->rp_orient == 'V')
+		bendDist = spath->rp_entry.p_x - p.p_x;
+	    if (bendDist < 0)
+		bendDist += rLayer->rl_routeType.rt_width;
+	}
+    }
+
     /* Loop through contacts that connect to current rLayer */
     for (cL=rLayer->rl_contactL; cL!=NULL; cL=LIST_TAIL(cL))
     {
-	RouteContact *rC = (RouteContact *) LIST_FIRST(cL);
+	rC = (RouteContact *) LIST_FIRST(cL);
 
 	/* Don't use inactive contacts */
 	if (!(rC->rc_routeType.rt_active))
@@ -827,18 +876,35 @@ mzExtendViaContacts(path)
 	}
 
 	/* Don't spread to inactive layers */
-	if (!(newRLayer->rl_routeType.rt_active))
-	    continue;
+	if (!(newRLayer->rl_routeType.rt_active)) continue;
 
 	/* Find tile on contact plane that contains point. */
-	{
-	    Tile *tp = TiSrPointNoHint(rC->rc_routeType.rt_hBlock,p);
 
-	    /* if blocked, don't place a contact */
-	    if (TiGetType(tp) != TT_SPACE)
-	    {
+	tp = TiSrPointNoHint(rC->rc_routeType.rt_hBlock, &p);
+	type = TiGetType(tp);
+
+	/* 1. If blocked, don't place a contact */
+	if ((type != TT_SPACE) && (type != TT_SAMENODE)) continue;
+
+	/* 2. Check distance from last contact, if the same type */
+	if (rC->rc_routeType.rt_tileType == lastCtype)
+	{
+	    int cdist = rC->rc_routeType.rt_spacing[lastCtype] +
+			rC->rc_routeType.rt_width;
+	    if ((abs(p.p_x - lastCpos->p_x) < cdist) &&
+			(abs(p.p_y - lastCpos->p_y) < cdist))
+		continue;
+	}
+
+	/* 3. Check distance to last route bend */
+	if (bendDist != 0)
+	{
+	    int cwidth = rC->rc_routeType.rt_width;
+	    int spacing = rC->rc_routeType.rt_spacing[rLayer->rl_routeType.rt_tileType];
+	    if (bendDist > cwidth && bendDist < cwidth + spacing)
+		continue;
+	    if (bendDist < 0 && bendDist > -spacing)
 		continue;
-	    }
 	}
 
 	/* OK to drop a contact here */
@@ -850,52 +916,51 @@ mzExtendViaContacts(path)
 	    conCost = (dlong) rC->rc_cost;
 
 	    /* determine appropriate extendcode */
+	    tp = TiSrPointNoHint(newRLayer->rl_routeType.rt_hBlock, &p);
+	    type = TiGetType(tp);
+
+	    switch (type)
 	    {
-		Tile *tp = TiSrPointNoHint(newRLayer->rl_routeType.rt_hBlock,
-					   p);
-		TileType type = TiGetType(tp);
+		case TT_SPACE:
+		case TT_SAMENODE:
+		   extendCode = EC_RIGHT | EC_LEFT | EC_UP | EC_DOWN;
+		   break;
 
-		switch (type)
-		{
-		    case TT_SPACE:
-			extendCode = EC_RIGHT | EC_LEFT | EC_UP | EC_DOWN;
-			break;
+		case TT_LEFT_WALK:
+		    extendCode = EC_WALKRIGHT;
+		    break;
 
-		    case TT_LEFT_WALK:
-			extendCode = EC_WALKRIGHT;
-			break;
+		case TT_RIGHT_WALK:
+		    extendCode = EC_WALKLEFT;
+		    break;
 
-		    case TT_RIGHT_WALK:
-			extendCode = EC_WALKLEFT;
-			break;
+		case TT_TOP_WALK:
+		    extendCode = EC_WALKDOWN;
+		    break;
 
-		    case TT_TOP_WALK:
-			extendCode = EC_WALKDOWN;
-			break;
+		case TT_BOTTOM_WALK:
+		    extendCode = EC_WALKUP;
+		    break;
 
-		    case TT_BOTTOM_WALK:
-			extendCode = EC_WALKUP;
-			break;
+		case TT_ABOVE_WALK:
+		case TT_BELOW_WALK:
+		    /* TO DO:  Check if stacked contacts are allowed! */
+		    extendCode = EC_WALKCONTACT;
+		    break;
 
-		    case TT_CONTACT_WALK:
-			/* skip this contact after all, to avoid stacked
-			 * contacts.
-			 */
-			continue;
-			break;
+		case TT_DEST_AREA:
 
-		    case TT_DEST_AREA:
-			extendCode = EC_COMPLETE;
-			break;
+		    extendCode = EC_COMPLETE;
+		    break;
 
-		    default:
-			/* this shouldn't happen */
-			ASSERT(FALSE,"mzExtendViaContacts");
-		}
+		default:
+		    /* this shouldn't happen */
+		    ASSERT(FALSE,"mzExtendViaContacts");
+		    continue;
 	    }
 
 	    /* Finally add point after contact */
-	    mzAddPoint(path, p, newRLayer, 'O', extendCode, conCost);
+	    mzAddPoint(path, &p, newRLayer, 'O', extendCode, &conCost);
 	}
     }
 
@@ -920,9 +985,8 @@ mzExtendViaContacts(path)
  *
  * ----------------------------------------------------------------------------
  */
-/* BY NP */
 void
-mzAddPoint(path, p, rLayer, orient, extendCode, cost)
+mzAddPoint(path, p, rLayer, orient, extendCode, costptr)
     RoutePath *path;	/* path that new point extends */
     Point *p;		/* new point */
     RouteLayer *rLayer;	/* Route Layer of new point */
@@ -930,14 +994,14 @@ mzAddPoint(path, p, rLayer, orient, extendCode, cost)
 			 * 'O' = contact, 'B' = first point in path and blocked
 			 */
     int extendCode;	/* interesting directions to extend in */
-    dlong cost;		/* Incremental cost of new path segment */
+    dlong *costptr;	/* Incremental cost of new path segment */
 
 {
     RoutePath *newPath;
     RoutePath *hashedPath;
     HashEntry *he;
     PointKey pk;
-    dlong togo;
+    dlong togo, cost;
 
     /* DEBUG - print candidate frontier point */
     if (DebugIsSet(mzDebugID, mzDebMaze))
@@ -948,6 +1012,7 @@ mzAddPoint(path, p, rLayer, orient, extendCode, cost)
 	    	orient);
     }
 
+    cost = *costptr;
     ASSERT(cost >= 0,"mzAddPoint");
 
     /* Ignore if outside of bounding box */
@@ -965,23 +1030,24 @@ mzAddPoint(path, p, rLayer, orient, extendCode, cost)
 	/* initially cost = cost of new leg in path */
 
 	/* Add jogcost if appropriate */
-	if(path!=NULL && 
+
+	if (path != NULL && 
 	   path->rp_rLayer == rLayer &&
 	   path->rp_orient != 'O' &&
 	   path->rp_orient != orient)
 	{
-	    cost +=  rLayer->rl_jogCost;
+	    cost += rLayer->rl_jogCost;
 	}
 
 	/* Add estimated total cost prior to new point, 
 	 * (or cost so far in the case of initial paths).
 	 */
-	if(path!=NULL)
+	if (path != NULL)
 	{
 	    cost += path->rp_cost;
 	}
 	/* If not initial path, subtract out old estimated cost togo */
-	if(mzPathSource!=SOURCE_INIT)
+	if (mzPathSource != SOURCE_INIT)
 	{
 	    cost -= path->rp_togo;
 	}
@@ -1046,7 +1112,7 @@ mzAddPoint(path, p, rLayer, orient, extendCode, cost)
 	    TxPrintf("PATH COMPLETE (WALKED IN).  Add to complete heap.\n");
 	}
 
-	HeapAddDInt(&mzMinCostCompleteHeap, newPath->rp_cost, 
+	HeapAddDLong(&mzMinCostCompleteHeap, newPath->rp_cost, 
 		  (char *) newPath);
 
 	/* compute stats and make completed path report */
@@ -1105,7 +1171,7 @@ mzAddPoint(path, p, rLayer, orient, extendCode, cost)
 	    else
 	    {
 		/* non preferred, add to heaps */
-		HeapAddDInt(&mzMaxToGoHeap, togo, (char *) newPath);
+		HeapAddDLong(&mzMaxToGoHeap, togo, (char *) newPath);
 	    }
 	    break;
 
@@ -1135,14 +1201,14 @@ mzAddPoint(path, p, rLayer, orient, extendCode, cost)
 		else
 		{
 		    /* increased cost, add to heaps */
-	  	    HeapAddDInt(&mzMaxToGoHeap, togo, (char *) newPath);
+	  	    HeapAddDLong(&mzMaxToGoHeap, togo, (char *) newPath);
 		}
 	    }
 	    break;
 
 	    case SOURCE_INIT:
 	    /* Initial points go on heap */
-	    HeapAddDInt(&mzMaxToGoHeap, togo, (char *) newPath);
+	    HeapAddDLong(&mzMaxToGoHeap, togo, (char *) newPath);
 	    break;
 	}
     }
@@ -1165,7 +1231,6 @@ mzAddPoint(path, p, rLayer, orient, extendCode, cost)
  *
  * ----------------------------------------------------------------------------
  */
-/* BY NP*/
 void
 mzBloomInit(path)
     RoutePath *path;	/* path that new point extends */
@@ -1196,7 +1261,6 @@ mzBloomInit(path)
  * ----------------------------------------------------------------------------
  */
 
-/* BY NP*/
 void
 mzMakeStatReport()
 {
