@@ -17,7 +17,7 @@
  *     *********************************************************************
  */
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/mzrouter/mzXtndUp.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/mzrouter/mzXtndUp.c,v 1.7 2006/10/14 21:52:36 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -52,7 +52,6 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/mzro
  *
  * ----------------------------------------------------------------------------
  */
-/* BY NP */
 void
 mzExtendUp(path)
     RoutePath *path;
@@ -66,6 +65,9 @@ mzExtendUp(path)
 			 * avoid extensions in uninteresting directions.
 			 */
     int extendCode;	/* Interesting directions to extend in */
+    bool overroute = FALSE;     /* Is crossing another route layer */
+    TileType ntype;
+    Tile *tpThis;
 
     /* DEBUG - trace calls to this routine. */
     if (DebugIsSet(mzDebugID, mzDebMaze))
@@ -99,7 +101,7 @@ mzExtendUp(path)
      *  
      */
     {
-	Tile *tpThis, *tpNext;
+	Tile *tpNext;
 	bool covered;
 
 	/* get tpThis, extending bounds plane and iterating, until
@@ -115,7 +117,7 @@ mzExtendUp(path)
 				     &pOrg);
 
 	    /* org point should not be blocked */
-	    ASSERT(TiGetType(tpThis)==TT_SPACE,"mzExtendUp");
+	    ASSERT(TiGetType(tpThis) == TT_SPACE, "mzExtendUp");
 
 	    /* check to see if covered, if not extend bounds and start over */
 	    covered = TRUE;
@@ -165,28 +167,30 @@ mzExtendUp(path)
 
 	/* also get next tile over */ 
 	NEXT_TILE_UP(tpNext, tpThis, pOrg.p_x);
+	ntype = TiGetType(tpNext);
 
-        if(TiGetType(tpNext)!=TT_SPACE)
+        if ((ntype != TT_SPACE) && (ntype != TT_SAMENODE))
 	{
 	    /* path blocked */
-	    if(BOTTOM(tpNext)==pStep.p_y)
+	    if (BOTTOM(tpNext) == pStep.p_y)
 	    {
 		/* pOrg right up against block */
-		if(TiGetType(tpNext)==TT_BOTTOM_WALK)
+		if (ntype == TT_BOTTOM_WALK)
 		{
 		    /* Block is walk, enter it */
 		    pNew.p_y = pOrg.p_y + 1;
 		    reasons = RC_WALK;
 		    goto donePruning;
 		}
-		else if(TiGetType(tpNext)==TT_CONTACT_WALK)
+		else if (ntype == TT_ABOVE_WALK ||
+			ntype == TT_BELOW_WALK)
 		{
 		    /* Block is contact walk, enter it */
 		    pNew.p_y = pOrg.p_y + 1;
 		    reasons = RC_WALKC;
 		    goto donePruning;
 		}
-		else if(TiGetType(tpNext)==TT_DEST_AREA)
+		else if (ntype == TT_DEST_AREA)
 		{
 		    /* Block is destination, enter it */
 		    pNew.p_y = pOrg.p_y + 1;
@@ -237,7 +241,8 @@ mzExtendUp(path)
      */
     for(rL=mzActiveRLs; rL!=NULL; rL=rL->rl_nextActive)
     {
-	Tile *tpThis, *tpNext;
+	Tile *tpNext;
+	TileType tpType;
 	bool covered;
 
 	/* skip current layer (already handled above) */
@@ -250,7 +255,8 @@ mzExtendUp(path)
 	tpThis = TiSrPointNoHint(rL->rl_routeType.rt_hBlock, 
 				 &pOrg);
 
-	if(TiGetType(tpThis)!=TT_SPACE)
+	tpType = TiGetType(tpThis);
+	if ((tpType != TT_SPACE) && (tpType != TT_SAMENODE))
 	{
 	    /* ORG POINT BLOCKED */
 	    /* this case handled by contact code below, so skip to next layer*/
@@ -325,8 +331,9 @@ mzExtendUp(path)
 
 	    /* get next tile over */ 
 	    NEXT_TILE_UP(tpNext, tpThis, pOrg.p_x);
+	    ntype = TiGetType(tpNext);
 
-	    if(TiGetType(tpNext)!=TT_SPACE)
+	    if ((ntype != TT_SPACE) && (ntype != TT_SAMENODE))
 	    {
 		/* path blocked */
 		if(BOTTOM(tpNext)==pStep.p_y)
@@ -411,6 +418,7 @@ mzExtendUp(path)
     {
 	List *cL;
 	Tile *tp;
+	TileType tpType;
 
 	/* Loop thru contact types connecting to current route layer */
 	for (cL=path->rp_rLayer->rl_contactL; cL!=NULL; cL=LIST_TAIL(cL))
@@ -420,7 +428,8 @@ mzExtendUp(path)
 		((RouteContact*) LIST_FIRST(cL))->rc_routeType.rt_vBlock, 
 		&pStep);
 
-	    if (TiGetType(tp) == TT_SPACE)
+	    tpType = TiGetType(tp);
+	    if ((tpType == TT_SPACE) || (tpType == TT_SAMENODE))
 	    {
 		/* SPACE TILE */ 
 		if(BOTTOM(tp) > pOrg.p_y)
@@ -434,6 +443,7 @@ mzExtendUp(path)
 	    {
 		/* BLOCK TILE - so prune to just beyond tile */
 		PRUNE_TO_MIN(pNew.p_y, TOP(tp), reasons, RC_CONTACT);
+		if (tpType == TT_BLOCKED) overroute = TRUE;
 	    }
 	}
     }
@@ -528,6 +538,14 @@ mzExtendUp(path)
 	}
     }
 
+    /* If we end inside SAMENODE, then the cost to this point is	*/
+    /* zeroed, and that section of the path will not be painted.	*/
+
+    tpThis = TiSrPointNoHint(path->rp_rLayer->rl_routeType.rt_hBlock, &pNew);
+    if (TiGetType(tpThis) == TT_SAMENODE)
+        if ((!path->rp_back) || (path->rp_back->rp_cost == (dlong)0))
+            path->rp_cost = (dlong)0;
+
     /* compute cost of path segment from pOrg to pNew */
     {
 	Tile *tp;
@@ -539,6 +557,9 @@ mzExtendUp(path)
 	if (rotate)
 	    segCost = (dlong) ((pNew.p_y - pOrg.p_y) *
 			path->rp_rLayer->rl_hCost);
+	else if (overroute)
+	    segCost = (dlong) ((pNew.p_y - pOrg.p_y) *
+			path->rp_rLayer->rl_overCost);
 	else
 	    segCost = (dlong) ((pNew.p_y - pOrg.p_y) *
 			path->rp_rLayer->rl_vCost);
@@ -595,7 +616,7 @@ mzExtendUp(path)
     }
 
     /* Process the new point */
-    mzAddPoint(path, &pNew, path->rp_rLayer, 'V', extendCode, segCost);
+    mzAddPoint(path, &pNew, path->rp_rLayer, 'V', extendCode, &segCost);
 
     return;
 }
