@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/select/selEnum.c,v 1.3 2001/03/29 19:50:11 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selEnum.c,v 1.2 2006/09/24 23:35:58 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -559,16 +559,16 @@ selEnumCFunc2(scx, arg)
  *	form
  *
  *	int
- *	func(label, cellDef, transform, clientData)
+ *	func(label, cellUse, transform, clientData)
  *	    Label *label;
- *	    CellDef *cellDef;
+ *	    CellUse *cellUse;
  *	    Transform *transform;
  *	    ClientData clientData;
  *	{
  *	}
  *
  *	Label is a pointer to a selected label.  It refers to the label
- *	in cellDef, and transform gives the transform from that
+ *	in cellUse, and transform gives the transform from that
  *	cell's coordinates to root coordinates.  ClientData is just
  *	the clientData argument passed to this procedure.  Func
  *	should normally return 0.  If it returns a non-zero return
@@ -642,7 +642,7 @@ SelEnumLabels(layers, editOnly, foundNonEdit, func, clientData)
 	    continue;
 	}
 
-	if ((*func)(arg.sea_foundLabel, arg.sea_foundUse->cu_def,
+	if ((*func)(arg.sea_foundLabel, arg.sea_foundUse,
 	    &arg.sea_foundTrans, clientData) != 0) return 1;
     }
 
