@@ -18,15 +18,11 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic/textio/txCommands.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-7.5/textio/txCommands.c,v 1.2 2006/05/13 07:36:14 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#ifdef  SYSV
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 #include <ctype.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -1140,6 +1136,12 @@ TxTclDispatch(clientData, argc, argv)
     int n, asize;
     TxCommand *tclcmd;
 
+    if (argc > TX_MAXARGS)
+    {
+	TxError("Error: number of command arguments exceeds %d!\n", TX_MAXARGS);
+	return;
+    }
+
     DRCBreak();
 
     SigIOReady = FALSE;
@@ -1150,7 +1152,14 @@ TxTclDispatch(clientData, argc, argv)
     tclcmd->tx_argc = argc;
 
     asize = 0;
-    for (n = 0; n < argc; n++) {
+    for (n = 0; n < argc; n++)
+    {
+	if (asize + strlen(argv[n]) >= TX_MAX_CMDLEN)
+	{
+	    TxError("Error: command length exceeds %d characters!\n", TX_MAX_CMDLEN);
+	    TxFreeCommand(tclcmd);
+	    return;
+	}
 	strcpy(&tclcmd->tx_argstring[asize], argv[n]);
 	tclcmd->tx_argv[n] = &tclcmd->tx_argstring[asize];
 	asize += (1 + strlen(argv[n]));
