@@ -17,7 +17,7 @@
  *     * of the United States of America may require an export license.    * 
  *     *********************************************************************
  *
- * rcsid "$Header: /ufs/repository/magic/tiles/tile.h,v 1.5 2001/11/29 20:50:09 tim Exp $"
+ * rcsid "$Header: /usr/cvsroot/magic-7.5/tiles/tile.h,v 1.3 2006/04/26 04:37:33 tim Exp $"
  */
 
 #ifndef _TILES_H
@@ -132,11 +132,11 @@ typedef struct
  * from either INFINITY or MINFINITY not cause overflow.
  *
  * Consequently, we define INFINITY to be the largest integer
- * representable in wordsize - 5 bits.
+ * representable in wordsize - 2 bits.
  */
 
 #undef INFINITY
-#define	INFINITY	((1 << (8*sizeof (int) - 6)) - 4)
+#define	INFINITY	((1 << (8*sizeof (int) - 2)) - 4)
 #define	MINFINITY	(-INFINITY)
 
 /* ------------------------ Flags, etc -------------------------------- */
@@ -195,17 +195,17 @@ extern Tile *TiSrPoint();
  *
  */
 
-#define TiGetType(tp)		((TileType)((tp)->ti_body) & TT_LEFTMASK)
-#define TiGetTypeExact(tp)	((TileType) (tp)->ti_body)
-#define SplitDirection(tp)	((TileType)((tp)->ti_body) & TT_DIRECTION ? 1 : 0)
-#define SplitSide(tp)		((TileType)((tp)->ti_body) & TT_SIDE ? 1 : 0)
-#define IsSplit(tp)		((TileType)((tp)->ti_body) & TT_DIAGONAL ? TRUE : FALSE)
+#define TiGetType(tp)		((TileType)(spointertype)((tp)->ti_body) & TT_LEFTMASK)
+#define TiGetTypeExact(tp)	((TileType)(spointertype) (tp)->ti_body)
+#define SplitDirection(tp)	((TileType)(spointertype)((tp)->ti_body) & TT_DIRECTION ? 1 : 0)
+#define SplitSide(tp)		((TileType)(spointertype)((tp)->ti_body) & TT_SIDE ? 1 : 0)
+#define IsSplit(tp)		((TileType)(spointertype)((tp)->ti_body) & TT_DIAGONAL ? TRUE : FALSE)
    
-#define SplitLeftType(tp)	((TileType)((tp)->ti_body) & TT_LEFTMASK)
-#define SplitRightType(tp)	(((TileType)((tp)->ti_body) & TT_RIGHTMASK) >> 14)
-#define SplitTopType(tp)	(((TileType)((tp)->ti_body) & TT_DIRECTION) ? \
+#define SplitLeftType(tp)	((TileType)(spointertype)((tp)->ti_body) & TT_LEFTMASK)
+#define SplitRightType(tp)	(((TileType)(spointertype)((tp)->ti_body) & TT_RIGHTMASK) >> 14)
+#define SplitTopType(tp)	(((TileType)(spointertype)((tp)->ti_body) & TT_DIRECTION) ? \
 					SplitRightType(tp) : SplitLeftType(tp))
-#define SplitBottomType(tp)	(((TileType)((tp)->ti_body) & TT_DIRECTION) ? \
+#define SplitBottomType(tp)	(((TileType)(spointertype)((tp)->ti_body) & TT_DIRECTION) ? \
 					SplitLeftType(tp) : SplitRightType(tp))
 
 #define TiGetLeftType(tp)	SplitLeftType(tp)
@@ -214,7 +214,7 @@ extern Tile *TiSrPoint();
 #define TiGetBottomType(tp)	((IsSplit(tp)) ? SplitBottomType(tp) : TiGetType(tp))
  
 #else
-#define	TiGetType(tp)		((TileType) (tp)->ti_body)
+#define	TiGetType(tp)		((TileType)(spointertype) (tp)->ti_body)
 #define TiGetTypeExact(tp)	TiGetType(tp)
 #define TiGetLeftType(tp)	TiGetType(tp)
 #define TiGetRightType(tp)	TiGetType(tp)
@@ -223,9 +223,10 @@ extern Tile *TiSrPoint();
 #endif
 
 #define	TiGetBody(tp)		((tp)->ti_body)
-#define	TiSetBody(tp, b)	((tp)->ti_body = (ClientData) (b))
+/* See diagnostic subroutine version in tile.c */
+#define	TiSetBody(tp, b)	((tp)->ti_body = (ClientData)(pointertype) (b))
 #define	TiGetClient(tp)		((tp)->ti_client)
-#define	TiSetClient(tp,b)	((tp)->ti_client = (ClientData) (b))
+#define	TiSetClient(tp,b)	((tp)->ti_client = (ClientData)(pointertype) (b))
 
 Tile *TiAlloc();
 void TiFree();
