@@ -19,16 +19,12 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/utils/main.c,v 1.2 2005/05/31 21:41:24 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/utils/main.c,v 1.2 2006/07/28 15:57:57 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifndef SYSV
-#include <strings.h>
-#else
 #include <string.h>
-#endif
 
 #include <sys/types.h>
 #include <sys/times.h>
@@ -561,14 +557,8 @@ mainInitAfterArgs()
     TechAddClient("mzrouter", nullProc,nullProc,nullProc,
 			sec_types|sec_planes, &sec_mzrouter, TRUE);
 #endif
-    TechAddClient("drc", DRCTechInit, DRCTechAddRule, DRCTechFinal,
+    TechAddClient("drc", DRCTechStyleInit, DRCTechLine, DRCTechFinal,
 			sec_types|sec_planes, &sec_drc, FALSE);
-    TechAddClient("drc", PlowDRCInit, PlowDRCLine, PlowDRCFinal,
-			(SectionID) 0, (SectionID *) 0, FALSE);
-#ifdef ROUTE_MODULE
-    TechAddClient("drc", MZDRCInit, MZDRCLine, nullProc,
-			sec_types|sec_planes, (SectionID *) 0, TRUE);
-#endif
 
 #ifdef LEF_MODULE
     TechAddClient("lef", LefTechInit, LefTechLine, nullProc,
@@ -603,9 +593,6 @@ mainInitAfterArgs()
 			sec_types, &sec_plot, TRUE);
 #endif
 
-    /* Pre-Load initialization of the DRC module */
-    DRCPreInit();
-
     /* Load the technology file */
 
     if (!TechLoad(TechDefault, 0))
@@ -678,9 +665,12 @@ mainInitAfterArgs()
      * NOTE the mzrouter must be initialized prior to the irouter
      * so that default parameters will be completely setup
      */
-    IRInit();
+    IRDebugInit();
+    IRAfterTech();
 #endif
 
+    PlowAfterTech();	/* Copies DRC rule information into plow database */
+
 	/* Initialize the Sim Module (the part of it which involves (i)rsim) */
 #if !defined(NO_SIM_MODULE) && defined(RSIM_MODULE)
     SimInit();
